@@ -17,7 +17,6 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #
 
-
 # GUICommonWindows.py - functions to open common
 # windows in lower part of the screen
 ###################################################
@@ -27,8 +26,18 @@ from GUIDefines import *
 from ie_stats import *
 from ie_modal import *
 from ie_action import *
-from GUICommon import *
-from LUCommon import *
+import GUICommon
+import LUCommon
+import InventoryCommon
+
+# needed for all the Open*Window callbacks in the OptionsWindow
+import GUIJRNL
+import GUIMA
+import GUIMG
+import GUIINV
+import GUIOPT
+import GUIPR
+import GUIREC
 
 FRAME_PC_SELECTED = 0
 FRAME_PC_TARGET   = 1
@@ -61,7 +70,7 @@ def SetupMenuWindowControls (Window, Gears, ReturnToGame):
 	#Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetSprites ("GUILSOP", 0,0,1,20,0)
 	Button.SetVarAssoc ("SelectedWindow", 1)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenMapWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIMA.OpenMapWindow)
 
 	# Journal
 	Button = Window.GetControl (2)
@@ -69,7 +78,7 @@ def SetupMenuWindowControls (Window, Gears, ReturnToGame):
 	#Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetSprites ("GUILSOP", 0,4,5,22,4)
 	Button.SetVarAssoc ("SelectedWindow", 2)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenJournalWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIJRNL.OpenJournalWindow)
 
 	# Inventory
 	Button = Window.GetControl (3)
@@ -77,7 +86,7 @@ def SetupMenuWindowControls (Window, Gears, ReturnToGame):
 	#Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetSprites ("GUILSOP", 0,2,3,21,2)
 	Button.SetVarAssoc ("SelectedWindow", 3)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenInventoryWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIINV.OpenInventoryWindow)
 
 	# Records
 	Button = Window.GetControl (4)
@@ -85,7 +94,7 @@ def SetupMenuWindowControls (Window, Gears, ReturnToGame):
 	#Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetSprites ("GUILSOP", 0,6,7,23,6)
 	Button.SetVarAssoc ("SelectedWindow", 4)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenRecordsWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIREC.OpenRecordsWindow)
 
 	# Mage
 	Button = Window.GetControl (5)
@@ -93,7 +102,7 @@ def SetupMenuWindowControls (Window, Gears, ReturnToGame):
 	#Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetSprites ("GUILSOP", 0,8,9,24,8)
 	Button.SetVarAssoc ("SelectedWindow", 5)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenMageWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIMG.OpenMageWindow)
 
 	# Priest
 	Button = Window.GetControl (6)
@@ -101,7 +110,7 @@ def SetupMenuWindowControls (Window, Gears, ReturnToGame):
 	#Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetSprites ("GUILSOP", 0,10,11,25,10)
 	Button.SetVarAssoc ("SelectedWindow", 6)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenPriestWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIPR.OpenPriestWindow)
 
 	# Options
 	Button = Window.GetControl (7)
@@ -109,12 +118,12 @@ def SetupMenuWindowControls (Window, Gears, ReturnToGame):
 	#Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetSprites ("GUILSOP", 0,12,13,26,12)
 	Button.SetVarAssoc ("SelectedWindow", 7)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenOptionsWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIOPT.OpenOptionsWindow)
 
 	# Party mgmt
 	Button = Window.GetControl (8)
 	Button.SetTooltip (16312)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenPartyWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None) #TODO: OpenPartyWindow
 	
 	#gears
 	if Gears:
@@ -124,8 +133,8 @@ def SetupMenuWindowControls (Window, Gears, ReturnToGame):
 		Button.SetAnimation ("CGEAR")
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 		Button.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_ANIMATED|IE_GUI_BUTTON_NORMAL, OP_SET)
-		Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, "GearsClicked")
-		SetGamedaysAndHourToken()
+		Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, GUICommon.GearsClicked)
+		GUICommon.SetGamedaysAndHourToken()
 		Button.SetTooltip(16041)
 
 	MarkMenuButton (Window)
@@ -167,14 +176,18 @@ def EmptyControls ():
 		Button = Window.GetControl (i)
 		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 		Button.SetPicture ("")
+		Button.SetText ("")
+		Button.SetActionIcon (globals(), -1)
 	return
 
 def SelectFormationPreset ():
+	"""Choose the default formation."""
 	GemRB.GameSetFormation (GemRB.GetVar ("Value"), GemRB.GetVar ("Formation") )
 	GroupControls ()
 	return
 
 def SetupFormation ():
+	"""Opens the formation selection section."""
 	global ActionsWindow
 
 	Window = ActionsWindow
@@ -184,28 +197,30 @@ def SetupFormation ():
 		Button.SetSprites ("GUIBTBUT",0,0,1,2,3)
 		Button.SetBAM ("FORM%x"%i,0,0,-1)
 		Button.SetVarAssoc ("Value", i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SelectFormationPreset")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectFormationPreset)
 	return
 
 def GroupControls ():
+	"""Sections that control group actions."""
+
 	global ActionsWindow
 
 	GemRB.SetVar ("ActionLevel", 0)
 	Window = ActionsWindow
 	Button = Window.GetControl (0)
-	Button.SetActionIcon (7)
+	Button.SetActionIcon (globals(), 14)
 	Button = Window.GetControl (1)
-	Button.SetActionIcon (15)
+	Button.SetActionIcon (globals(), 7)
 	Button = Window.GetControl (2)
-	Button.SetActionIcon (21)
+	Button.SetActionIcon (globals(), 15)
 	Button = Window.GetControl (3)
-	Button.SetActionIcon (-1)
+	Button.SetActionIcon (globals(), 21)
 	Button = Window.GetControl (4)
-	Button.SetActionIcon (-1)
+	Button.SetActionIcon (globals(), -1)
 	Button = Window.GetControl (5)
-	Button.SetActionIcon (-1)
+	Button.SetActionIcon (globals(), -1)
 	Button = Window.GetControl (6)
-	Button.SetActionIcon (-1)
+	Button.SetActionIcon (globals(), -1)
 	GemRB.SetVar ("Formation", GemRB.GameGetFormation ())
 	for i in range (5):
 		Button = Window.GetControl (7+i)
@@ -215,8 +230,8 @@ def GroupControls ():
 		Button.SetSprites ("GUIBTBUT",0,0,1,2,3)
 		Button.SetBAM ("FORM%x"%idx,0,0,-1)
 		Button.SetVarAssoc ("Formation", i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SelectFormation")
-		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, "SetupFormation")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommon.SelectFormation)
+		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, SetupFormation)
 		str = GemRB.GetString (4935)
 		Button.SetTooltip ("F%d - %s"%(8+i,str) )
 	return
@@ -233,6 +248,36 @@ def OpenActionsWindowControls (Window):
 	UpdateActionsWindow ()
 	return
 
+def SelectItemAbility():
+	pc = GemRB.GameGetFirstSelectedActor ()
+	slot = GemRB.GetVar ("Slot")
+	ability = GemRB.GetVar ("Ability")
+	GemRB.SetupQuickSlot (pc, 0, slot, ability, 1)
+	GemRB.SetVar ("ActionLevel", 0)
+	return
+
+def SetupItemAbilities(pc, slot):
+	Window = ActionsWindow
+
+	slot_item = GemRB.GetSlotItem(pc, slot, 1)
+	item = GemRB.GetItem (slot_item["ItemResRef"])
+	Tips = item["Tooltips"]
+
+	for i in range (12):
+		Button = Window.GetControl (i)
+		Button.SetPicture ("")
+		if i<len(Tips):
+			Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON|IE_GUI_BUTTON_NORMAL, OP_SET)
+			Button.SetSprites ("GUIBTBUT",0,0,1,2,3)
+			Button.SetItemIcon (slot_item['ItemResRef'], i+6)
+			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectItemAbility)
+			Button.SetVarAssoc ("Ability", i)
+	
+			Button.SetTooltip ("F%d - %s"%(i+1,GemRB.GetString(Tips[i])) )
+		else:
+			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
+	return
+
 def UpdateActionsWindow ():
 	global ActionsWindow
 	global level, TopIndex
@@ -243,14 +288,7 @@ def UpdateActionsWindow ():
 	if ActionsWindow == None:
 		return
 
-	pc = 0
-	for i in range (PARTY_SIZE):
-		if GemRB.GameIsPCSelected (i+1):
-			if pc == 0:
-				pc = i+1
-			else:
-				pc = -1
-				break
+	Selected = GemRB.GetSelectedSize()
 
 	#setting up the disabled button overlay (using the second border slot)
 	for i in range (12):
@@ -260,38 +298,47 @@ def UpdateActionsWindow ():
 		Button.SetFont ("NUMBER")
 		Button.SetText ("")
 
-	if pc == 0:
+	if Selected == 0:
 		EmptyControls ()
 		return
-	if pc == -1:
+	if Selected > 1:
 		GroupControls ()
 		return
-	#this is based on class
+
+	#we are sure there is only one actor selected
+	pc = GemRB.GameGetFirstSelectedActor ()
 
 	level = GemRB.GetVar ("ActionLevel")
 	TopIndex = GemRB.GetVar ("TopIndex")
 	if level == 0:
-		ActionsWindow.SetupControls (pc)
+		#this is based on class
+		ActionsWindow.SetupControls (globals(), pc, 0, 1)
 	elif level == 1:
-		ActionsWindow.SetupEquipmentIcons(pc, TopIndex)
+		ActionsWindow.SetupEquipmentIcons(globals(), pc, TopIndex, 0, 1)
 	elif level == 2: #spells
 		GemRB.SetVar ("Type", 3)
-		ActionsWindow.SetupSpellIcons(pc, 3, TopIndex)
+		ActionsWindow.SetupSpellIcons(globals(), pc, 3, TopIndex, 0, 1)
 	elif level == 3: #innates
 		GemRB.SetVar ("Type", 4)
-		ActionsWindow.SetupSpellIcons(pc, 4, TopIndex)
+		ActionsWindow.SetupSpellIcons(globals(), pc, 4, TopIndex, 0, 1)
+	elif level == 4: #quick weapon/item ability selection
+		SetupItemAbilities(pc, GemRB.GetVar("Slot") )
 	return
 
 def ActionQWeaponPressed (which):
-	pc = GemRB.GameGetFirstSelectedPC ()
+	"""Selects the given quickslot weapon if possible."""
 
-	if GemRB.GetEquippedQuickSlot (pc,1)==which and GemRB.GameControlGetTargetMode() != TARGET_MODE_ATTACK:
-		GemRB.GameControlSetTargetMode (TARGET_MODE_ATTACK)
+	pc = GemRB.GameGetFirstSelectedActor ()
+	qs = GemRB.GetEquippedQuickSlot (pc, 1, 1)
+
+	#38 is the magic slot
+	if ((qs==which) or (qs==38)) and GemRB.GameControlGetTargetMode() != TARGET_MODE_ATTACK:
+		GemRB.GameControlSetTargetMode (TARGET_MODE_ATTACK, GA_NO_DEAD|GA_NO_SELF|GA_NO_HIDDEN)
 	else:
 		GemRB.GameControlSetTargetMode (TARGET_MODE_NONE)
-		GemRB.SetEquippedQuickSlot (pc, which)
+		GemRB.SetEquippedQuickSlot (pc, which, -1, 1)
 
-	ActionsWindow.SetupControls (pc)
+	ActionsWindow.SetupControls (globals(), pc, 0, 1)
 	UpdateActionsWindow ()
 	return
 
@@ -309,6 +356,10 @@ def ActionQWeapon4Pressed ():
 
 #no check needed because the button wouldn't be drawn if illegal
 def ActionLeftPressed ():
+	"""Scrolls the actions window left.
+
+	Used primarily for spell selection."""
+
 	TopIndex = GemRB.GetVar ("TopIndex")
 	if TopIndex>10:
 		TopIndex -= 10
@@ -320,10 +371,16 @@ def ActionLeftPressed ():
 
 #no check needed because the button wouldn't be drawn if illegal
 def ActionRightPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
+	"""Scrolls the action window right.
+
+	Used primarily for spell selection."""
+
+	pc = GemRB.GameGetFirstSelectedActor ()
 	TopIndex = GemRB.GetVar ("TopIndex")
 	Type = GemRB.GetVar ("Type")
-	Max = GemRB.GetMemorizedSpellsCount(pc, Type)
+	#Type is a bitfield if there is no level given
+	#This is to make sure cleric/mages get all spells listed
+	Max = GemRB.GetMemorizedSpellsCount(pc, Type, -1, 1)
 	TopIndex += 10
 	if TopIndex > Max - 10:
 		if Max>10:
@@ -337,28 +394,31 @@ def ActionRightPressed ():
 
 def ActionBardSongPressed ():
 	"""Toggles the battle song."""
-	pc = GemRB.GameGetFirstSelectedPC ()
-	GemRB.SetModalState (pc, MS_BATTLESONG)
+	pc = GemRB.GameGetFirstSelectedActor ()
+	GemRB.SetModalState (pc, MS_BATTLESONG, 1)
 	GemRB.PlaySound ("act_01")
 	UpdateActionsWindow ()
 	return
 
 def ActionSearchPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
-	GemRB.SetModalState (pc, MS_DETECTTRAPS)
+	"""Toggles detect traps."""
+	pc = GemRB.GameGetFirstSelectedActor ()
+	GemRB.SetModalState (pc, MS_DETECTTRAPS, 1)
 	UpdateActionsWindow ()
 	return
 
 def ActionStealthPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
-	GemRB.SetModalState (pc, MS_STEALTH)
+	"""Toggles stealth."""
+	pc = GemRB.GameGetFirstSelectedActor ()
+	GemRB.SetModalState (pc, MS_STEALTH, 1)
 	GemRB.PlaySound ("act_07")
 	UpdateActionsWindow ()
 	return
-	
+
 def ActionTurnPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
-	GemRB.SetModalState (pc, MS_TURNUNDEAD)
+	"""Toggles turn undead."""
+	pc = GemRB.GameGetFirstSelectedActor ()
+	GemRB.SetModalState (pc, MS_TURNUNDEAD, 1)
 	GemRB.PlaySound ("act_06")
 	UpdateActionsWindow ()
 	return
@@ -370,35 +430,101 @@ def ActionUseItemPressed ():
 	return
 
 def ActionCastPressed ():
+	"""Opens the spell choice scrollbar."""
 	GemRB.SetVar ("TopIndex", 0)
 	GemRB.SetVar ("ActionLevel", 2)
 	UpdateActionsWindow ()
 	return
 
+def ActionQItemPressed (action):
+	"""Uses the given quick item."""
+	pc = GemRB.GameGetFirstSelectedActor ()
+	#quick slot
+	GemRB.UseItem (pc, -2, action, -1, 1)
+	return
+
+def ActionQItem1Pressed ():
+	ActionQItemPressed (ACT_QSLOT1)
+	return
+
+def ActionQItem2Pressed ():
+	ActionQItemPressed (ACT_QSLOT2)
+	return
+
+def ActionQItem3Pressed ():
+	ActionQItemPressed (ACT_QSLOT3)
+	return
+
+def ActionQItem4Pressed ():
+	ActionQItemPressed (ACT_QSLOT4)
+	return
+
+def ActionQItem5Pressed ():
+	ActionQItemPressed (ACT_QSLOT5)
+	return
+
+def ActionQItemRightPressed (action):
+	"""Selects the used ability of the quick item."""
+	pc = GemRB.GameGetFirstSelectedActor ()
+	GemRB.SetVar ("Slot", action)
+	GemRB.SetVar ("ActionLevel", 4)
+	UpdateActionsWindow ()
+	return
+
+def ActionQItem1RightPressed ():
+	ActionQItemRightPressed (19)
+
+def ActionQItem2RightPressed ():
+	ActionQItemRightPressed (20)
+
+def ActionQItem3RightPressed ():
+	ActionQItemRightPressed (21)
+
+def ActionQItem4RightPressed ():
+	ActionQItemRightPressed (22)
+
+def ActionQItem5RightPressed ():
+	ActionQItemRightPressed (23)
+
+def ActionQWeapon1RightPressed ():
+	ActionQItemRightPressed (10)
+
+def ActionQWeapon2RightPressed ():
+	ActionQItemRightPressed (11)
+
+def ActionQWeapon3RightPressed ():
+	ActionQItemRightPressed (12)
+
+def ActionQWeapon4RightPressed ():
+	ActionQItemRightPressed (13)
+
 def ActionInnatePressed ():
+	"""Opens the innate spell scrollbar."""
 	GemRB.SetVar ("TopIndex", 0)
 	GemRB.SetVar ("ActionLevel", 3)
 	UpdateActionsWindow ()
 	return
 
 def SpellPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
+	"""Prepares a spell to be cast."""
+
+	pc = GemRB.GameGetFirstSelectedActor ()
 
 	GemRB.GameControlSetTargetMode (TARGET_MODE_CAST)
 	Spell = GemRB.GetVar ("Spell")
 	Type = GemRB.GetVar ("Type")
-	GemRB.SpellCast (pc, Type, Spell)
+	GemRB.SpellCast (pc, Type, Spell, 1)
 	GemRB.SetVar ("ActionLevel", 0)
 	UpdateActionsWindow ()
 	return
 
 def EquipmentPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
+	pc = GemRB.GameGetFirstSelectedActor ()
 
 	GemRB.GameControlSetTargetMode (TARGET_MODE_CAST)
 	Item = GemRB.GetVar ("Equipment")
 	#equipment index
-	GemRB.UseItem (pc, -1, Item)
+	GemRB.UseItem (pc, -1, Item, -1, 1)
 	GemRB.SetVar ("ActionLevel", 0)
 	UpdateActionsWindow ()
 	return
@@ -406,12 +532,15 @@ def EquipmentPressed ():
 SelectionChangeHandler = None
 
 def SetSelectionChangeHandler (handler):
+	"""Updates the selection handler."""
+
 	global SelectionChangeHandler
 
 	# Switching from walking to non-walking environment:
 	# set the first selected PC in walking env as a selected
 	# in nonwalking env
-	if (not SelectionChangeHandler) and handler:
+	#if (not SelectionChangeHandler) and handler:
+	if (not SelectionChangeHandler) and handler and (not GUICommon.NextWindowFn):
 		sel = GemRB.GameGetFirstSelectedPC ()
 		if not sel:
 			sel = 1
@@ -421,10 +550,12 @@ def SetSelectionChangeHandler (handler):
 
 	# redraw selection on change main selection | single selection
 	SelectionChanged ()
+	return
 
 def RunSelectionChangeHandler ():
 	if SelectionChangeHandler:
 		SelectionChangeHandler ()
+	return
 
 def OpenPortraitWindow (needcontrols):
 	global PortraitWindow
@@ -433,7 +564,7 @@ def OpenPortraitWindow (needcontrols):
 
 	if needcontrols:
 		#Button=Window.GetControl (8)
-		#Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "MinimizePortraits")
+		#Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, MinimizePortraits)
 
 		# AI
 		Button = Window.GetControl (6)
@@ -441,7 +572,7 @@ def OpenPortraitWindow (needcontrols):
 		Button.SetFlags (IE_GUI_BUTTON_CHECKBOX, OP_OR)
 		#this control is crippled
 		Button.SetSprites ("GUIBTACT", 0, 46, 47, 48, 49)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "AIPress")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, AIPress)
 		Button.SetVarAssoc ("", GSFlags)
 		if GSFlags:
 			Button.SetTooltip (15917)
@@ -451,12 +582,12 @@ def OpenPortraitWindow (needcontrols):
 		#Select All
 		Button = Window.GetControl (7)
 		Button.SetTooltip (10485)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SelectAllOnPress")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommon.SelectAllOnPress)
 	else:
 		#Rest
 		Button = Window.GetControl (6)
 		Button.SetTooltip (11942)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "RestPress")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommon.RestPress)
 
 	for i in range (PARTY_SIZE):
 		Button = Window.GetControl (i)
@@ -464,17 +595,17 @@ def OpenPortraitWindow (needcontrols):
 		Button.SetVarAssoc ("PressedPortrait", i+1)
 
 		if (needcontrols):
-			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, "OpenInventoryWindowClick")
+			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, GUIINV.OpenInventoryWindowClick)
 		else:
-			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, "PortraitButtonOnPress")
+			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, PortraitButtonOnPress)
 
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "PortraitButtonOnPress")
-		Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, "PortraitButtonOnShiftPress")
-		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, "OnDropItemToPC")
-		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP_PORTRAIT, "OnDropPortraitToPC")
-		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG, "PortraitButtonOnDrag")
-		Button.SetEvent (IE_GUI_MOUSE_ENTER_BUTTON, "PortraitButtonOnMouseEnter")
-		Button.SetEvent (IE_GUI_MOUSE_LEAVE_BUTTON, "PortraitButtonOnMouseLeave")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitButtonOnPress)
+		Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, PortraitButtonOnShiftPress)
+		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, InventoryCommon.OnDropItemToPC)
+		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP_PORTRAIT, OnDropPortraitToPC)
+		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG, PortraitButtonOnDrag)
+		Button.SetEvent (IE_GUI_MOUSE_ENTER_BUTTON, PortraitButtonOnMouseEnter)
+		Button.SetEvent (IE_GUI_MOUSE_LEAVE_BUTTON, PortraitButtonOnMouseLeave)
 
 		Button.SetBorder (FRAME_PC_SELECTED, 1, 1, 2, 2, 0, 255, 0, 255)
 		Button.SetBorder (FRAME_PC_TARGET, 3, 3, 4, 4, 255, 255, 0, 255)
@@ -497,6 +628,12 @@ def UpdatePortraitWindow ():
 		if Inventory and pc != portid+1:
 			pic = None
 
+		if pic and GemRB.GetPlayerStat(portid+1, IE_STATE_ID) & STATE_DEAD:
+			import GUISTORE
+			# dead pcs are hidden in all stores but temples
+			if GUISTORE.StoreWindow and not GUISTORE.StoreHealWindow:
+				pic = None
+
 		if not pic:
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
@@ -504,23 +641,13 @@ def UpdatePortraitWindow ():
 			Button.SetTooltip ("")
 			continue
 
-		Button.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_ALIGN_BOTTOM|IE_GUI_BUTTON_ALIGN_LEFT|IE_GUI_BUTTON_HORIZONTAL|IE_GUI_BUTTON_DRAGGABLE, OP_SET)
-		Button.SetState (IE_GUI_BUTTON_ENABLED)
+		Button.SetFlags (IE_GUI_BUTTON_PICTURE| IE_GUI_BUTTON_HORIZONTAL| \
+				IE_GUI_BUTTON_ALIGN_LEFT| IE_GUI_BUTTON_ALIGN_TOP| \
+				IE_GUI_BUTTON_DRAGGABLE|IE_GUI_BUTTON_MULTILINE, OP_SET)
+
+		Button.SetState (IE_GUI_BUTTON_LOCKED)
 		Button.SetPicture (pic, "NOPORTSM")
-		hp = GemRB.GetPlayerStat (portid+1, IE_HITPOINTS)
-		hp_max = GemRB.GetPlayerStat (portid+1, IE_MAXHITPOINTS)
-		state = GemRB.GetPlayerStat (portid+1, IE_STATE_ID)
-
-		if (hp_max<1):
-			ratio = 0.0
-		else:
-			ratio = (hp+0.0) / hp_max
-
-		if hp<1 or (state & STATE_DEAD):
-			Button.SetOverlay (ratio, 64,64,64,200, 64,64,64,200)
-		else:
-			Button.SetOverlay (ratio, 255,0,0,200, 128,0,0,200)
-		Button.SetTooltip (GemRB.GetPlayerName (portid+1, 1) + "\n%d/%d" %(hp, hp_max))
+		GUICommon.SetupDamageInfo (portid+1, Button)
 
 		#add effects on the portrait
 		effects = GemRB.GetPlayerStates (portid+1)
@@ -532,8 +659,9 @@ def UpdatePortraitWindow ():
 			states = "\n" + states
 		states = "\n" + states
 
+		# character - 1 == bam cycle
 		# blank space
-		flag = chr(38)
+		flag = blank = chr(33)
 
 		# shopping icon
 		if pc==portid+1:
@@ -543,10 +671,10 @@ def UpdatePortraitWindow ():
 		if GemRB.GameGetSelectedPCSingle(1)==portid+1:
 			flag = chr(37)
 
-		if CanLevelUp (portid+1):
-			states = flag+chr(238)+chr(39) + states
+		if LUCommon.CanLevelUp (portid+1):
+			states = flag+blank+chr(255) + states
 		else:
-			states = flag+chr(238)+chr(238) + states
+			states = flag+blank+blank + states
 		Button.SetText(states)
 	return
 
@@ -559,8 +687,10 @@ def PortraitButtonOnDrag ():
 	return
 
 def PortraitButtonOnPress ():
+	"""Selects the portrait individually."""
+
 	i = GemRB.GetVar ("PressedPortrait")
-	
+
 	if not i:
 		return
 
@@ -579,6 +709,8 @@ def PortraitButtonOnPress ():
 	return
 
 def PortraitButtonOnShiftPress ():
+	"""Handles selecting multiple portaits with shift."""
+
 	i = GemRB.GetVar ("PressedPortrait")
 
 	if not i:
@@ -594,10 +726,14 @@ def PortraitButtonOnShiftPress ():
 		RunSelectionChangeHandler ()
 	return
 
-# Run by Game class when selection was changed
 def SelectionChanged ():
+	"""Ran by the Game class when a PC selection is changed."""
+
 	global PortraitWindow
 
+	if not PortraitWindow:
+		return
+
 	GemRB.SetVar ("ActionLevel", 0)
 	if (not SelectionChangeHandler):
 		UpdateActionsWindow ()
@@ -610,18 +746,22 @@ def SelectionChanged ():
 		for i in range (PARTY_SIZE):
 			Button = PortraitWindow.GetControl (i)
 			Button.EnableBorder (FRAME_PC_SELECTED, i + 1 == sel)
+	import CommonWindow
+	CommonWindow.CloseContainerWindow()
 	return
 
 def PortraitButtonOnMouseEnter ():
 	global DraggedPortrait
 
 	i = GemRB.GetVar ("PressedPortrait")
+
+	GemRB.GameControlSetLastActor( i )
 	if GemRB.IsDraggingItem()==2:
 		if DraggedPortrait != None:
 			GemRB.SwapPCs (DraggedPortrait, i)
 			GemRB.SetVar ("PressedPortrait", DraggedPortrait)
 			DraggedPortrait = i
-			GemRB.SetTimedEvent ("CheckDragging",1)
+			GemRB.SetTimedEvent (CheckDragging, 1)
 		else:
 			OnDropPortraitToPC()
 		return
@@ -658,9 +798,26 @@ def PortraitButtonOnMouseLeave ():
 	Button = PortraitWindow.GetControl (i-1)
 	Button.EnableBorder (FRAME_PC_TARGET, 0)
 	GemRB.SetVar ("PressedPortrait", 0)
-	GemRB.SetTimedEvent ("CheckDragging",1)
+	GemRB.SetTimedEvent (CheckDragging, 1)
 	return
 
+def ActionStopPressed ():
+	for i in GemRB.GetSelectedActors():
+		GemRB.ClearActions (i, 1)
+	return
+
+def ActionTalkPressed ():
+	GemRB.GameControlSetTargetMode (TARGET_MODE_TALK,GA_NO_DEAD|GA_NO_ENEMY|GA_NO_HIDDEN)
+
+def ActionAttackPressed ():
+	GemRB.GameControlSetTargetMode (TARGET_MODE_ATTACK,GA_NO_DEAD|GA_NO_SELF|GA_NO_HIDDEN)
+
+def ActionDefendPressed ():
+	GemRB.GameControlSetTargetMode (TARGET_MODE_DEFEND,GA_NO_SELF|GA_NO_ENEMY|GA_NO_HIDDEN)
+
+def ActionThievingPressed ():
+	GemRB.GameControlSetTargetMode (TARGET_MODE_PICK, GA_NO_DEAD|GA_NO_SELF|GA_NO_ENEMY|GA_NO_HIDDEN)
+
 def OpenWaitForDiscWindow ():
 	global DiscWindow
 
@@ -706,4 +863,4 @@ def OpenWaitForDiscWindow ():
 		DiscWindow.SetVisible (WINDOW_VISIBLE)
 
 def CheckLevelUp(pc):
-	GemRB.SetVar ("CheckLevelUp"+str(pc), CanLevelUp (pc))
+	GemRB.SetVar ("CheckLevelUp"+str(pc), LUCommon.CanLevelUp (pc))
