@@ -22,14 +22,15 @@
 
 ###################################################
 import GemRB
-import GUICommonWindows
+import GUICommon
+import CommonTables
+import LUCommon
+import GUIWORLD
+import DualClass
+import LevelUp
 from GUIDefines import *
 from ie_stats import *
 from ie_restype import *
-from GUICommon import *
-from LUCommon import CanLevelUp, GetNextLevelExp
-from GUICommonWindows import *
-from GUIWORLD import OpenReformPartyWindow
 
 ###################################################
 RecordsWindow = None
@@ -51,6 +52,7 @@ ScriptsTable = None
 ColorTable = None
 ColorIndex = None
 ScriptTextArea = None
+OldVoiceSet = None
 
 # the available sounds
 SoundSequence = [ 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', \
@@ -60,10 +62,12 @@ SoundIndex = 0
 
 ###################################################
 def OpenRecordsWindow ():
+	import GUICommonWindows
+
 	global RecordsWindow, OptionsWindow, PortraitWindow
 	global OldPortraitWindow, OldOptionsWindow
 
-	if CloseOtherWindow (OpenRecordsWindow):
+	if GUICommon.CloseOtherWindow (OpenRecordsWindow):
 		if InformationWindow: OpenInformationWindow ()
 
 		if RecordsWindow:
@@ -75,17 +79,17 @@ def OpenRecordsWindow ():
 
 		RecordsWindow = None
 		GemRB.SetVar ("OtherWindow", -1)
-		GameWindow.SetVisible(WINDOW_VISIBLE)
+		GUICommon.GameWindow.SetVisible(WINDOW_VISIBLE)
 		GemRB.UnhideGUI ()
 		GUICommonWindows.PortraitWindow = OldPortraitWindow
 		OldPortraitWindow = None
 		GUICommonWindows.OptionsWindow = OldOptionsWindow
 		OldOptionsWindow = None
-		SetSelectionChangeHandler (None)
+		GUICommonWindows.SetSelectionChangeHandler (None)
 		return
 
 	GemRB.HideGUI ()
-	GameWindow.SetVisible(WINDOW_INVISIBLE)
+	GUICommon.GameWindow.SetVisible(WINDOW_INVISIBLE)
 
 	GemRB.LoadWindowPack ("GUIREC", 640, 480)
 	RecordsWindow = Window = GemRB.LoadWindow (2)
@@ -93,48 +97,48 @@ def OpenRecordsWindow ():
 	#saving the original portrait window
 	OldOptionsWindow = GUICommonWindows.OptionsWindow
 	OptionsWindow = GemRB.LoadWindow (0)
-	MarkMenuButton (OptionsWindow)
-	SetupMenuWindowControls (OptionsWindow, 0, "OpenRecordsWindow")
+	GUICommonWindows.MarkMenuButton (OptionsWindow)
+	GUICommonWindows.SetupMenuWindowControls (OptionsWindow, 0, OpenRecordsWindow)
 	OptionsWindow.SetFrame ()
 	OldPortraitWindow = GUICommonWindows.PortraitWindow
-	PortraitWindow = OpenPortraitWindow (0)
+	PortraitWindow = GUICommonWindows.OpenPortraitWindow (0)
 
 	# dual class
 	Button = Window.GetControl (0)
 	Button.SetText (7174)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DualClassWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DualClass.DualClassWindow)
 
 	# levelup
 	Button = Window.GetControl (37)
 	Button.SetText (7175)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenLevelUpWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LevelUp.OpenLevelUpWindow)
 
 	# information
 	Button = Window.GetControl (1)
 	Button.SetText (11946)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenInformationWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenInformationWindow)
 
 	# reform party
 	Button = Window.GetControl (51)
 	Button.SetText (16559)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenRecReformPartyWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenRecReformPartyWindow)
 
 	# customize
 	Button = Window.GetControl (50)
 	Button.SetText (10645)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenCustomizeWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenCustomizeWindow)
 
 	# export
 	Button = Window.GetControl (36)
 	Button.SetText (13956)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenExportWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenExportWindow)
 
 	# kit info
 	Button = Window.GetControl (52)
 	Button.SetText (61265)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "KitInfoWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenKitInfoWindow)
 
-	SetSelectionChangeHandler (UpdateRecordsWindow)
+	GUICommonWindows.SetSelectionChangeHandler (UpdateRecordsWindow)
 	UpdateRecordsWindow ()
 	OptionsWindow.SetVisible (WINDOW_VISIBLE)
 	Window.SetVisible (WINDOW_FRONT)
@@ -144,7 +148,7 @@ def OpenRecordsWindow ():
 #original returns to game before continuing...
 def OpenRecReformPartyWindow ():
 	OpenRecordsWindow ()
-	GemRB.SetTimedEvent ("OpenReformPartyWindow", 1)
+	GemRB.SetTimedEvent (GUIWORLD.OpenReformPartyWindow, 1)
 	return
 
 def UpdateRecordsWindow ():
@@ -159,10 +163,10 @@ def UpdateRecordsWindow ():
 
 	#update mage school
 	GemRB.SetVar ("MAGESCHOOL", 0)
-	Kit = GetKitIndex (pc)
-	if Kit and KitListTable.GetValue (Kit, 7) == 1:
+	Kit = GUICommon.GetKitIndex (pc)
+	if Kit and CommonTables.KitList.GetValue (Kit, 7) == 1:
 		MageTable = GemRB.LoadTable ("magesch")
-		GemRB.SetVar ("MAGESCHOOL", MageTable.FindValue (3, KitListTable.GetValue (Kit, 6) ) )
+		GemRB.SetVar ("MAGESCHOOL", MageTable.FindValue (3, CommonTables.KitList.GetValue (Kit, 6) ) )
 
 	# exportable
 	Button = Window.GetControl (36)
@@ -173,14 +177,14 @@ def UpdateRecordsWindow ():
 
 	# dual-classable
 	Button = Window.GetControl (0)
-	if CanDualClass (pc):
+	if GUICommon.CanDualClass (pc):
 		Button.SetState (IE_GUI_BUTTON_DISABLED)
 	else:
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 
 	# levelup
 	Button = Window.GetControl (37)
-	if CanLevelUp (pc):
+	if LUCommon.CanLevelUp (pc):
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 	else:
 		Button.SetState (IE_GUI_BUTTON_DISABLED)
@@ -259,12 +263,12 @@ def UpdateRecordsWindow ():
 	Label.SetTextColor (cchr[0], cchr[1], cchr[2])
 
 	# class
-	ClassTitle = GetActorClassTitle (pc)
+	ClassTitle = GUICommon.GetActorClassTitle (pc)
 	Label = Window.GetControl (0x10000030)
 	Label.SetText (ClassTitle)
 
 	# race
-	text = RaceTable.GetValue (RaceTable.FindValue (3, GemRB.GetPlayerStat (pc, IE_RACE)) , 0)
+	text = CommonTables.Races.GetValue (CommonTables.Races.FindValue (3, GemRB.GetPlayerStat (pc, IE_RACE)) , 0)
 
 	Label = Window.GetControl (0x1000000f)
 	Label.SetText (text)
@@ -313,6 +317,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	str_None = GemRB.GetString (61560)
 
 	GS = lambda s, pc=pc: GemRB.GetPlayerStat (pc, s)
+	GB = lambda s, pc=pc: GemRB.GetPlayerStat (pc, s, 1)
 	GA = lambda s, col, pc=pc: GemRB.GetAbilityBonus (s, col, GS (s) )
 
 	stats = []
@@ -322,16 +327,20 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	# Next Level: <NEXTLEVEL>
 
 	# collecting tokens for stat overview
-	ClassTitle = GetActorClassTitle (pc)
+	ClassTitle = GUICommon.GetActorClassTitle (pc)
 	GemRB.SetToken ("CLASS", ClassTitle)
 	Class = GemRB.GetPlayerStat (pc, IE_CLASS)
-	Class = ClassTable.FindValue (5, Class)
-	Class = ClassTable.GetRowName (Class)
-	Dual = IsDualClassed (pc, 1)
-	Multi = IsMultiClassed (pc, 1)
+	Class = CommonTables.Classes.FindValue (5, Class)
+	Class = CommonTables.Classes.GetRowName (Class)
+	Dual = GUICommon.IsDualClassed (pc, 1)
+	Multi = GUICommon.IsMultiClassed (pc, 1)
 	XP = GemRB.GetPlayerStat (pc, IE_XP)
 	LevelDrain = GS (IE_LEVELDRAIN)
 
+	if GS (IE_STATE_ID) & STATE_DEAD:
+		stats.append ( (11829,1,'c') ) # DEAD
+		stats.append (None)
+
 	if Multi[0] > 1: # we're multiclassed
 		print "\tMulticlassed"
 		Levels = [GemRB.GetPlayerStat (pc, IE_LEVEL), GemRB.GetPlayerStat (pc, IE_LEVEL2), GemRB.GetPlayerStat (pc, IE_LEVEL3)]
@@ -339,17 +348,17 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 		stats.append ( (19721,1,'c') )
 		stats.append (None)
 		for i in range (Multi[0]):
-			ClassIndex = ClassTable.FindValue (5, Multi[i+1])
-			ClassTitle = GemRB.GetString (ClassTable.GetValue (ClassIndex, 2))
+			ClassIndex = CommonTables.Classes.FindValue (5, Multi[i+1])
+			ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (ClassIndex, 2))
 			GemRB.SetToken ("CLASS", ClassTitle)
-			Class = ClassTable.GetRowName (ClassIndex)
+			Class = CommonTables.Classes.GetRowName (ClassIndex)
 			GemRB.SetToken ("LEVEL", str (Levels[i]+LevelDiff[i]-int(LevelDrain/Multi[0])) )
 			GemRB.SetToken ("EXPERIENCE", str (XP/Multi[0]) )
 			if LevelDrain:
 				stats.append ( (GemRB.GetString (19720),1,'d') )
 				stats.append ( (GemRB.GetString (57435),1,'d') ) # LEVEL DRAINED
 			else:
-				GemRB.SetToken ("NEXTLEVEL", GetNextLevelExp (Levels[i]+LevelDiff[i], Class) )
+				GemRB.SetToken ("NEXTLEVEL", LUCommon.GetNextLevelExp (Levels[i]+LevelDiff[i], Class) )
 				stats.append ( (GemRB.GetString (16480),"",'d') )
 			stats.append (None)
 			print "\t\tClass (Level):",Class,"(",Levels[i],")"
@@ -363,45 +372,45 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 
 		# the levels are stored in the class order (eg. FIGHTER_MAGE)
 		# the current active class does not matter!
-		if IsDualSwap (pc):
+		if GUICommon.IsDualSwap (pc):
 			Levels = [Levels[1], Levels[0], Levels[2]]
 
 		Levels[0] += LevelDiff[0]
 
-		ClassTitle = GemRB.GetString (ClassTable.GetValue (Dual[2], 2))
+		ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (Dual[2], 2))
 		GemRB.SetToken ("CLASS", ClassTitle)
 		GemRB.SetToken ("LEVEL", str (Levels[0]-LevelDrain))
-		Class = ClassTable.GetRowName (Dual[2])
+		Class = CommonTables.Classes.GetRowName (Dual[2])
 		XP2 = GemRB.GetPlayerStat (pc, IE_XP)
 		GemRB.SetToken ("EXPERIENCE", str (XP2) )
 		if LevelDrain:
 			stats.append ( (GemRB.GetString (19720),1,'d') )
 			stats.append ( (GemRB.GetString (57435),1,'d') ) # LEVEL DRAINED
 		else:
-			GemRB.SetToken ("NEXTLEVEL", GetNextLevelExp (Levels[0], Class) )
+			GemRB.SetToken ("NEXTLEVEL", LUCommon.GetNextLevelExp (Levels[0], Class) )
 			stats.append ( (GemRB.GetString (16480),"",'d') )
 		stats.append (None)
 
 		# the first class (shown second)
 		if Dual[0] == 1:
-			ClassTitle = GemRB.GetString (KitListTable.GetValue (Dual[1], 2))
+			ClassTitle = GemRB.GetString (CommonTables.KitList.GetValue (Dual[1], 2))
 		elif Dual[0] == 2:
-			ClassTitle = GemRB.GetString (ClassTable.GetValue (Dual[1], 2))
+			ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (Dual[1], 2))
 		GemRB.SetToken ("CLASS", ClassTitle)
 		GemRB.SetToken ("LEVEL", str (Levels[1]) )
 
 		# the xp table contains only classes, so we have to determine the base class for kits
 		if Dual[0] == 2:
-			BaseClass = ClassTable.GetRowName (Dual[1])
+			BaseClass = CommonTables.Classes.GetRowName (Dual[1])
 		else:
-			BaseClass = GetKitIndex (pc)
-			BaseClass = KitListTable.GetValue (BaseClass, 7)
-			BaseClass = ClassTable.FindValue (5, BaseClass)
-			BaseClass = ClassTable.GetRowName (BaseClass)
+			BaseClass = GUICommon.GetKitIndex (pc)
+			BaseClass = CommonTables.KitList.GetValue (BaseClass, 7)
+			BaseClass = CommonTables.Classes.FindValue (5, BaseClass)
+			BaseClass = CommonTables.Classes.GetRowName (BaseClass)
 		# the first class' XP is discarded and set to the minimum level
 		# requirement, so if you don't dual class right after a levelup,
 		# the game would eat some of your XP
-		XP1 = NextLevelTable.GetValue (BaseClass, str (Levels[1]))
+		XP1 = CommonTables.NextLevel.GetValue (BaseClass, str (Levels[1]))
 		GemRB.SetToken ("EXPERIENCE", str (XP1) )
 
 		# inactive until the new class SURPASSES the former
@@ -420,7 +429,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 			stats.append ( (19720,1,'c') )
 			stats.append ( (57435,1,'c') ) # LEVEL DRAINED
 		else:
-			GemRB.SetToken ("NEXTLEVEL", GetNextLevelExp (Level, Class) )
+			GemRB.SetToken ("NEXTLEVEL", LUCommon.GetNextLevelExp (Level, Class) )
 			stats.append ( (16480,1,'c') )
 		stats.append (None)
 		print "\t\tClass (Level):",Class,"(",Level,")"
@@ -474,7 +483,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 
 	#this hack only displays LOH if we know the spell
 	#TODO: the core should just not set LOH if the paladin can't learn it
-	if (HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, "SPCL211") >= 0):
+	if (GUICommon.HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, "SPCL211") >= 0):
 		stats.append ( (12127, GS (IE_LAYONHANDSAMOUNT), '') )
 	#script
 	aiscript = GemRB.GetPlayerScript (pc )
@@ -484,15 +493,15 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	# 17379 Saving throws
 	stats.append (17379)
 	# 17380 Paralyze/Poison/Death
-	stats.append ( (17380, GS (IE_SAVEVSDEATH), '0') )
+	stats.append ( (17380, IE_SAVEVSDEATH, 's') )
 	# 17381 Rod/Staff/Wand
-	stats.append ( (17381, GS (IE_SAVEVSWANDS), '0') )
+	stats.append ( (17381, IE_SAVEVSWANDS, 's') )
 	# 17382 Petrify/Polymorph
-	stats.append ( (17382, GS (IE_SAVEVSPOLY), '0') )
+	stats.append ( (17382, IE_SAVEVSPOLY, 's') )
 	# 17383 Breath weapon
-	stats.append ( (17383, GS (IE_SAVEVSBREATH), '0') )
+	stats.append ( (17383, IE_SAVEVSBREATH, 's') )
 	# 17384 Spells
-	stats.append ( (17384, GS (IE_SAVEVSSPELL), '0') )
+	stats.append ( (17384, IE_SAVEVSSPELL, 's') )
 	stats.append (None)
 
 	# 9466 Weapon proficiencies
@@ -509,13 +518,13 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	# 11766 AC bonuses
 	stats.append (11766)
 	# 11770 AC vs. Crushing
-	stats.append ((11770, GS (IE_ACCRUSHINGMOD), ''))
+	stats.append ((11770, GS (IE_ACCRUSHINGMOD), 'p'))
 	# 11767 AC vs. Missile
-	stats.append ((11767, GS (IE_ACMISSILEMOD), ''))
+	stats.append ((11767, GS (IE_ACMISSILEMOD), 'p'))
 	# 11769 AC vs. Piercing
-	stats.append ((11769, GS (IE_ACPIERCINGMOD), ''))
+	stats.append ((11769, GS (IE_ACPIERCINGMOD), 'p'))
 	# 11768 AC vs. Slashing
-	stats.append ((11768, GS (IE_ACSLASHINGMOD), ''))
+	stats.append ((11768, GS (IE_ACSLASHINGMOD), 'p'))
 	stats.append (None)
 
 	# 10315 Ability bonuses
@@ -523,9 +532,9 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	value = GemRB.GetPlayerStat (pc, IE_STR)
 	ex = GemRB.GetPlayerStat (pc, IE_STREXTRA)
 	# 10332 to hit
-	stats.append ( (10332, GemRB.GetAbilityBonus (IE_STR,0,value,ex), '0') )
+	stats.append ( (10332, GemRB.GetAbilityBonus (IE_STR,0,value,ex), 'p') )
 	# 10336 damage
-	stats.append ( (10336, GemRB.GetAbilityBonus (IE_STR,1,value,ex), '0') )
+	stats.append ( (10336, GemRB.GetAbilityBonus (IE_STR,1,value,ex), 'p') )
 	# 10337 open doors (bend bars lift gates)
 	stats.append ( (10337, GemRB.GetAbilityBonus (IE_STR,2,value,ex), '0') )
 	# 10338 weight allowance
@@ -533,11 +542,11 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	# 10339 AC
 	stats.append ( (10339, GA (IE_DEX,2), '0') )
 	# 10340 Missile adjustment
-	stats.append ( (10340, GA (IE_DEX,1), '0') )
+	stats.append ( (10340, GA (IE_DEX,1), 'p') )
 	# 10341 Reaction adjustment
-	stats.append ( (10341, GA (IE_DEX,0), '0') )
+	stats.append ( (10341, GA (IE_DEX,0), 'p') )
 	# 10342 CON HP Bonus/Level
-	stats.append ( (10342, GA (IE_CON,0), '0') )
+	stats.append ( (10342, GA (IE_CON,0), 'p') )
 	# 10343 Chance To Learn spell
 	if GemRB.GetMemorizableSpellsCount (pc, IE_SPELL_TYPE_WIZARD, 0, 0)>0:
 		stats.append ( (10343, GA (IE_INT,0), '%' ) )
@@ -554,7 +563,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 			base = GemRB.GetMemorizableSpellsCount (pc, IE_SPELL_TYPE_PRIEST, level, 0)
 			if base:
 				count = GemRB.GetMemorizableSpellsCount (pc, IE_SPELL_TYPE_PRIEST, level)
-				stats.append ( (GemRB.GetString (10345), count-base, 'b') )
+				stats.append ( (GemRB.GetString (10345), count-base, 'r') )
 		stats.append (None)
 
 	# 32204 Resistances
@@ -609,6 +618,24 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 				continue
 			if type == '+': #pluses
 				res.append ("[capital=0]"+GemRB.GetString (strref) + ' '+ '+' * val)
+			elif type == 'p': #a plus prefix if positive
+				if val > 0:
+					res.append ("[capital=0]" + GemRB.GetString (strref) + ' +' + str (val) )
+				else:
+					res.append ("[capital=0]" + GemRB.GetString (strref) + ' ' + str (val) )
+			elif type == 'r': #a plus prefix if positive, strref is an already resolved string
+				if val > 0:
+					res.append ("[capital=0]" + strref + ' +' + str (val) )
+				else:
+					res.append ("[capital=0]" + strref + ' ' + str (val) )
+			elif type == 's': #both base and (modified) stat, but only if they differ
+				base = GB (val)
+				stat = GS (val)
+				base_str = "[capital=0]" + GemRB.GetString (strref) + ': ' + str(stat)
+				if base == stat:
+					res.append (base_str)
+				else:
+					res.append (base_str + " (" + str(stat-base) + ")")
 			elif type == 'x': #x character before value
 				res.append ("[capital=0]"+GemRB.GetString (strref)+': x' + str (val) )
 			elif type == 'a': #value (portrait icon) + string
@@ -666,12 +693,12 @@ def OpenInformationWindow ():
 	# Biography
 	Button = Window.GetControl (26)
 	Button.SetText (18003)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenBiographyWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenBiographyWindow)
 
 	# Done
 	Button = Window.GetControl (24)
 	Button.SetText (11973)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenInformationWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenInformationWindow)
 
 	TotalPartyExp = 0
 	ChapterPartyExp = 0
@@ -691,7 +718,7 @@ def OpenInformationWindow ():
 	Label = Window.GetControl (0x10000000)
 	Label.SetText (GemRB.GetPlayerName (pc, 1))
 	# class
-	ClassTitle = GetActorClassTitle (pc)
+	ClassTitle = GUICommon.GetActorClassTitle (pc)
 	Label = Window.GetControl (0x10000018)
 	Label.SetText (ClassTitle)
 
@@ -732,15 +759,16 @@ def OpenInformationWindow ():
 	#are they used?
 	Label.SetText (stat['FavouriteWeapon'])
 
-	#total xp
-	Label = Window.GetControl (0x1000000f)
+	#total party xp
+	Label = Window.GetControl (0x10000013)
 	if TotalPartyExp != 0:
 		PartyExp = int ((stat['KillsTotalXP'] * 100) / TotalPartyExp)
 		Label.SetText (str (PartyExp) + '%')
 	else:
 		Label.SetText ("0%")
 
-	Label = Window.GetControl (0x10000013)
+	#chapter party xp
+	Label = Window.GetControl (0x1000000f)
 	if ChapterPartyExp != 0:
 		PartyExp = int ((stat['KillsChapterXP'] * 100) / ChapterPartyExp)
 		Label.SetText (str (PartyExp) + '%')
@@ -748,14 +776,15 @@ def OpenInformationWindow ():
 		Label.SetText ("0%")
 
 	#total xp
-	Label = Window.GetControl (0x10000010)
+	Label = Window.GetControl (0x10000014)
 	if TotalCount != 0:
 		PartyExp = int ((stat['KillsTotalCount'] * 100) / TotalCount)
 		Label.SetText (str (PartyExp) + '%')
 	else:
 		Label.SetText ("0%")
 
-	Label = Window.GetControl (0x10000014)
+	#chapter xp
+	Label = Window.GetControl (0x10000010)
 	if ChapterCount != 0:
 		PartyExp = int ((stat['KillsChapterCount'] * 100) / ChapterCount)
 		Label.SetText (str (PartyExp) + '%')
@@ -795,7 +824,9 @@ def OpenBiographyWindow ():
 	# Done
 	Button = Window.GetControl (2)
 	Button.SetText (11973)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenBiographyWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenBiographyWindow)
+
+	Window.ShowModal (MODAL_SHADOW_GRAY)
 
 	return
 
@@ -820,9 +851,9 @@ def OpenExportWindow ():
 
 	NameField = ExportWindow.GetControl (6)
 
-	ExportDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "ExportDonePress")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "ExportCancelPress")
-	NameField.SetEvent (IE_GUI_EDIT_ON_CHANGE, "ExportEditChanged")
+	ExportDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, ExportDonePress)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, ExportCancelPress)
+	NameField.SetEvent (IE_GUI_EDIT_ON_CHANGE, ExportEditChanged)
 	ExportWindow.ShowModal (MODAL_SHADOW_GRAY)
 	NameField.SetStatus (IE_GUI_CONTROL_FOCUSED)
 	return
@@ -850,7 +881,7 @@ def ExportEditChanged():
 		ExportDoneButton.SetState (IE_GUI_BUTTON_ENABLED)
 	return
 
-def KitInfoWindow ():
+def OpenKitInfoWindow ():
 	global KitInfoWindow
 
 	KitInfoWindow = GemRB.LoadWindow (24)
@@ -858,41 +889,41 @@ def KitInfoWindow ():
 	#back button (setting first, to be less error prone)
 	DoneButton = KitInfoWindow.GetControl (2)
 	DoneButton.SetText (11973)
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "KitDonePress")
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, KitDonePress)
 
 	#kit or class description
 	TextArea = KitInfoWindow.GetControl (0)
 
 	pc = GemRB.GameGetSelectedPCSingle ()
 	Class = GemRB.GetPlayerStat (pc, IE_CLASS)
-	ClassIndex = ClassTable.FindValue (5, Class)
-	Multi = ClassTable.GetValue (ClassIndex, 4)
-	Dual = IsDualClassed (pc, 1)
+	ClassIndex = CommonTables.Classes.FindValue (5, Class)
+	Multi = CommonTables.Classes.GetValue (ClassIndex, 4)
+	Dual = GUICommon.IsDualClassed (pc, 1)
 
 	if Multi and Dual[0] == 0: # true multi class
-		text = ClassTable.GetValue (ClassIndex, 1)
+		text = CommonTables.Classes.GetValue (ClassIndex, 1)
 		TextArea.SetText (text)
 		KitInfoWindow.ShowModal (MODAL_SHADOW_GRAY)
 		return
 
-	KitIndex = GetKitIndex (pc)
+	KitIndex = GUICommon.GetKitIndex (pc)
 
 	if Dual[0]: # dual class
 		# first (previous) kit or class of the dual class
 		if Dual[0] == 1:
-			text = KitListTable.GetValue (Dual[1], 3)
+			text = CommonTables.KitList.GetValue (Dual[1], 3)
 		elif Dual[0] == 2:
-			text = ClassTable.GetValue (Dual[1], 1)
+			text = CommonTables.Classes.GetValue (Dual[1], 1)
 
 		TextArea.SetText (text)
 		TextArea.Append ("\n\n")
-		text = ClassTable.GetValue (Dual[2], 1)
+		text = CommonTables.Classes.GetValue (Dual[2], 1)
 
 	else: # ordinary class or kit
 		if KitIndex:
-			text = KitListTable.GetValue (KitIndex, 3)
+			text = CommonTables.KitList.GetValue (KitIndex, 3)
 		else:
-			text = ClassTable.GetValue (ClassIndex, 1)
+			text = CommonTables.Classes.GetValue (ClassIndex, 1)
 
 	TextArea.Append (text)
 
@@ -953,13 +984,13 @@ def OpenCustomizeWindow ():
 	CancelButton.SetText (13727)
 	CancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
-	AppearanceButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenAppearanceWindow")
-	SoundButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenSoundWindow")
-	ColorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenColorWindow")
-	ScriptButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenScriptWindow")
-	BiographyButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenBiographyEditWindow")
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CustomizeDonePress")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CustomizeCancelPress")
+	AppearanceButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenAppearanceWindow)
+	SoundButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenSoundWindow)
+	ColorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenColorWindow)
+	ScriptButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenScriptWindow)
+	BiographyButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenBiographyEditWindow)
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CustomizeDonePress)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CustomizeCancelPress)
 
 	CustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -1013,11 +1044,11 @@ def OpenAppearanceWindow ():
 	CustomPortraitButton = SubCustomizeWindow.GetControl (5)
 	CustomPortraitButton.SetText (17545)
 
-	LeftButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "PortraitLeftPress")
-	RightButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "PortraitRightPress")
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DonePortraitWindow")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubCustomizeWindow")
-	CustomPortraitButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenCustomPortraitWindow")
+	LeftButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitLeftPress)
+	RightButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitRightPress)
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DonePortraitWindow)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubCustomizeWindow)
+	CustomPortraitButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenCustomPortraitWindow)
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 
 	while True:
@@ -1091,19 +1122,19 @@ def OpenCustomPortraitWindow ():
 	# Portrait List Large
 	PortraitList1 = SubSubCustomizeWindow.GetControl (2)
 	RowCount1 = PortraitList1.GetPortraits (0)
-	PortraitList1.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, "LargeCustomPortrait")
+	PortraitList1.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, LargeCustomPortrait)
 	GemRB.SetVar ("Row1", RowCount1)
 	PortraitList1.SetVarAssoc ("Row1",RowCount1)
 
 	# Portrait List Small
 	PortraitList2 = SubSubCustomizeWindow.GetControl (3)
 	RowCount2 = PortraitList2.GetPortraits (1)
-	PortraitList2.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, "SmallCustomPortrait")
+	PortraitList2.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, SmallCustomPortrait)
 	GemRB.SetVar ("Row2", RowCount2)
 	PortraitList2.SetVarAssoc ("Row2",RowCount2)
 
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DonePortraitCustomizeWindow")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubSubCustomizeWindow")
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DonePortraitCustomizeWindow)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubSubCustomizeWindow)
 
 	SubSubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -1166,16 +1197,19 @@ def OpenSoundWindow ():
 	global SubCustomizeWindow
 	global VoiceList
 	global Gender
+	global OldVoiceSet
 
+	pc = GemRB.GameGetSelectedPCSingle ()
+	OldVoiceSet = GemRB.GetPlayerSound (pc)
 	SubCustomizeWindow = GemRB.LoadWindow (20)
 
 	VoiceList = SubCustomizeWindow.GetControl (5)
 	VoiceList.SetFlags (IE_GUI_TEXTAREA_SELECTABLE)
-	pc = GemRB.GameGetSelectedPCSingle ()
 	Gender = GemRB.GetPlayerStat (pc, IE_SEX, 1)
 
 	VoiceList.SetVarAssoc ("Selected", 0)
 	RowCount=VoiceList.GetCharSounds()
+	VoiceList.SelectText (OldVoiceSet)
 
 	PlayButton = SubCustomizeWindow.GetControl (7)
 	PlayButton.SetText (17318)
@@ -1191,13 +1225,19 @@ def OpenSoundWindow ():
 	CancelButton.SetText (13727)
 	CancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
-	PlayButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "PlaySoundPressed")
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DoneSoundWindow")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubCustomizeWindow")
+	PlayButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, PlaySoundPressed)
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneSoundWindow)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSoundWindow)
 
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
+def CloseSoundWindow ():
+	pc = GemRB.GameGetSelectedPCSingle ()
+	GemRB.SetPlayerSound (pc, OldVoiceSet)
+	CloseSubCustomizeWindow ()
+	return
+
 def DoneSoundWindow ():
 	pc = GemRB.GameGetSelectedPCSingle ()
 	CharSound = VoiceList.QueryText ()
@@ -1210,9 +1250,21 @@ def PlaySoundPressed():
 	global CharSoundWindow, SoundIndex, SoundSequence
 
 	CharSound = VoiceList.QueryText ()
+	tmp = SoundIndex
 	while (not GemRB.HasResource (CharSound + SoundSequence[SoundIndex], RES_WAV)):
 		NextSound()
-	GemRB.PlaySound (CharSound + SoundSequence[SoundIndex], 0, 0, 4)
+		if SoundIndex == tmp:
+			break
+	else:
+		NextSound()
+	GemRB.PlaySound (CharSound + SoundSequence[SoundIndex], 0, 0, 5)
+	return
+
+def NextSound():
+	global SoundIndex, SoundSequence
+	SoundIndex += 1
+	if SoundIndex >= len(SoundSequence):
+		SoundIndex = 0
 	return
 
 def OpenColorWindow ():
@@ -1246,12 +1298,12 @@ def OpenColorWindow ():
 	CancelButton.SetText (13727)
 	CancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
-	HairButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SetHairColor")
-	SkinButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SetSkinColor")
-	MajorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SetMajorColor")
-	MinorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SetMinorColor")
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DoneColorWindow")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubCustomizeWindow")
+	HairButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetHairColor)
+	SkinButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetSkinColor)
+	MajorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetMajorColor)
+	MinorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetMinorColor)
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneColorWindow)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubCustomizeWindow)
 	UpdatePaperDoll ()
 
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
@@ -1275,7 +1327,7 @@ def UpdatePaperDoll ():
 	Color5 = GemRB.GetPlayerStat (pc, IE_LEATHER_COLOR)
 	Color6 = GemRB.GetPlayerStat (pc, IE_ARMOR_COLOR)
 	HairButton.SetBAM ("COLGRAD", 0, 0, HairColor&0xff)
-	PortraitButton.SetPLT (GetActorPaperDoll (pc),
+	PortraitButton.SetPLT (GUICommon.GetActorPaperDoll (pc),
 		Color1, MinorColor, MajorColor, SkinColor, Color5, Color6, HairColor, 0, 0)
 	return
 
@@ -1335,7 +1387,7 @@ def OpenColorPicker ():
 			#Selected = i
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 		Button.SetVarAssoc("Selected",i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DonePress")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DonePress)
 
 	SubSubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -1389,9 +1441,9 @@ def OpenScriptWindow ():
 	CancelButton.SetText (13727)
 	CancelButton.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DoneScriptWindow")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubCustomizeWindow")
-	ScriptTextArea.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, "UpdateScriptSelection")
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneScriptWindow)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubCustomizeWindow)
+	ScriptTextArea.SetEvent (IE_GUI_TEXTAREA_ON_CHANGE, UpdateScriptSelection)
 
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -1461,10 +1513,10 @@ def OpenBiographyEditWindow ():
 	TextArea.SetBufferLength (65535)
 	TextArea.SetText (BioStrRef)
 
-	ClearButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "ClearBiography")
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "DoneBiographyWindow")
-	RevertButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "RevertBiography")
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CloseSubCustomizeWindow")
+	ClearButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, ClearBiography)
+	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneBiographyWindow)
+	RevertButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, RevertBiography)
+	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubCustomizeWindow)
 
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
