@@ -112,12 +112,32 @@ int CharAnimations::GetAnimType() const
 	return AvatarTable[AvatarsRowNum].AnimationType;
 }
 
-char CharAnimations::GetSize() const
+int CharAnimations::GetSize() const
 {
 	if (AvatarsRowNum==~0u) return 0;
 	return AvatarTable[AvatarsRowNum].Size;
 }
 
+int CharAnimations::GetBloodColor() const
+{
+	if(AvatarsRowNum==~0u) return 0;
+	return AvatarTable[AvatarsRowNum].BloodColor;
+}
+
+static ieResRef EmptySound={0};
+
+const ieResRef &CharAnimations::GetWalkSound() const
+{
+	if(AvatarsRowNum==~0u) return EmptySound;
+	return AvatarTable[AvatarsRowNum].WalkSound;
+}
+
+int CharAnimations::GetWalkSoundCount() const
+{
+	if(AvatarsRowNum==~0u) return 0;
+	return AvatarTable[AvatarsRowNum].WalkSoundCount;
+}
+
 int CharAnimations::GetActorPartCount() const
 {
 	if (AvatarsRowNum==~0u) return -1;
@@ -253,6 +273,28 @@ void CharAnimations::SetColors(const ieDword *arg)
 	SetupColors(PAL_HELMET);
 }
 
+void CharAnimations::CheckColorMod()
+{
+	if (!GlobalColorMod.locked) {
+		if (GlobalColorMod.type != RGBModifier::NONE) {
+			GlobalColorMod.type = RGBModifier::NONE;
+			GlobalColorMod.speed = 0;
+		  change[0]=change[1]=change[2]=change[3]=true;
+		}
+	}
+	unsigned int location;
+
+	for (location = 0; location < 32; ++location) {
+		if (!ColorMods[location].phase) {
+		  if (ColorMods[location].type != RGBModifier::NONE) {
+				ColorMods[location].type = RGBModifier::NONE;
+				ColorMods[location].speed = 0;
+		    change[location>>3]=true;
+		  }
+		}
+	}
+}
+
 void CharAnimations::SetupColors(PaletteType type)
 {
 	Palette* pal = palette[(int)type];
@@ -287,11 +329,16 @@ void CharAnimations::SetupColors(PaletteType type)
 		if (GlobalColorMod.type != RGBModifier::NONE) {
 			needmod = true;
 		}
+		//don't drop the palette, it disables rgb pulse effects
+		//also don't bail out, we need to free the modified palette later
+		//so this entire block is needless
+		/*
 		if ((colorcount == 0) && (needmod==false) ) {
-			gamedata->FreePalette(palette[PAL_MAIN], PaletteResRef);
+		  gamedata->FreePalette(palette[PAL_MAIN], PaletteResRef);
 			PaletteResRef[0]=0;
 			return;
 		}
+		*/
 		for (int i = 0; i < colorcount; i++) {
 			core->GetPalette( Colors[i]&255, size,
 				&palette[PAL_MAIN]->col[dest] );
@@ -394,11 +441,6 @@ static int compare_avatars(const void *a, const void *b)
 {
 	unsigned int aa = ((AvatarStruct *)a)->AnimID;
 	unsigned int bb = ((AvatarStruct *)b)->AnimID;
-/*
-	if (aa>bb) return 1;
-	if (aa<bb) return -1;
-	return 0;
-*/
 	return (int) (aa-bb);
 }
 
@@ -431,7 +473,11 @@ void CharAnimations::InitAvatarsTable()
 		else {
 			AvatarTable[i].PaletteType=atoi(Avatars->QueryField(i,AV_USE_PALETTE) );
 		}
-		AvatarTable[i].Size=Avatars->QueryField(i,AV_SIZE)[0];
+		char size = Avatars->QueryField(i,AV_SIZE)[0];
+		if (size == '*') {
+			size = 0;
+		}
+		AvatarTable[i].Size = size;
 
 		AvatarTable[i].WalkScale = 0;
 		AvatarTable[i].RunScale = 0;
@@ -451,6 +497,58 @@ void CharAnimations::InitAvatarsTable()
 		}
 	}
 	qsort(AvatarTable, AvatarsCount, sizeof(AvatarStruct), compare_avatars);
+
+
+	AutoTable blood("bloodclr");
+	if (blood) {
+		int rows = blood->GetRowCount();
+		for(int i=0;i<rows;i++) {
+			unsigned long value = 0;
+			unsigned long rmin = 0;
+			unsigned long rmax = 0xffff;
+
+			valid_number(blood->QueryField(i,0), (long &)value);
+			valid_number(blood->QueryField(i,1), (long &)rmin);
+			valid_number(blood->QueryField(i,2), (long &)rmax);
+			if (value>255 || rmin>0xffff || rmax>0xffff) {
+				printMessage("CharAnimations", "bloodclr entry:", LIGHT_RED);
+				printf("%02x %04x-%04x ", (unsigned int) value, (unsigned int) rmin, (unsigned int) rmax);
+				printStatus("Invalid value!", LIGHT_RED);
+				continue;
+			}
+			for(int j=0;j<AvatarsCount;j++) {
+				if (rmax<AvatarTable[j].AnimID) break;
+				if (rmin>AvatarTable[j].AnimID) continue;
+				AvatarTable[j].BloodColor = value;
+			}
+		}
+	}
+
+	AutoTable walk("walksnd");
+	if (walk) {
+		int rows = walk->GetRowCount();
+		for(int i=0;i<rows;i++) {
+			ieResRef value;
+			unsigned long rmin = 0;
+			unsigned long rmax = 0xffff;
+			unsigned int range = 0;
+
+			strnuprcpy(value, walk->QueryField(i,0), 8);
+			valid_number(walk->QueryField(i,1), (long &)rmin);
+			valid_number(walk->QueryField(i,2), (long &)rmax);
+			valid_number(walk->QueryField(i,3), (long &)range);
+			if (value[0]=='*') {
+				value[0]=0;
+				range = 0;
+			}
+			for(int j=0;j<AvatarsCount;j++) {
+				if (rmax<AvatarTable[j].AnimID) break;
+				if (rmin>AvatarTable[j].AnimID) continue;
+				memcpy(AvatarTable[j].WalkSound, value, sizeof(ieResRef) );
+				AvatarTable[j].WalkSoundCount = range;
+			}
+		}
+	}
 }
 
 CharAnimations::CharAnimations(unsigned int AnimID, ieDword ArmourLevel)
@@ -458,6 +556,7 @@ CharAnimations::CharAnimations(unsigned int AnimID, ieDword ArmourLevel)
 	Colors = NULL;
 	int i,j;
 	for (i = 0; i < 4; ++i) {
+		change[i] = true;
 		modifiedPalette[i] = NULL;
 		palette[i] = NULL;
 	}
@@ -487,13 +586,14 @@ CharAnimations::CharAnimations(unsigned int AnimID, ieDword ArmourLevel)
 		// make initial phase depend on location to make the pulse appear
 		// less even
 		ColorMods[i].phase = 5*i;
+		ColorMods[i].locked = false;
 	}
 	GlobalColorMod.type = RGBModifier::NONE;
 	GlobalColorMod.speed = 0;
 	GlobalColorMod.phase = 0;
+	GlobalColorMod.locked = false;
 	lastModUpdate = 0;
 
-
 	AvatarsRowNum=AvatarsCount;
 	if (core->HasFeature(GF_ONE_BYTE_ANIMID) ) {
 		ieDword tmp = AnimID&0xf000;
@@ -800,7 +900,7 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 			// Equipment animation parts
 
 			anims[part] = 0;
-			if (GetSize() == '*' || GetSize() == 0) continue;
+			if (GetSize() == 0) continue;
 
 			if (part == actorPartCount) {
 				if (WeaponRef[0] == 0) continue;
@@ -1444,9 +1544,12 @@ void CharAnimations::AddNFSuffix(char* ResRef, unsigned char StanceID,
 
 //Attack
 //h1, h2, w2
-static const char *SlashPrefix[]={"a1","a4","a7"};
-static const char *BackPrefix[]={"a2","a5","a8"};
-static const char *JabPrefix[]={"a3","a6","a9"};
+//static const char *SlashPrefix[]={"a1","a4","a7"};
+//static const char *BackPrefix[]={"a2","a5","a8"};
+//static const char *JabPrefix[]={"a3","a6","a9"};
+static const char *SlashPrefix[]={"a1","a2","a7"};
+static const char *BackPrefix[]={"a3","a4","a8"};
+static const char *JabPrefix[]={"a5","a6","a9"};
 static const char *RangedPrefix[]={"sa","sx","ss"};
 static const char *RangedPrefixOld[]={"sa","sx","a1"};
 
@@ -1914,6 +2017,7 @@ void CharAnimations::AddLRSuffix2( char* ResRef, unsigned char StanceID,
 			strcpy( EquipData->Suffix, "g1" );
 			Cycle = 32 + Orient / 2;
 			break;
+		case IE_ANI_SLEEP:
 		case IE_ANI_TWITCH:
 			strcat( ResRef, "g1" );
 			strcpy( EquipData->Suffix, "g1" );
@@ -2015,7 +2119,6 @@ void CharAnimations::GetLREquipmentRef(char* ResRef, unsigned char& Cycle,
 	Cycle = equip->Cycle;
 	//hackhackhack
 	sprintf( ResRef, "%4s%c%s", this->ResRef, equipRef[0], equip->Suffix );
-printf("LREquipment resref: %s\n", ResRef);
 }
 
 //Only for the ogre animation (MOGR)
@@ -2253,7 +2356,7 @@ void CharAnimations::PulseRGBModifiers()
 	if (time - lastModUpdate > 400) lastModUpdate = time - 40;
 
 	int inc = (time - lastModUpdate)/40;
-	bool change[4] = { false, false, false, false };
+	
 	if (GlobalColorMod.type != RGBModifier::NONE &&
 		GlobalColorMod.speed > 0)
 	{
@@ -2265,6 +2368,7 @@ void CharAnimations::PulseRGBModifiers()
 			GlobalColorMod.type = RGBModifier::NONE;
 			GlobalColorMod.phase = 0;
 			GlobalColorMod.speed = 0;
+			GlobalColorMod.locked = false;
 		}
 	}
 
@@ -2274,13 +2378,31 @@ void CharAnimations::PulseRGBModifiers()
 		{
 			ColorMods[i].phase += inc;
 			change[i>>3] = true;
+			if (ColorMods[i].phase > 2*ColorMods[i].speed) {
+				ColorMods[i].type = RGBModifier::NONE;
+				ColorMods[i].phase = 0;
+				ColorMods[i].speed = 0;
+				ColorMods[i].locked = false;
+			}
 		}
 	}
 
-	if (change[0]) SetupColors(PAL_MAIN);
-	if (change[1]) SetupColors(PAL_WEAPON);
-	if (change[2]) SetupColors(PAL_OFFHAND);
-	if (change[3]) SetupColors(PAL_HELMET);
+	if (change[0]) {
+		change[0]=0;
+		SetupColors(PAL_MAIN);
+	}
+	if (change[1]) {
+		change[1]=0;
+		SetupColors(PAL_WEAPON);
+	}
+	if (change[2]) {
+		change[2]=0;
+		SetupColors(PAL_OFFHAND);
+	}
+	if (change[3]) {
+		change[3]=0;
+		SetupColors(PAL_HELMET);
+	}
 
 	lastModUpdate += inc*40;
 }
