@@ -26,6 +26,7 @@
 
 #include "Interface.h"
 #include "Video.h"
+#include "EventMgr.h"
 #include <iostream>
 #include <fstream>
 using namespace std;
@@ -45,29 +46,33 @@ EventMgr::EventMgr(void)
 	rk_delay = 250;
 	rk_flags = GEM_RK_DISABLE;
 	fakeRMB=false;
+        proximitySensorEnabled=true;
+        rmbk='x';
 }
 
 EventMgr::~EventMgr(void)
 {
 }
 
-bool EventMgr::isProxymitySensorClosed()
+bool EventMgr::isProximitySensorClosed()
 {
 	bool res=false;
-	ifstream::pos_type size;
-	char * memblock;
-	ifstream file ("/sys/devices/platform/gpio-switch/proximity/state", ios::in|ios::binary|ios::ate);
-	//ifstream file ("/home/maemo/workspace/state", ios::in|ios::binary|ios::ate);
-	if (file.is_open())
-	{
-		size = file.tellg();
-		memblock = new char [size];
-		file.seekg (0, ios::beg);
-		file.read (memblock, size);
-		file.close();
-		res=(memblock[0]=='c');
-		delete[] memblock;
-	}
+        if(proximitySensorEnabled){
+            ifstream::pos_type size;
+            char * memblock;
+            ifstream file ("/sys/devices/platform/gpio-switch/proximity/state", ios::in|ios::binary|ios::ate);
+            //ifstream file ("/home/maemo/workspace/state", ios::in|ios::binary|ios::ate);
+            if (file.is_open())
+            {
+                    size = file.tellg();
+                    memblock = new char [size];
+                    file.seekg (0, ios::beg);
+                    file.read (memblock, size);
+                    file.close();
+                    res=(memblock[0]=='c');
+                    delete[] memblock;
+            }
+        }
 	return res;
 }
 
@@ -254,7 +259,7 @@ bool EventMgr::ClickMatch(unsigned short x, unsigned short y, unsigned long this
 unsigned short EventMgr::getFakeButton(unsigned short Button)
 {
 	unsigned short res=Button;
-	if(fakeRMB||isProxymitySensorClosed())
+	if(fakeRMB||isProximitySensorClosed())
 		res=GEM_MB_MENU;
 	return res;
 }
@@ -346,7 +351,7 @@ void EventMgr::MouseIdle(unsigned long /*time*/)
 /** BroadCast Key Press Event */
 void EventMgr::KeyPress(unsigned char Key, unsigned short Mod)
 {
-	if(Key=='x')
+	if(Key==rmbk)
 			fakeRMB=true;
 	if (last_win_focused == NULL) return;
 	Control *ctrl = last_win_focused->GetFocus();
@@ -356,7 +361,7 @@ void EventMgr::KeyPress(unsigned char Key, unsigned short Mod)
 /** BroadCast Key Release Event */
 void EventMgr::KeyRelease(unsigned char Key, unsigned short Mod)
 {
-	if(Key=='x')
+	if(Key==rmbk)
 		fakeRMB=false;
 	if (last_win_focused == NULL) return;
 	Control *ctrl = last_win_focused->GetFocus();
