@@ -26,14 +26,16 @@
 #include "strrefs.h"
 #include "win32def.h"
 
-#include "DataStream.h"
-#include "GameControl.h"
+#include "DisplayMessage.h"
 #include "GameData.h"
-#include "GameScript.h"
 #include "Interface.h"
 #include "MapMgr.h"
 #include "MusicMgr.h"
+#include "Particles.h"
 #include "ScriptEngine.h"
+#include "GameScript/GameScript.h"
+#include "GUI/GameControl.h"
+#include "System/DataStream.h"
 
 #define MAX_MAPS_LOADED 1
 
@@ -64,7 +66,7 @@ Game::Game(void) : Scriptable( ST_GLOBAL )
 	timestop_owner = NULL;
 	timestop_end = 0;
 	event_timer = 0;
-	event_handler[0] = 0;
+	event_handler = NULL;
 	weather = new Particles(200);
 	weather->SetRegion(0, 0, core->Width, core->Height);
 	LastScriptUpdate = 0;
@@ -94,6 +96,8 @@ Game::Game(void) : Scriptable( ST_GLOBAL )
 	}
 
 	interval = 1000/AI_UPDATE_TIME;
+	hasInfra = false;
+	familiarBlock = false;
 	//FIXME:i'm not sure in this...
 	NoInterrupt();
 }
@@ -136,12 +140,12 @@ Game::~Game(void)
 
 	i=savedpositions.size();
 	while(i--) {
-		delete savedpositions[i];
+		free (savedpositions[i]);
 	}
 
 	i=planepositions.size();
 	while(i--) {
-		delete planepositions[i];
+		free (planepositions[i]);
 	}
 }
 
@@ -200,6 +204,23 @@ Actor* Game::FindNPC(const char *scriptingname)
 	return NULL;
 }
 
+Actor *Game::GetGlobalActorByGlobalID(ieDword globalID)
+{
+	unsigned int slot;
+
+	for (slot=0; slot<PCs.size(); slot++) {
+		if (PCs[slot]->GetGlobalID()==globalID ) {
+			return PCs[slot];
+		}
+	}
+	for (slot=0; slot<NPCs.size(); slot++) {
+		if (NPCs[slot]->GetGlobalID()==globalID ) {
+			return NPCs[slot];
+		}
+	}
+	return NULL;
+}
+
 Actor* Game::GetPC(unsigned int slot, bool onlyalive)
 {
 	if (slot >= PCs.size()) {
@@ -296,8 +317,10 @@ void Game::ConsolidateParty()
 
 int Game::LeaveParty (Actor* actor)
 {
+	core->SetEventFlag(EF_PORTRAIT);
 	actor->CreateStats(); //create or update stats for leaving
 	actor->SetBase(IE_EXPLORE, 0);
+
 	SelectActor(actor, false, SELECT_NORMAL);
 	int slot = InParty( actor );
 	if (slot < 0) {
@@ -306,7 +329,9 @@ int Game::LeaveParty (Actor* actor)
 	std::vector< Actor*>::iterator m = PCs.begin() + slot;
 	PCs.erase( m );
 
+	ieDword id = actor->GetGlobalID();
 	for ( m = PCs.begin(); m != PCs.end(); ++m) {
+		(*m)->PCStats->LastLeft = id;
 		if ( (*m)->InParty>actor->InParty) {
 			(*m)->InParty--;
 		}
@@ -332,17 +357,31 @@ bool Game::DetermineStartPosType(const TableMgr *strta)
 	return false;
 }
 
+#define PMODE_COUNT 3
+
 void Game::InitActorPos(Actor *actor)
 {
 	//start.2da row labels
-	const char *mode[3] = { "NORMAL", "TUTORIAL", "EXPANSION" };
+	const char *mode[PMODE_COUNT] = { "NORMAL", "TUTORIAL", "EXPANSION" };
 
 	unsigned int ip = (unsigned int) (actor->InParty-1);
 	AutoTable start("start");
 	AutoTable strta("startpos");
+
+	if (!start || !strta) {
+		printMessage("Game","Game is missing character start data.\n",RED);
+		abort();
+	}
 	// 0 - single player, 1 - tutorial, 2 - expansion
 	ieDword playmode = 0;
 	core->GetDictionary()->Lookup( "PlayMode", playmode );
+
+	//Sometimes playmode is set to -1 (in pregenerate)
+	//normally execution shouldn't ever come here, but it actually does
+	//preventing problems by defaulting to the regular entry points
+	if (playmode>PMODE_COUNT) {
+		playmode = 0;
+	}
 	const char *xpos = start->QueryField(mode[playmode],"XPOS");
 	const char *ypos = start->QueryField(mode[playmode],"YPOS");
 	const char *area = start->QueryField(mode[playmode],"AREA");
@@ -352,17 +391,16 @@ void Game::InitActorPos(Actor *actor)
 	actor->Pos.y = actor->Destination.y = (short) atoi( strta->QueryField( strta->GetRowIndex(ypos), ip ) );
 	actor->SetOrientation( atoi( strta->QueryField( strta->GetRowIndex(rot), ip) ), false );
 
-	strta.load("startare");
-	strnlwrcpy(actor->Area, strta->QueryField( strta->GetRowIndex(area), 0 ), 8 );
-	//TODO: set viewport
-	// strta->QueryField(strta->GetRowIndex(xpos),0);
-	// strta->QueryField(strta->GetColumnIndex(ypos),0);
-
-	SelectActor(actor,true, SELECT_QUIET);
+	if (strta.load("startare")) {
+		strnlwrcpy(actor->Area, strta->QueryField( strta->GetRowIndex(area), 0 ), 8 );
+	} else {
+		strnlwrcpy(actor->Area, CurrentArea, 8 );
+	}
 }
 
 int Game::JoinParty(Actor* actor, int join)
 {
+	core->SetEventFlag(EF_PORTRAIT);
 	actor->CreateStats(); //create stats if they didn't exist yet
 	actor->InitButtons(actor->GetStat(IE_CLASS), false); //init actor's buttons
 	actor->SetBase(IE_EXPLORE, 1);
@@ -373,9 +411,23 @@ int Game::JoinParty(Actor* actor, int join)
 	if (slot != -1) {
 		return slot;
 	}
+	size_t size = PCs.size();
+	//set the lastjoined trigger
+
 	if (join&JP_JOIN) {
+		//update kit abilities of actor
+		actor->ApplyKit(false);
+		//update the quickslots
+		actor->ReinitQuickSlots();
+		//set the joining date
 		actor->PCStats->JoinDate = GameTime;
-		if (!PCs.size() ) {
+		if (size) {
+			ieDword id = actor->GetGlobalID();
+			for (size_t i=0;i<size; i++) {
+				Actor *a = GetPC(i, false);
+				a->PCStats->LastJoined = id;
+			}
+		} else {
 			Reputation = actor->GetStat(IE_REPUTATION);
 		}
 	}
@@ -384,13 +436,18 @@ int Game::JoinParty(Actor* actor, int join)
 		std::vector< Actor*>::iterator m = NPCs.begin() + slot;
 		NPCs.erase( m );
 	}
-	size_t size = PCs.size();
+
 
 	PCs.push_back( actor );
 	if (!actor->InParty) {
 		actor->InParty = (ieByte) (size+1);
 	}
 
+	if (join&(JP_INITPOS|JP_SELECT)) {
+		actor->Selected = 0; // don't confuse SelectActor!
+		SelectActor(actor,true, SELECT_NORMAL);
+	}
+
 	return ( int ) size;
 }
 
@@ -426,7 +483,7 @@ void Game::SetHotKey(unsigned long Key)
 bool Game::SelectPCSingle(int index)
 {
 	Actor* actor = FindPC( index );
-	if (!actor || ! actor->ValidTarget( GA_SELECT | GA_NO_DEAD | GA_NO_HIDDEN ))
+	if (!actor || ! actor->ValidTarget( GA_NO_HIDDEN ))
 		return false;
 
 	SelectedSingle = index;
@@ -454,48 +511,64 @@ bool Game::SelectActor(Actor* actor, bool select, unsigned flags)
 {
 	std::vector< Actor*>::iterator m;
 
-	// actor was not specified, which means all PCs should be (de)selected
+	// actor was not specified, which means all selectables should be (de)selected
 	if (! actor) {
 		for ( m = selected.begin(); m != selected.end(); ++m) {
 			(*m)->Select( false );
 			(*m)->SetOver( false );
 		}
-
 		selected.clear();
+
 		if (select) {
+			area->SelectActors();
+/*
 			for ( m = PCs.begin(); m != PCs.end(); ++m) {
 				if (! *m) {
 					continue;
 				}
 				SelectActor( *m, true, SELECT_QUIET );
 			}
+*/
 		}
 
 		if (! (flags & SELECT_QUIET)) {
 			core->SetEventFlag(EF_SELECTION);
 		}
+		Infravision();
 		return true;
 	}
 
 	// actor was specified, so we will work with him
-
-	// If actor is already (de)selected, report success, but do nothing
-	//if (actor->IsSelected() == select)
-	//	return true;
-
-
 	if (select) {
 		if (! actor->ValidTarget( GA_SELECT | GA_NO_DEAD ))
 			return false;
 
 		// deselect all actors first when exclusive
 		if (flags & SELECT_REPLACE) {
+			if (selected.size() == 1 && actor->IsSelected()) {
+				assert(selected[0] == actor);
+				// already the only selected actor
+				return true;
+			}
 			SelectActor( NULL, false, SELECT_QUIET );
+		} else if (actor->IsSelected()) {
+			// already selected
+			return true;
 		}
 
 		actor->Select( true );
+		assert(actor->IsSelected());
 		selected.push_back( actor );
 	} else {
+		if (!actor->IsSelected()) {
+			// already not selected
+			return true;
+
+			/*for ( m = selected.begin(); m != selected.end(); ++m) {
+				assert((*m) != actor);
+			}
+			return true;*/
+		}
 		for ( m = selected.begin(); m != selected.end(); ++m) {
 			if ((*m) == actor) {
 				selected.erase( m );
@@ -503,11 +576,13 @@ bool Game::SelectActor(Actor* actor, bool select, unsigned flags)
 			}
 		}
 		actor->Select( false );
+		assert(!actor->IsSelected());
 	}
 
 	if (! (flags & SELECT_QUIET)) {
 		core->SetEventFlag(EF_SELECTION);
 	}
+	Infravision();
 	return true;
 }
 
@@ -549,7 +624,7 @@ Map* Game::GetMap(unsigned int index) const
 
 Map *Game::GetMap(const char *areaname, bool change)
 {
-	int index = LoadMap(areaname);
+	int index = LoadMap(areaname, change);
 	if (index >= 0) {
 		if (change) {
 			MapIndex = index;
@@ -558,7 +633,17 @@ Map *Game::GetMap(const char *areaname, bool change)
 			area->SetupAmbients();
 			//change the tileset if needed
 			area->ChangeMap(IsDay());
-			ChangeSong();
+			ChangeSong(false, true);
+			Infravision();
+
+			//call area customization script for PST
+			//moved here because the current area is set here
+			ScriptEngine *sE = core->GetGUIScriptEngine();
+			if (core->HasFeature(GF_AREA_OVERRIDE) && sE) {
+				//area ResRef is accessible by GemRB.GetGameString (STR_AREANAME)
+				sE->RunFunction("Maze", "CustomizeArea");
+			}
+
 			return area;
 		}
 		return GetMap(index);
@@ -656,29 +741,44 @@ int Game::DelMap(unsigned int index, int forced)
 	return 0;
 }
 
-/* Loads an area, changepf == true if you want to setup the pathfinder too */
-//FIXME: changepf is removed now
-int Game::LoadMap(const char* ResRef)
+/* Loads an area */
+int Game::LoadMap(const char* ResRef, bool loadscreen)
 {
 	unsigned int i;
+	Map *newMap;
+	PluginHolder<MapMgr> mM(IE_ARE_CLASS_ID);
+	ScriptEngine *sE = core->GetGUIScriptEngine();
+
+	//this shouldn't happen
+	if (!mM) {
+		return -1;
+	}
+
 	int index = FindMap(ResRef);
 	if (index>=0) {
 		return index;
 	}
 
+	bool hide = false;
+	if (loadscreen && sE) {
+		hide = core->HideGCWindow();
+		sE->RunFunction("LoadScreen", "StartLoadScreen");
+		sE->RunFunction("LoadScreen", "SetLoadScreen");
+	}
 	DataStream* ds = gamedata->GetResource( ResRef, IE_ARE_CLASS_ID );
 	if (!ds) {
-		return -1;
+		goto failedload;
 	}
-	PluginHolder<MapMgr> mM(IE_ARE_CLASS_ID);
 	if(!mM->Open( ds, true )) {
-		return -1;
+		goto failedload;
 	}
-	Map* newMap = mM->GetMap(ResRef, IsDay());
+	newMap = mM->GetMap(ResRef, IsDay());
 	if (!newMap) {
-		return -1;
+		goto failedload;
 	}
 
+	core->LoadProgress(100);
+
 	for (i = 0; i < PCs.size(); i++) {
 		if (stricmp( PCs[i]->Area, ResRef ) == 0) {
 			newMap->AddActor( PCs[i] );
@@ -689,7 +789,15 @@ int Game::LoadMap(const char* ResRef)
 			newMap->AddActor( NPCs[i] );
 		}
 	}
+	if (hide) {
+		core->UnhideGCWindow();
+	}
 	return AddMap( newMap );
+failedload:
+	if (hide) 
+		core->UnhideGCWindow();
+	core->LoadProgress(100);
+	return -1;
 }
 
 int Game::AddNPC(Actor* npc)
@@ -729,14 +837,14 @@ void Game::SwapPCs(unsigned int Index1, unsigned int Index2)
 	PCs[Index1]->InParty = PCs[Index2]->InParty;
 	PCs[Index2]->InParty = tmp;
 	//signal a change of the portrait window
-	core->SetEventFlag(EF_PORTRAIT);
+	core->SetEventFlag(EF_PORTRAIT | EF_SELECTION);
 }
 
 void Game::DeleteJournalEntry(ieStrRef strref)
 {
 	size_t i=Journals.size();
 	while(i--) {
-		if (Journals[i]->Text==strref) {
+		if ((Journals[i]->Text==strref) || (strref==(ieStrRef) -1) ) {
 			delete Journals[i];
 			Journals.erase(Journals.begin()+i);
 		}
@@ -781,6 +889,7 @@ bool Game::AddJournalEntry(ieStrRef strref, int Section, int Group)
 	ieDword chapter = 0;
 	locals->Lookup("CHAPTER", chapter);
 	je->Chapter = (ieByte) chapter;
+	je->unknown09 = 0;
 	je->Section = (ieByte) Section;
 	je->Group = (ieByte) Group;
 	je->Text = strref;
@@ -942,9 +1051,9 @@ void Game::ShareXP(int xp, int flags)
 	}
 
 	if (xp>0) {
-		core->DisplayConstantStringValue( STR_GOTXP, 0xbcefbc, (ieDword) xp); //you have gained ... xp
+		displaymsg->DisplayConstantStringValue( STR_GOTXP, 0xbcefbc, (ieDword) xp); //you have gained ... xp
 	} else {
-		core->DisplayConstantStringValue( STR_LOSTXP, 0xbcefbc, (ieDword) -xp); //you have lost ... xp
+		displaymsg->DisplayConstantStringValue( STR_LOSTXP, 0xbcefbc, (ieDword) -xp); //you have lost ... xp
 	}
 	for (unsigned int i=0; i<PCs.size(); i++) {
 		if (PCs[i]->GetStat(IE_STATE_ID)&STATE_DEAD) {
@@ -988,6 +1097,7 @@ bool Game::EveryoneNearPoint(Map *area, const Point &p, int flags) const
 			return false;
 		}
 		if (Distance(p,PCs[i])>MAX_TRAVELING_DISTANCE) {
+printf("Actor %s is not near!\n", PCs[i]->LongName);
 			return false;
 		}
 	}
@@ -1030,7 +1140,8 @@ void Game::IncrementChapter()
 	//chapter first set to 0 (prologue)
 	ieDword chapter = (ieDword) -1;
 	locals->Lookup("CHAPTER",chapter);
-	locals->SetAt("CHAPTER",chapter+1);
+	//increment chapter only if it exists
+	locals->SetAt("CHAPTER", chapter+1, core->HasFeature(GF_NO_NEW_VARIABLES) );
 	//clear statistics
 	for (unsigned int i=0; i<PCs.size(); i++) {
 		//all PCs must have this!
@@ -1043,9 +1154,9 @@ void Game::SetReputation(ieDword r)
 	if (r<10) r=10;
 	else if (r>200) r=200;
 	if (Reputation>r) {
-		core->DisplayConstantStringValue(STR_LOSTREP,0xc0c000,(Reputation-r)/10);
+		displaymsg->DisplayConstantStringValue(STR_LOSTREP,0xc0c000,(Reputation-r)/10);
 	} else if (Reputation<r) {
-		core->DisplayConstantStringValue(STR_GOTREP,0xc0c000,(r-Reputation)/10);
+		displaymsg->DisplayConstantStringValue(STR_GOTREP,0xc0c000,(r-Reputation)/10);
 	}
 	Reputation = r;
 	for (unsigned int i=0; i<PCs.size(); i++) {
@@ -1075,9 +1186,9 @@ void Game::AddGold(ieDword add)
 	old = PartyGold;
 	PartyGold += add;
 	if (old<PartyGold) {
-		core->DisplayConstantStringValue( STR_GOTGOLD, 0xc0c000, PartyGold-old);
+		displaymsg->DisplayConstantStringValue( STR_GOTGOLD, 0xc0c000, PartyGold-old);
 	} else {
-		core->DisplayConstantStringValue( STR_LOSTGOLD, 0xc0c000, old-PartyGold);
+		displaymsg->DisplayConstantStringValue( STR_LOSTGOLD, 0xc0c000, old-PartyGold);
 	}
 }
 
@@ -1138,7 +1249,7 @@ bool Game::PCInCombat(Actor* actor) const
 	if (actor->LastTarget) {
 		return true;
 	}
-	if (AttackersOf(actor->GetID(), actor->GetCurrentArea())) {
+	if (AttackersOf(actor->GetGlobalID(), actor->GetCurrentArea())) {
 		return true;
 	}
 	return false;
@@ -1198,16 +1309,42 @@ void Game::UpdateScripts()
 	ProcessActions(false);
 	size_t idx;
 
+	bool PartyAttack = false;
+
 	for (idx=0;idx<Maps.size();idx++) {
 		Maps[idx]->UpdateScripts();
 		size_t acnt=Attackers.size();
 		while(acnt--) {
 			Actor *actor = Maps[idx]->GetActorByGlobalID(Attackers[acnt]);
 			if (actor) {
-				//each attacker handles their own round initiation
-				//FIXME: individual combat counter
-				CombatCounter++;
-				actor->InitRound(GameTime, !(CombatCounter&1) );
+				if ( !Maps[idx]->GetActorByGlobalID(actor->LastTarget) ) {
+					//Actor's target left area
+					OutAttack(Attackers[acnt]);
+					continue;
+				} else {
+					//each attacker handles their own round initiation
+					actor->InitRound(GameTime);
+					if (actor->InParty) {
+						PartyAttack = true;
+					}
+				}
+			} else {
+				//Attacker is gone from area
+				OutAttack(Attackers[acnt]);
+			}
+		}
+	}
+
+	if (PartyAttack) {
+		//ChangeSong will set the battlesong only if CombatCounter is nonzero
+		CombatCounter=150;
+		ChangeSong(false, true);
+	} else {
+		if (CombatCounter) {
+			CombatCounter--;
+			//Change song if combatcounter went down to 0
+			if (!CombatCounter) {
+				ChangeSong(false, false);
 			}
 		}
 	}
@@ -1230,14 +1367,14 @@ void Game::UpdateScripts()
 	// (we should probably find a less silly way to handle this,
 	// because nothing can ever stop area music now..)
 	if (!core->GetMusicMgr()->IsPlaying()) {
-		 ChangeSong(false);
+		ChangeSong(false,false);
 	}
 
 	//this is used only for the death delay so far
-	if (event_handler[0]) {
+	if (event_handler) {
 		if (!event_timer) {
-			core->GetGUIScriptEngine()->RunFunction(event_handler);
-			event_handler[0]=0;
+			event_handler->call();
+			event_handler = NULL;
 		}
 		event_timer--;
 	}
@@ -1245,21 +1382,21 @@ void Game::UpdateScripts()
 	if (EveryoneDead()) {
 		//don't check it any more
 		protagonist = PM_NO;
-		core->GetGUIScriptEngine()->RunFunction("DeathWindow");
+		core->GetGUIScriptEngine()->RunFunction("GUIWORLD", "DeathWindow");
 		return;
 	}
 
 	if (PartyOverflow()) {
 		partysize = 0;
-		core->GetGUIScriptEngine()->RunFunction("OpenReformPartyWindow");
+		core->GetGUIScriptEngine()->RunFunction("GUIWORLD", "OpenReformPartyWindow");
 		return;
 	}
 }
 
-void Game::SetTimedEvent(const char *fname, int count)
+void Game::SetTimedEvent(EventHandler func, int count)
 {
 	event_timer = count;
-	strncpy(event_handler, fname, sizeof(event_handler) );
+	event_handler = func;
 }
 
 void Game::SetProtagonistMode(int mode)
@@ -1320,7 +1457,7 @@ void Game::RestParty(int checks, int dream, int hp)
 	if (!(checks&REST_NOSCATTER) ) {
 		if (!EveryoneNearPoint( area, leader->Pos, 0 ) ) {
 			//party too scattered
-			core->DisplayConstantString( STR_SCATTERED, 0xff0000 );
+			displaymsg->DisplayConstantString( STR_SCATTERED, 0xff0000 );
 			return;
 		}
 	}
@@ -1328,12 +1465,12 @@ void Game::RestParty(int checks, int dream, int hp)
 	if (!(checks&REST_NOCRITTER) ) {
 		//don't allow resting while in combat
 		if (AnyPCInCombat()) {
-			core->DisplayConstantString( STR_CANTRESTMONS, 0xff0000 );
+			displaymsg->DisplayConstantString( STR_CANTRESTMONS, 0xff0000 );
 			return;
 		}
 		//don't allow resting if hostiles are nearby
 		if (area->AnyEnemyNearPoint(leader->Pos)) {
-			core->DisplayConstantString( STR_CANTRESTMONS, 0xff0000 );
+			displaymsg->DisplayConstantString( STR_CANTRESTMONS, 0xff0000 );
 			return;
 		}
 	}
@@ -1344,13 +1481,13 @@ void Game::RestParty(int checks, int dream, int hp)
 	if (!(checks&REST_NOAREA) ) {
 		//you cannot rest here
 		if (area->AreaFlags&1) {
-			core->DisplayConstantString( STR_MAYNOTREST, 0xff0000 );
+			displaymsg->DisplayConstantString( STR_MAYNOTREST, 0xff0000 );
 			return;
 		}
 		//you may not rest here, find an inn
 		if (!(area->AreaType&(AT_OUTDOOR|AT_FOREST|AT_DUNGEON|AT_CAN_REST) ))
 		{
-			core->DisplayConstantString( STR_MAYNOTREST, 0xff0000 );
+			displaymsg->DisplayConstantString( STR_MAYNOTREST, 0xff0000 );
 			return;
 		}
 		//area encounters
@@ -1360,7 +1497,6 @@ void Game::RestParty(int checks, int dream, int hp)
 	}
 	AdvanceTime(2400*AI_UPDATE_TIME);
 
-
 	int i = GetPartySize(true); // party size, only alive
 
 	while (i--) {
@@ -1377,7 +1513,6 @@ void Game::RestParty(int checks, int dream, int hp)
 
 	//movie and cutscene dreams
 	if (dream>=0) {
-
 		//cutscene dreams
 		if (gamedata->Exists("player1d",IE_BCS_CLASS_ID, true))
 			PlayerDream();
@@ -1400,16 +1535,18 @@ void Game::RestParty(int checks, int dream, int hp)
 	core->SetEventFlag(EF_ACTION);
 
 	//restindex will be -1 in the case of PST
-	int restindex = core->GetStringReference(STR_REST);
+	//FIXME: I don't quite see why we can't sumply use the same strings.2da entry
+	//It seems we could reduce complexity here, and free up 2-3 string slots too
+	int restindex = displaymsg->GetStringReference(STR_REST);
 	int strindex;
 	char* tmpstr = NULL;
 
 	core->GetTokenDictionary()->SetAtCopy("HOUR", hours);
 	if (restindex != -1) {
-		strindex = core->GetStringReference(STR_HOURS);
+		strindex = displaymsg->GetStringReference(STR_HOURS);
 	} else {
-		strindex = core->GetStringReference(STR_PST_HOURS);
-		restindex = core->GetStringReference(STR_PST_REST);
+		strindex = displaymsg->GetStringReference(STR_PST_HOURS);
+		restindex = displaymsg->GetStringReference(STR_PST_REST);
 	}
 
 	//this would be bad
@@ -1420,7 +1557,7 @@ void Game::RestParty(int checks, int dream, int hp)
 
 	core->GetTokenDictionary()->SetAtCopy("DURATION", tmpstr);
 	core->FreeString(tmpstr);
-	core->DisplayString(restindex, 0xffffff, 0);
+	displaymsg->DisplayString(restindex, 0xffffff, 0);
 }
 
 //timestop effect
@@ -1430,6 +1567,25 @@ void Game::TimeStop(Actor* owner, ieDword end)
 	timestop_end=GameTime+end;
 }
 
+//recalculate the party's infravision state
+void Game::Infravision()
+{
+	hasInfra = false;
+	Map *map = GetCurrentArea();
+	if (!map) return;
+	for(size_t i=0;i<PCs.size();i++) {
+		Actor *actor = PCs[i];
+		if (!IsAlive(actor)) continue;
+		if (actor->GetCurrentArea()!=map) continue;
+		//Group infravision overrides this???
+		if (!actor->Selected) continue;
+		if (actor->GetStat(IE_STATE_ID) & STATE_INFRA) {
+			hasInfra = true;
+			return;
+		}
+	}
+}
+
 //returns the colour which should be applied onto the whole game area viewport
 //this is based on timestop, dream area, weather, daytime
 
@@ -1469,6 +1625,7 @@ const Color *Game::GetGlobalTint() const
 			return &FogTint;
 		}
 	}
+
 	return NULL;
 }
 
@@ -1489,10 +1646,6 @@ void Game::InAttack(ieDword globalID)
 		if (*idx==globalID) return;
 	}
 	Attackers.push_back(globalID);
-	if (!CombatCounter) {
-		CombatCounter++;
-		ChangeSong();
-	}
 }
 
 void Game::OutAttack(ieDword globalID)
@@ -1502,28 +1655,27 @@ void Game::OutAttack(ieDword globalID)
 	for(idx=Attackers.begin(); idx!=Attackers.end();idx++) {
 		if (*idx==globalID) {
 			Attackers.erase(idx);
-			if (!Attackers.size()) {
-				CombatCounter = 0;
-				ChangeSong();
-			}
 			break;
 		}
 	}
 }
 
-void Game::ChangeSong(bool force)
+void Game::ChangeSong(bool always, bool force)
 {
 	int Song;
 
 	if (CombatCounter) {
 		//battlesong
-		Song = 3;
+		Song = SONG_BATTLE;
 	} else {
-		//night or day
+		//will select SONG_DAY or SONG_NIGHT
 		Song = (GameTime/AI_UPDATE_TIME)%7200/3600;
 	}
 	//area may override the song played (stick in battlemusic)
-	area->PlayAreaSong( Song, force );
+	//always transition gracefully with ChangeSong
+	//force just means, we schedule the song for later, if currently
+	//is playing
+	area->PlayAreaSong( Song, always, force );
 }
 
 int Game::AttackersOf(ieDword globalID, Map *area) const
@@ -1619,9 +1771,27 @@ void Game::StartRainOrSnow(bool conditional, int w)
 	weather->SetPhase(P_FADE);
 }
 
-void Game::SetExpansion(int exp)
+void Game::SetExpansion(ieDword value)
 {
-	Expansion = exp;
+	if (Expansion>=value) {
+		return;
+	}
+	Expansion = value;
+
+	switch(Expansion) {
+	default:
+		core->SetEventFlag(EF_EXPANSION);
+		break;
+	//TODO: move this hardcoded hack to the scripts
+	case 5:
+		core->GetDictionary()->SetAt( "PlayMode", 2 );
+
+		int i = GetPartySize(false);
+		while(i--) {
+			Actor *actor = GetPC(i, false);
+			InitActorPos(actor);
+		}
+	}
 }
 
 void Game::DebugDump()
@@ -1634,23 +1804,45 @@ void Game::DebugDump()
 
 		printf("%s\n",map->GetScriptName());
 	}
+	printf("Current area: %s   Previous area: %s\n", CurrentArea, PreviousArea);
+	printf("Global script: %s\n", Scripts[0]->GetName());
 	printf("CombatCounter: %d\n", (int) CombatCounter);
 	printf("Attackers count: %d\n", (int) Attackers.size());
+	for(idx=0;idx<Attackers.size(); idx++) {
+		Actor *actor = GetActorByGlobalID(Attackers[idx]);
+		if (!actor) {
+			printf("Name: ???\n");
+			continue;
+		}
+		Actor *whom = GetActorByGlobalID(actor->LastTarget);
+		printf("Name: %s Attacking : %s\n", actor->ShortName, whom?whom->ShortName:"???");
+	}
+
 	printf("Party size: %d\n", (int) PCs.size());
 	for(idx=0;idx<PCs.size();idx++) {
 		Actor *actor = PCs[idx];
 
-		printf("Name: %s Order %d\n",actor->ShortName, actor->InParty);
+		printf("Name: %s Order %d %s\n",actor->ShortName, actor->InParty, actor->Selected?"x":"-");
 	}
 }
 
-Actor *Game::GetActorByGlobalID(ieWord objectID) {
+Actor *Game::GetActorByGlobalID(ieDword globalID)
+{
 	size_t mc = GetLoadedMapCount();
 	while(mc--) {
 		Map *map = GetMap(mc);
-		Actor *actor = map->GetActorByGlobalID(objectID);
+		Actor *actor = map->GetActorByGlobalID(globalID);
 		if (actor) return actor;
 	}
-	return NULL;
+	return GetGlobalActorByGlobalID(globalID);
+}
+
+ieByte *Game::AllocateMazeData()
+{
+	if (mazedata) {
+		free(mazedata);
+	}
+	mazedata = (ieByte*)malloc(MAZE_DATA_SIZE);
+	return mazedata;
 }
 
