@@ -725,6 +725,10 @@ private:
 	/** the internal (without cache) part of GetListFrom2DA */
 	ieDword *GetListFrom2DAInternal(const ieResRef resref);
 public:
+        //N900 specific config
+        char RMBK;
+        int ProximitySensorEnabled;
+        //En N900 variables
 	char GameDataPath[_MAX_PATH];
 	char GameOverridePath[_MAX_PATH];
 	char GameSoundsPath[_MAX_PATH];
