@@ -26,12 +26,13 @@
 #include "win32def.h"
 #include "strrefs.h"
 
-#include "Actor.h"
+#include "DisplayMessage.h"
 #include "Game.h"
 #include "GameData.h"
 #include "Interface.h"
 #include "Item.h"
 #include "ScriptEngine.h"
+#include "Scriptable/Actor.h"
 
 #include <cstdio>
 
@@ -94,6 +95,7 @@ Inventory::Inventory()
 	Equipped = IW_NO_EQUIPPED;
 	EquippedHeader = 0;
 	ItemExcl = 0;
+	memset(ItemTypes, 0, sizeof(ItemTypes));
 }
 
 Inventory::~Inventory()
@@ -106,6 +108,39 @@ Inventory::~Inventory()
 	}
 }
 
+// duplicates the source inventory into the current one
+// also changes the items to not drop, so simulacrum and similar don't become factories
+void Inventory::CopyFrom(const Actor *source)
+{
+	if (!source) {
+		return;
+	}
+
+	SetSlotCount(source->inventory.GetSlotCount());
+
+	// allocate the items and mark them undroppable
+	CREItem *tmp, *item;
+	for (size_t i = 0; i < source->inventory.Slots.size(); i++) {
+		item = source->inventory.Slots[i];
+		if (item) {
+			tmp = new CREItem();
+			memcpy(tmp, item, sizeof(CREItem));
+			tmp->Flags |= IE_INV_ITEM_UNDROPPABLE;
+			int ret = AddSlotItem(tmp, i);
+			if (ret != ASI_SUCCESS) {
+				delete tmp;
+			}
+		}
+	}
+
+	// preserve the equipped status
+	Equipped = source->inventory.GetEquipped();
+	EquippedHeader = source->inventory.GetEquippedHeader();
+
+	Changed = true;
+	CalculateWeight();
+}
+
 CREItem *Inventory::GetItem(unsigned int slot)
 {
 	if (slot >= Slots.size() ) {
@@ -117,11 +152,38 @@ CREItem *Inventory::GetItem(unsigned int slot)
 	return item;
 }
 
+//This hack sets the charge counters for non-rechargeable items,
+//if their charge is zero
+inline void HackCharges(CREItem *item)
+{
+	Item *itm = gamedata->GetItem( item->ItemResRef );
+	if (itm) {
+		for (int i=0;i<3;i++) {
+			if (item->Usages[i]) {
+				continue;
+			}
+			ITMExtHeader *h = itm->GetExtHeader(i);
+			if (h && !(h->RechargeFlags&IE_ITEM_RECHARGE)) {
+				//HACK: the original (bg2) allows for 0 charged gems
+				if (h->Charges) {
+					item->Usages[i] = h->Charges;
+				} else {
+					item->Usages[i] = 1;
+				}
+			}
+		}
+		gamedata->FreeItem( itm, item->ItemResRef, false );
+	}
+}
+
 void Inventory::AddItem(CREItem *item)
 {
 	if (!item) return; //invalid items get no slot
 	Slots.push_back(item);
-	//Changed=true; //probably not needed, chests got no encumbrance
+	HackCharges(item);
+	//this will update the flags (needed for unmovable items in containers)
+	//but those *can* be picked up (like the bg2 portal key), so we skip it
+	//Changed=true;
 }
 
 void Inventory::CalculateWeight()
@@ -135,7 +197,6 @@ void Inventory::CalculateWeight()
 		if (!slot) {
 			continue;
 		}
-		//printf ("%2d: %8s : %d x %d\n", (int) i, slot->ItemResRef, slot->Weight, slot->Usages[0]);
 		if (slot->Weight == -1) {
 			Item *itm = gamedata->GetItem( slot->ItemResRef );
 			if (itm) {
@@ -200,6 +261,11 @@ void Inventory::AddSlotEffects(ieDword index)
 		return;
 	}
 	ItemExcl|=itm->ItemExcl;
+	ieDword pos = itm->ItemType/32;
+	ieDword bit = itm->ItemType%32;
+	if (pos<4) {
+		ItemTypes[pos]|=1<<bit;
+	}
 
 	ieWord gradient = itm->GetWieldedGradient();
 	if (gradient!=0xffff) {
@@ -207,7 +273,7 @@ void Inventory::AddSlotEffects(ieDword index)
 	}
 
 	//get the equipping effects
-	EffectQueue *eqfx = itm->GetEffectBlock(-1, index, 0);
+	EffectQueue *eqfx = itm->GetEffectBlock(Owner, Owner->Pos, -1, index, 0);
 	gamedata->FreeItem( itm, slot->ItemResRef, false );
 
 	Owner->RefreshEffects(eqfx);
@@ -264,6 +330,14 @@ bool Inventory::HasItemInSlot(const char *resref, unsigned int slot) const
 	return false;
 }
 
+bool Inventory::HasItemType(ieDword type) const
+{
+	if (type>255) return false;
+	int idx = type/32;
+	int bit = type%32;
+	return (ItemTypes[idx] & (1<<bit) )!=0;
+}
+
 /** counts the items in the inventory, if stacks == 1 then stacks are
 		accounted for their heap size */
 int Inventory::CountItems(const char *resref, bool stacks) const
@@ -323,12 +397,12 @@ void Inventory::KillSlot(unsigned int index)
 	}
 
 	//the used up item vanishes from the quickslot bar
-	if (Owner->InParty) {
+	if (Owner->IsSelected()) {
 		core->SetEventFlag( EF_ACTION );
 	}
 
 	Slots[index] = NULL;
- 	int effect = core->QuerySlotEffects( index );
+	int effect = core->QuerySlotEffects( index );
 	if (!effect) {
 		return;
 	}
@@ -391,7 +465,7 @@ void Inventory::KillSlot(unsigned int index)
 			// reset Equipped if it is a ranged weapon slot
 			// but not magic weapon slot!
 
-			UpdateWeaponAnimation();      
+			UpdateWeaponAnimation();
 			break;
 		case SLOT_EFFECT_HEAD:
 			Owner->SetUsedHelmet("");
@@ -455,7 +529,7 @@ unsigned int Inventory::DestroyItem(const char *resref, ieDword flags, ieDword c
 		if (count && (destructed>=count) )
 			break;
 	}
-	if (Changed && Owner && Owner->InParty) core->DisplayConstantString(STR_LOSTITEM, 0xbcefbc);
+	if (Changed && Owner && Owner->InParty) displaymsg->DisplayConstantString(STR_LOSTITEM, 0xbcefbc);
 
 	return destructed;
 }
@@ -470,6 +544,11 @@ CREItem *Inventory::RemoveItem(unsigned int slot, unsigned int count)
 	}
 	Changed = true;
 	item = Slots[slot];
+
+	if (!item) {
+		return NULL;
+	}
+
 	if (!count || !(item->Flags&IE_INV_ITEM_STACKED) ) {
 		KillSlot(slot);
 		return item;
@@ -490,13 +569,21 @@ CREItem *Inventory::RemoveItem(unsigned int slot, unsigned int count)
 int Inventory::RemoveItem(const char *resref, unsigned int flags, CREItem **res_item)
 {
 	size_t slot = Slots.size();
+	unsigned int mask = (flags^(IE_INV_ITEM_UNDROPPABLE|IE_INV_ITEM_EQUIPPED));
+	if (core->HasFeature(GF_NO_DROP_CAN_MOVE) ) {
+		mask &= ~IE_INV_ITEM_UNDROPPABLE;
+	}
 	while(slot--) {
 		CREItem *item = Slots[slot];
 		if (!item) {
 			continue;
 		}
-		if ( ((flags^(IE_INV_ITEM_UNDROPPABLE|IE_INV_ITEM_EQUIPPED))&item->Flags)!=flags) {
-				continue;
+
+		if (flags && (mask&item->Flags)==flags) {
+			continue;
+		}
+		if (!flags && (mask&item->Flags)!=0) {
+			continue;
 		}
 		if (resref[0] && strnicmp(item->ItemResRef, resref, 8) ) {
 			continue;
@@ -518,10 +605,13 @@ void Inventory::SetSlotItem(CREItem* item, unsigned int slot)
 	if (Slots[slot]) {
 		delete Slots[slot];
 	}
+
+	HackCharges(item);
+
 	Slots[slot] = item;
 
 	//update the action bar next time
-	if (Owner->InParty) {
+	if (Owner->IsSelected()) {
 		core->SetEventFlag( EF_ACTION );
 	}
 }
@@ -535,22 +625,21 @@ int Inventory::AddSlotItem(CREItem* item, int slot, int slottype)
 			return ASI_FAILED;
 		}
 
-		//check for equipping weapons 
+		//check for equipping weapons
 		if (WhyCantEquip(slot,twohanded)) {
 			return ASI_FAILED;
 		}
 
 		if (!Slots[slot]) {
 			item->Flags |= IE_INV_ITEM_ACQUIRED;
-			Slots[slot] = item;
-			Changed = true;
+			SetSlotItem(item, slot);
 			EquipItem(slot);
 			return ASI_SUCCESS;
 		}
 
 		CREItem *myslot = Slots[slot];
 		if (ItemsAreCompatible( myslot, item )) {
- 			//calculate with the max movable stock
+			//calculate with the max movable stock
 			int chunk = item->Usages[0];
 			int newamount = myslot->Usages[0]+chunk;
 			if (newamount>myslot->StackAmount) {
@@ -596,9 +685,6 @@ int Inventory::AddSlotItem(CREItem* item, int slot, int slottype)
 				continue;
 			}
 		}
-		//this is called by AddSlotItem as well
-		//if (WhyCantEquip(i,twohanded))
-		//	continue;
 		int part_res = AddSlotItem (item, i);
 		if (part_res == ASI_SUCCESS) return ASI_SUCCESS;
 		else if (part_res == ASI_PARTIAL) res = ASI_PARTIAL;
@@ -607,6 +693,26 @@ int Inventory::AddSlotItem(CREItem* item, int slot, int slottype)
 	return res;
 }
 
+//Used by FillSlot
+void Inventory::TryEquipAll(int slot)
+{
+	for(int i=SLOT_INV;i<=LAST_INV;i++) {
+		CREItem *item = Slots[i];
+		if (!item) {
+			continue;
+		}
+
+		Slots[i]=NULL;
+		if (AddSlotItem(item, slot) == ASI_SUCCESS) {
+			return;
+		}
+		//try to stuff it back, it should work
+		if (AddSlotItem(item, i) != ASI_SUCCESS) {
+			delete item;
+		}
+	}
+}
+
 int Inventory::AddStoreItem(STOItem* item, int action)
 {
 	CREItem *temp;
@@ -702,13 +808,17 @@ int Inventory::DepleteItem(ieDword flags)
 // TODO: once all callers have been checked, this can be reversed to make more sense
 int Inventory::FindItem(const char *resref, unsigned int flags) const
 {
+	unsigned int mask = (flags^IE_INV_ITEM_UNDROPPABLE);
+	if (core->HasFeature(GF_NO_DROP_CAN_MOVE) ) {
+		mask &= ~IE_INV_ITEM_UNDROPPABLE;
+	}
 	for (size_t i = 0; i < Slots.size(); i++) {
 		const CREItem *item = Slots[i];
 		if (!item) {
 			continue;
 		}
-		if ( (flags^IE_INV_ITEM_UNDROPPABLE) & item->Flags ) {
-				continue;
+		if ( mask & item->Flags ) {
+			continue;
 		}
 		if (resref[0] && strnicmp(item->ItemResRef, resref, 8) ) {
 			continue;
@@ -772,6 +882,8 @@ bool Inventory::DropItemAtLocation(const char *resref, unsigned int flags, Map *
 		if (resref[0] && strnicmp(item->ItemResRef, resref, 8) ) {
 			continue;
 		}
+		// mark it as unequipped, so it doesn't cause problems in stores
+		item->Flags &= ~ IE_INV_ITEM_EQUIPPED;
 		map->AddItemToLocation(loc, item);
 		Changed = true;
 		dropped = true;
@@ -780,6 +892,26 @@ bool Inventory::DropItemAtLocation(const char *resref, unsigned int flags, Map *
 		if (resref[0])
 			break;
 	}
+
+	//dropping gold too
+	if (!resref[0]) {
+		if (Owner->Type==ST_ACTOR) {
+			Actor *act = (Actor *) Owner;
+			if (! act->BaseStats[IE_GOLD]) {
+				return dropped;
+			}
+			CREItem *gold = new CREItem();
+		
+			gold->Expired=0;
+			gold->Flags=0;
+			gold->Usages[1]=0;
+			gold->Usages[2]=0;
+			memcpy(gold->ItemResRef, core->GoldResRef, sizeof(ieResRef) );
+			gold->Usages[0] = act->BaseStats[IE_GOLD];
+			act->BaseStats[IE_GOLD] = 0;
+			map->AddItemToLocation(loc, gold);
+		}
+	}
 	return dropped;
 }
 
@@ -872,7 +1004,7 @@ bool Inventory::EquipItem(unsigned int slot)
 			UpdateWeaponAnimation();
 		}
 		break;
-	case SLOT_EFFECT_MISSILE:		
+	case SLOT_EFFECT_MISSILE:
 		//Get the ranged header of the projectile (so we theoretically allow shooting of daggers)
 		EquippedHeader = itm->GetWeaponHeaderNumber(true);
 		header = itm->GetExtHeader(EquippedHeader);
@@ -908,6 +1040,9 @@ bool Inventory::EquipItem(unsigned int slot)
 		if (item->Flags & IE_INV_ITEM_CURSED) {
 			item->Flags|=IE_INV_ITEM_UNDROPPABLE;
 		}
+		if (effect == SLOT_EFFECT_MISSILE) {
+			slot = FindRangedWeapon();
+		}
 		AddSlotEffects( slot );
 	}
 	return true;
@@ -929,8 +1064,10 @@ bool Inventory::UnEquipItem(unsigned int slot, bool removecurse)
 			return false;
 		}
 	}
-	if (item->Flags & IE_INV_ITEM_UNDROPPABLE) {
-		return false;
+	if (!core->HasFeature(GF_NO_DROP_CAN_MOVE) || (item->Flags&IE_INV_ITEM_CURSED) ) {
+		if (item->Flags & IE_INV_ITEM_UNDROPPABLE ) {
+			return false;
+		}
 	}
 	item->Flags &= ~IE_INV_ITEM_EQUIPPED; //no idea if this is needed, won't hurt
 	return true;
@@ -1319,7 +1456,7 @@ void Inventory::dump()
 			continue;
 		}
 
-		printf ( "%2u: %8.8s - (%d %d %d) %x Wt: %d x %dLb\n", i, itm->ItemResRef, itm->Usages[0], itm->Usages[1], itm->Usages[2], itm->Flags, itm->StackAmount, itm->Weight );
+		printf ( "%2u: %8.8s - (%d %d %d) Fl:0x%x Wt: %d x %dLb\n", i, itm->ItemResRef, itm->Usages[0], itm->Usages[1], itm->Usages[2], itm->Flags, itm->StackAmount, itm->Weight );
 	}
 
 	printf( "Equipped: %d\n", Equipped );
@@ -1347,7 +1484,7 @@ void Inventory::EquipBestWeapon(int flags)
 		for(i=SLOT_RANGED;i<LAST_RANGED;i++) {
 			const Item *itm = GetItemPointer(i, Slot);
 			if (!itm) continue;
- 			//best ranged
+			//best ranged
 			int tmp = itm->GetDamagePotential(true, header);
 			if (tmp>damage) {
 				best_slot = i;
@@ -1359,7 +1496,7 @@ void Inventory::EquipBestWeapon(int flags)
 		}
 
 		//ranged melee weapons like throwing daggers (not bows!)
-		for(i=SLOT_MELEE;i<LAST_MELEE;i++) {
+		for(i=SLOT_MELEE;i<=LAST_MELEE;i++) {
 			const Item *itm = GetItemPointer(i, Slot);
 			if (!itm) continue;
 			//best ranged
@@ -1375,7 +1512,7 @@ void Inventory::EquipBestWeapon(int flags)
 	}
 
 	if (flags&EQUIP_MELEE) {
-		for(i=SLOT_MELEE;i<LAST_MELEE;i++) {
+		for(i=SLOT_MELEE;i<=LAST_MELEE;i++) {
 			const Item *itm = GetItemPointer(i, Slot);
 			if (!itm) continue;
 			//the Slot flag is enough for this
@@ -1446,7 +1583,7 @@ bool Inventory::GetEquipmentInfo(ItemExtHeader *array, int startindex, int count
 				memcpy(array[pos].itemname, slot->ItemResRef, sizeof(ieResRef) );
 				array[pos].slot = idx;
 				array[pos].headerindex = ehc;
- 				int slen = ((char *) &(array[pos].itemname)) -((char *) &(array[pos].AttackType));
+				int slen = ((char *) &(array[pos].itemname)) -((char *) &(array[pos].AttackType));
 				memcpy(&(array[pos].AttackType), &(ext_header->AttackType), slen);
 				if (ext_header->Charges) {
 					//don't modify ehc, it is a counter
@@ -1504,7 +1641,7 @@ void Inventory::UpdateShieldAnimation(Item *it)
 void Inventory::UpdateWeaponAnimation()
 {
 	int slot = GetEquippedSlot();
- 	int effect = core->QuerySlotEffects( slot );
+	int effect = core->QuerySlotEffects( slot );
 	if (effect == SLOT_EFFECT_MISSILE) {
 		// ranged weapon
 		slot = FindRangedWeapon();
@@ -1550,8 +1687,7 @@ void Inventory::UpdateWeaponAnimation()
 	}
 
 	if (header)
-		memcpy(MeleeAnimation,header->MeleeAnimation,
-			 sizeof(MeleeAnimation) );
+		memcpy(MeleeAnimation,header->MeleeAnimation, sizeof(MeleeAnimation) );
 	if (itm)
 		gamedata->FreeItem( itm, Slot->ItemResRef, false );
 	Owner->SetUsedWeapon(AnimationType, MeleeAnimation, WeaponType);
@@ -1658,3 +1794,63 @@ void Inventory::ChargeAllItems(int hours)
 		gamedata->FreeItem( itm, item->ItemResRef, false );
 	}
 }
+
+#define ITM_STEALING (IE_INV_ITEM_UNSTEALABLE | IE_INV_ITEM_MOVABLE | IE_INV_ITEM_EQUIPPED)
+unsigned int Inventory::FindStealableItem()
+{
+	unsigned int slot;
+	int inc;
+
+	slot = core->Roll(1, Slots.size(),-1);
+	inc = slot&1?1:-1;
+
+	printf("Start Slot: %d, increment: %d\n", slot, inc);
+	//as the unsigned value underflows, it will be greater than Slots.size()
+	for(;slot<Slots.size(); slot+=inc) {
+		CREItem *item = Slots[slot];
+		//can't steal empty slot
+		if (!item) continue;
+		//bit 1 is stealable slot
+		if (!(core->QuerySlotFlags(slot)&1) ) continue;
+		//can't steal equipped weapon
+		if ((unsigned int) (Equipped+SLOT_MELEE) == core->QuerySlot(slot)) continue;
+		//can't steal flagged items
+		if ((item->Flags & ITM_STEALING) != IE_INV_ITEM_MOVABLE) continue;
+		return slot;
+	}
+	return 0;
+}
+
+// extension to allow more or less than head gear to avert critical hits:
+// If an item with bit 25 set is equipped in a non-helmet slot, aversion is enabled
+// If an item with bit 25 set is equipped in a helmet slot, aversion is disabled
+bool Inventory::ProvidesCriticalAversion()
+{
+	for (size_t i = 0; i < Slots.size(); i++) {
+		CREItem *item = Slots[i];
+		if (!item || ! (item->Flags & IE_INV_ITEM_EQUIPPED)) {
+			continue;
+		}
+
+		Item *itm = gamedata->GetItem(item->ItemResRef);
+		if (!itm) {
+			continue;
+		}
+
+		for (int h = 0; h < itm->ExtHeaderCount; h++) {
+			ITMExtHeader *header = itm->GetExtHeader(h);
+			if ((int)i == SLOT_HEAD) {
+				if (header && (header->RechargeFlags & IE_ITEM_TOGGLE_CRITS)) {
+					return false;
+				} else {
+					return true;
+				}
+			} else {
+				if (header && (header->RechargeFlags & IE_ITEM_TOGGLE_CRITS)) {
+					return true;
+				}
+			}
+		}
+	}
+	return false;
+}
