@@ -27,14 +27,15 @@
 #include "Ambient.h"
 #include "AmbientMgr.h"
 #include "Audio.h"
-#include "GSUtils.h"
+#include "DisplayMessage.h"
 #include "Game.h"
-#include "GameControl.h"
 #include "GameData.h"
 #include "Interface.h"
 #include "MapMgr.h"
 #include "MusicMgr.h"
+#include "ImageMgr.h"
 #include "Palette.h"
+#include "Particles.h"
 #include "PathFinder.h"
 #include "Projectile.h"
 #include "ScriptedAnimation.h"
@@ -42,6 +43,11 @@
 #include "Video.h"
 #include "WorldMap.h"
 #include "strrefs.h"
+#include "GameScript/GSUtils.h"
+#include "GUI/GameControl.h"
+#include "Scriptable/Container.h"
+#include "Scriptable/Door.h"
+#include "Scriptable/InfoPoint.h"
 
 #include <cmath>
 #include <cassert>
@@ -64,7 +70,8 @@ static Point **VisibilityMasks=NULL;
 static bool PathFinderInited = false;
 static Variables Spawns;
 static int LargeFog;
-static ieWord globalActorCounter;
+static TerrainSounds *terrainsounds=NULL;
+static int tsndcount = -1;
 
 void ReleaseSpawnGroup(void *poi)
 {
@@ -78,11 +85,14 @@ void Map::ReleaseMemory()
 			free(VisibilityMasks[i]);
 		}
 		free(VisibilityMasks);
-		VisibilityMasks=NULL;
+		VisibilityMasks = NULL;
 	}
-
 	Spawns.RemoveAll(ReleaseSpawnGroup);
 	PathFinderInited = false;
+	if (terrainsounds) {
+		delete [] terrainsounds;
+		terrainsounds = NULL;
+	}
 }
 
 inline static AnimationObjectType SelectObject(Actor *actor, int q, AreaAnimation *a, ScriptedAnimation *sca, Particles *spark, Projectile *pro)
@@ -97,7 +107,8 @@ inline static AnimationObjectType SelectObject(Actor *actor, int q, AreaAnimatio
 
 	int aah;
 	if (a) {
-		aah = a->Pos.y;//+a->height;
+		//aah = a->Pos.y;//+a->height;
+		aah = a->GetHeight();
 	} else {
 		aah = 0x7fffffff;
 	}
@@ -185,29 +196,44 @@ void InitSpawnGroups()
 void InitPathFinder()
 {
 	PathFinderInited = true;
+	tsndcount = 0;
 	AutoTable tm("pathfind");
-	if (tm) {
-		const char* poi;
 
-		for (int i = 0; i < 16; i++) {
-			poi = tm->QueryField( 0, i );
-			if (*poi != '*')
-				Passable[i] = atoi( poi );
+	if (!tm) {
+		return;
+	}
+
+	const char* poi;
+
+	for (int i = 0; i < 16; i++) {
+		poi = tm->QueryField( 0, i );
+		if (*poi != '*')
+			Passable[i] = atoi( poi );
+	}
+	poi = tm->QueryField( 1, 0 );
+	if (*poi != '*')
+		NormalCost = atoi( poi );
+	poi = tm->QueryField( 1, 1 );
+	if (*poi != '*')
+		AdditionalCost = atoi( poi );
+	int rc = tm->GetRowCount()-2;
+	if (rc>0) {
+		terrainsounds = new TerrainSounds[rc];
+		tsndcount = rc;
+		while(rc--) {
+			strnuprcpy(terrainsounds[rc].Group,tm->GetRowName(rc+2), sizeof(ieResRef)-1 );
+			for(int i = 0; i<16;i++) {
+				strnuprcpy(terrainsounds[rc].Sounds[i], tm->QueryField(rc+2, i), sizeof(ieResRef)-1 );
+			}
 		}
-		poi = tm->QueryField( 1, 0 );
-		if (*poi != '*')
-			NormalCost = atoi( poi );
-		poi = tm->QueryField( 1, 1 );
-		if (*poi != '*')
-			AdditionalCost = atoi( poi );
 	}
 }
 
 void AddLOS(int destx, int desty, int slot)
 {
 	for (int i=0;i<MaxVisibility;i++) {
-		int x=(destx*i+MaxVisibility/2)/MaxVisibility*16;
-		int y=(desty*i+MaxVisibility/2)/MaxVisibility*12;
+		int x = ((destx*i + MaxVisibility/2) / MaxVisibility) * 16;
+		int y = ((desty*i + MaxVisibility/2) / MaxVisibility) * 12;
 		if (LargeFog) {
 			x += 16;
 			y += 12;
@@ -278,10 +304,10 @@ Map::Map(void)
 	area=this;
 	TMap = NULL;
 	LightMap = NULL;
-	SearchMap = NULL;
 	HeightMap = NULL;
 	SmallMap = NULL;
 	MapSet = NULL;
+	SrchMap = NULL;
 	Walls = NULL;
 	WallCount = 0;
 	queue[PR_SCRIPT] = NULL;
@@ -301,13 +327,13 @@ Map::Map(void)
 		InitPathFinder();
 		InitSpawnGroups();
 		InitExplore();
-		globalActorCounter = 0;
 	}
 	ExploredBitmap = NULL;
 	VisibleBitmap = NULL;
 	version = 0;
-	localActorCounter = 0;
 	MasterArea = core->GetGame()->MasterArea(scriptName);
+	Background = NULL;
+	BgDuration = 0;
 }
 
 Map::~Map(void)
@@ -315,6 +341,7 @@ Map::~Map(void)
 	unsigned int i;
 
 	free( MapSet );
+	free( SrchMap );
 	delete TMap;
 	delete INISpawn;
 	aniIterator aniidx;
@@ -337,7 +364,6 @@ Map::~Map(void)
 		delete spawns[i];
 	}
 	delete LightMap;
-	delete SearchMap;
 	delete HeightMap;
 	core->GetVideoDriver()->FreeSprite( SmallMap );
 	for (i = 0; i < QUEUE_COUNT; i++) {
@@ -398,21 +424,24 @@ void Map::AddTileMap(TileMap* tm, Image* lm, Bitmap* sr, Sprite2D* sm, Bitmap* h
 	// CHECKME: leaks? Should the old TMap, LightMap, etc... be freed?
 	TMap = tm;
 	LightMap = lm;
-	SearchMap = sr;
 	HeightMap = hm;
 	SmallMap = sm;
 	Width = (unsigned int) (TMap->XCellCount * 4);
-	Height = (unsigned int) (( TMap->YCellCount * 64 ) / 12);
+	Height = (unsigned int) (( TMap->YCellCount * 64 + 63) / 12);
 	//Filling Matrices
 	MapSet = (unsigned short *) malloc(sizeof(unsigned short) * Width * Height);
-	//converting searchmap to internal format
-	int y=SearchMap->GetHeight();
+	//Internal Searchmap
+	int y = sr->GetHeight();
+	SrchMap = (unsigned short *) calloc(Width * Height, sizeof(unsigned short));
 	while(y--) {
-		int x=SearchMap->GetWidth();
+		int x=sr->GetWidth();
 		while(x--) {
-			SearchMap->SetAt(x,y,Passable[SearchMap->GetAt(x,y)&PATH_MAP_AREAMASK]);
+			SrchMap[y*Width+x] = Passable[sr->GetAt(x,y)&PATH_MAP_AREAMASK];
 		}
 	}
+
+	//delete the original searchmap
+	delete sr;
 }
 
 void Map::MoveToNewArea(const char *area, const char *entrance, unsigned int direction, int EveryOne, Actor *actor)
@@ -423,10 +452,18 @@ void Map::MoveToNewArea(const char *area, const char *entrance, unsigned int dir
 	//check worldmap entry, if that doesn't contain anything,
 	//make a random pick
 
+	Game* game = core->GetGame();
 	if (EveryOne==CT_WHOLE) {
+		//copy the area name if it exists on the worldmap
+		unsigned int index;
+
+		WMPAreaEntry* entry = core->GetWorldMap()->FindNearestEntry(area, index);
+		if (entry) {
+			memcpy (game->PreviousArea, entry->AreaName, 8);
+		}
+
 		core->GetGameControl()->AutoSave();
 	}
-	Game* game = core->GetGame();
 	Map* map = game->GetMap(area, false);
 	if (!map) {
 		printMessage("Map", " ", LIGHT_RED);
@@ -475,6 +512,7 @@ void Map::MoveToNewArea(const char *area, const char *entrance, unsigned int dir
 		while (i--) {
 			Actor *pc = game->GetPC(i,false);
 			if (pc->GetCurrentArea()==this) {
+				pc->UseExit(0);
 				pc->ClearPath();
 				pc->ClearActions();
 				pc->AddAction( GenerateAction( command ) );
@@ -492,6 +530,7 @@ void Map::MoveToNewArea(const char *area, const char *entrance, unsigned int dir
 				continue;
 			}
 			if (pc->GetCurrentArea()==this) {
+				pc->UseExit(0);
 				pc->ClearPath();
 				pc->ClearActions();
 				pc->AddAction( GenerateAction( command ) );
@@ -514,7 +553,7 @@ void Map::UseExit(Actor *actor, InfoPoint *ip)
 	int EveryOne = ip->CheckTravel(actor);
 	switch(EveryOne) {
 	case CT_GO_CLOSER:
-		core->DisplayConstantString(STR_WHOLEPARTY,0xffffff); //white
+		displaymsg->DisplayConstantString(STR_WHOLEPARTY,0xffffff); //white
 		if (game->EveryoneStopped()) {
 			ip->Flags&=~TRAP_RESET; //exit triggered
 		}
@@ -526,14 +565,14 @@ void Map::UseExit(Actor *actor, InfoPoint *ip)
 		break;
 	}
 
-	actor->UseExit(false);
 	if (ip->Destination[0] != 0) {
-		// 0 here is direction, can infopoints specify that or is an entrance always provided?
+		// the 0 here is default orientation, can infopoints specify that or
+		// is an entrance always provided?
 		MoveToNewArea(ip->Destination, ip->EntranceName, 0, EveryOne, actor);
 		return;
 	}
 	if (ip->Scripts[0]) {
-		ip->LastTriggerObject = ip->LastTrigger = ip->LastEntered = actor->GetID();
+		ip->LastTriggerObject = ip->LastTrigger = ip->LastEntered = actor->GetGlobalID();
 		ip->ExecuteScript( 1 );
 		ip->ProcessActions(true);
 	}
@@ -592,8 +631,9 @@ void Map::UpdateScripts()
 	// possibly wrong (so if you have problems, revert this and find
 	// another way)
 	if (has_pcs) {
-		//Run the Map Script
-		ExecuteScript( 1 );
+		//Run all the Map Scripts (as in the original)
+		//The default area script is in the last slot anyway
+		ExecuteScript( MAX_SCRIPTS );
 	}
 	
 	//Execute Pending Actions
@@ -613,11 +653,6 @@ void Map::UpdateScripts()
 	Actor *timestop_owner = game->timestop_owner;
 	bool timestop = game->timestop_end>game->GameTime;
 
-	// this is silly, the speed should be pre-calculated somewhere
-	//int *actor_speeds = (int *)calloc(Qcount[PR_SCRIPT], sizeof(int));
-
-	//bool *no_more_steps_for_actor = (bool *)calloc(Qcount[PR_SCRIPT], sizeof(bool));
-
 	while (q--) {
 		Actor* actor = queue[PR_SCRIPT][q];
 		//actor just moved away, don't run its script from this side
@@ -636,8 +671,9 @@ void Map::UpdateScripts()
 		actor->fxqueue.Cleanup();
 
 		//if the actor is immobile, don't run the scripts
+		//FIXME: this is not universaly true, only some states have this effect
 		if (!game->StateOverrideFlag && !game->StateOverrideTime) {
-			if (actor->Immobile()) {
+			if (/*actor->Immobile() ||*/ actor->GetStat(IE_STATE_ID) & STATE_SLEEP) {
 				actor->no_more_steps = true;
 				continue;
 			}
@@ -659,6 +695,13 @@ void Map::UpdateScripts()
 
 	}
 
+	//clean up effects on dead actors too
+	q=Qcount[PR_DISPLAY];
+	while(q--) {
+		Actor* actor = queue[PR_DISPLAY][q];
+		actor->fxqueue.Cleanup();
+	}
+
 	q=Qcount[PR_SCRIPT];
 	while (q--) {
 		Actor* actor = queue[PR_SCRIPT][q];
@@ -750,17 +793,15 @@ void Map::UpdateScripts()
 		if (!ip)
 			break;
 		//If this InfoPoint has no script and it is not a Travel Trigger, skip it
-		bool wasActive = (ip->Scripts[0] || ( ip->Type == ST_TRAVEL ));
 		// InfoPoints of all types don't run scripts if TRAP_DEACTIVATED is set
 		// (eg, TriggerActivation changes this, see lightning room from SoA)
- 		if (wasActive)
-			wasActive = !(ip->Flags&TRAP_DEACTIVATED);
+		int wasActive = (!(ip->Flags&TRAP_DEACTIVATED) ) || (ip->Type==ST_TRAVEL);
 
 		//If this InfoPoint is a Switch Trigger
 		if (ip->Type == ST_TRIGGER) {
 			//Check if this InfoPoint was activated
 			if (ip->LastTrigger) {
-				if (wasActive) {
+				if (wasActive && ip->Scripts[0]) {
 					//Run the InfoPoint script
 					ip->ExecuteScript( 1 );
 				}
@@ -776,18 +817,20 @@ void Map::UpdateScripts()
 
 		if (wasActive) {
 			q=Qcount[PR_SCRIPT];
+			ieDword exitID = ip->GetGlobalID();
 			while (q--) {
 				Actor* actor = queue[PR_SCRIPT][q];
 				if (ip->Type == ST_PROXIMITY) {
 					if(ip->Entered(actor)) {
 						//if trap triggered, then mark actor
 						actor->SetInTrap(ipCount);
+						wasActive|=TRAP_USEPOINT;
 					}
 				} else {
 					//ST_TRAVEL
 					//don't move if doing something else
 					// added CurrentAction as part of blocking action fixes
-					if (actor->CannotPassEntrance() ) {
+					if (actor->CannotPassEntrance(exitID) ) {
 						continue;
 					}
 					//this is needed, otherwise the travel
@@ -802,12 +845,26 @@ void Map::UpdateScripts()
 
 		if (wasActive) {
 			ip->ExecuteScript( 1 );
+			//Play the PST specific enter sound
+			if (wasActive&TRAP_USEPOINT) {
+				core->GetAudioDrv()->Play(ip->EnterWav, ip->TrapLaunch.x, ip->TrapLaunch.y);
+			}
 		}
 		//Execute Pending Actions
 		ip->ProcessActions(false);
 	}
 }
 
+void Map::ResolveTerrainSound(ieResRef &sound, Point &Pos) {
+	for(int i=0;i<tsndcount;i++) {
+		if (!memcmp(sound, terrainsounds[i].Group, sizeof(ieResRef) ) ) {
+			int type = GetInternalSearchMap( Pos.x/16, Pos.y/12 )&PATH_MAP_AREAMASK;
+			memcpy(sound, terrainsounds[i].Sounds[type], sizeof(ieResRef) );
+			return;
+		}
+	}  
+}
+
 bool Map::DoStepForActor(Actor *actor, int speed, ieDword time) {
 	bool no_more_steps = true;
 
@@ -835,7 +892,7 @@ bool Map::DoStepForActor(Actor *actor, int speed, ieDword time) {
 }
 
 void Map::ClearSearchMapFor( Movable *actor ) {
-	Actor** nearActors = GetAllActorsInRadius(actor->Pos, GA_NO_DEAD, MAX_CIRCLE_SIZE*2*16);
+	Actor** nearActors = GetAllActorsInRadius(actor->Pos, GA_NO_DEAD|GA_NO_LOS, MAX_CIRCLE_SIZE*2*16);
 	BlockSearchMap( actor->Pos, actor->size, PATH_MAP_FREE);
 
 	// Restore the searchmap areas of any nearby actors that could
@@ -998,20 +1055,33 @@ void Map::DrawMap(Region screen)
 		INISpawn->CheckSpawn();
 	}
 
-	int rain;
-	if (HasWeather()) {
-		//zero when the weather particles are all gone
-		rain = game->weather->GetPhase()-P_EMPTY;
-	} else {
-		rain = 0;
-	}
-	TMap->DrawOverlays( screen, rain );
-
 	//Blit the Background Map Animations (before actors)
 	Video* video = core->GetVideoDriver();
+	int bgoverride = false;
 
-	//Draw Outlines
-	DrawHighlightables( screen );
+	if (Background) {
+		if (BgDuration<gametime) {
+			video->FreeSprite(Background);
+		} else {
+			video->BlitSprite(Background,0,0,true);
+			bgoverride = true;
+		}
+	}
+
+	if (!bgoverride) {
+		int rain;
+		if (HasWeather()) {
+			//zero when the weather particles are all gone
+			rain = game->weather->GetPhase()-P_EMPTY;
+		} else {
+			rain = 0;
+		}
+
+		TMap->DrawOverlays( screen, rain );
+
+		//Draw Outlines
+		DrawHighlightables( screen );
+	}
 
 	Region vp = video->GetViewport();
 	//if it is only here, then the scripting will fail?
@@ -1037,6 +1107,7 @@ void Map::DrawMap(Region screen)
 		switch(SelectObject(actor,q,a,sca,spark,pro)) {
 		case AOT_ACTOR:
 			actor->Draw( screen );
+			actor->UpdateAnimations();
 			actor = GetNextActor(q, index);
 			break;
 		case AOT_AREA:
@@ -1101,7 +1172,7 @@ void Map::DrawMap(Region screen)
 		}
 	}
 
-	if ((core->FogOfWar&FOG_DRAWSEARCHMAP) && SearchMap) {
+	if ((core->FogOfWar&FOG_DRAWSEARCHMAP) && SrchMap) {
 		DrawSearchMap(screen);
 	} else {
 		if ((core->FogOfWar&FOG_DRAWFOG) && TMap) {
@@ -1151,6 +1222,8 @@ void Map::DrawMap(Region screen)
 void Map::DrawSearchMap(const Region &screen)
 {
 	Color inaccessible = { 128, 128, 128, 128 };
+	Color impassible = { 128, 64, 64, 128 }; // red-ish
+	Color sidewall = { 64, 64, 128, 128 }; // blue-ish
 	Video *vid=core->GetVideoDriver();
 	Region rgn=vid->GetViewport();
 	Region block;
@@ -1162,33 +1235,37 @@ void Map::DrawSearchMap(const Region &screen)
 
 	for(int x=0;x<w;x++) {
 		for(int y=0;y<h;y++) {
-			if (!(GetBlocked(x+rgn.x/16, y+rgn.y/12) & PATH_MAP_PASSABLE) ) {
+			unsigned char blockvalue = GetBlocked(x+rgn.x/16, y+rgn.y/12);
+			if (!(blockvalue & PATH_MAP_PASSABLE)) {
 				block.x=screen.x+x*16-(rgn.x % 16);
 				block.y=screen.y+y*12-(rgn.y % 12);
-				vid->DrawRect(block,inaccessible);
+				if (blockvalue == PATH_MAP_IMPASSABLE) { // 0
+					vid->DrawRect(block,impassible);
+				} else if (blockvalue & PATH_MAP_SIDEWALL) {
+					vid->DrawRect(block,sidewall);
+				} else {
+					vid->DrawRect(block,inaccessible);
+				}
 			}
 		}
 	}
 }
 
 //adding animation in order, based on its height parameter
-void Map::AddAnimation(AreaAnimation* anim)
+void Map::AddAnimation(AreaAnimation* panim)
 {
-	//this hack is to make sure animations flagged with background
-	//are always drawn first (-9999 seems sufficiently small)
-	if (anim->Flags&A_ANI_BACKGROUND) {
-		anim->height=-9999;
-	}
+	//copy external memory to core memory for msvc's sake
+	AreaAnimation *anim = new AreaAnimation();
+	memcpy(anim, panim, sizeof(AreaAnimation) );
+
+	anim->InitAnimation();
 
 	aniIterator iter;
-	for(iter=animations.begin(); (iter!=animations.end()) && ((*iter)->height<anim->height); iter++) ;
-	animations.insert(iter, anim);
-	/*
-	Animation *a = anim->animation[0];
-	anim->SetSpriteCover(BuildSpriteCover(anim->Pos.x, anim->Pos.y,-a->animArea.x,
-			-a->animArea.y, a->animArea.w, a->animArea.h,0
-		));
-	*/
+	
+	int Height = anim->GetHeight();
+printf("Adding %s at height %d, Pos: %d.%d\n", anim->Name, Height, anim->Pos.x, anim->Pos.y);
+	for(iter=animations.begin(); (iter!=animations.end()) && ((*iter)->GetHeight()<Height); iter++) ;
+	animations.insert(iter, anim);	
 }
 
 //reapplying all of the effects on the actors of this map
@@ -1213,10 +1290,10 @@ void Map::Shout(Actor* actor, int shoutID, unsigned int radius)
 			}
 		}
 		if (shoutID) {
-			listener->LastHeard = actor->GetID();
+			listener->LastHeard = actor->GetGlobalID();
 			listener->LastShout = shoutID;
 		} else {
-			listener->LastHelp = actor->GetID();
+			listener->LastHelp = actor->GetGlobalID();
 		}
 	}
 }
@@ -1268,9 +1345,7 @@ void Map::AddActor(Actor* actor)
 {
 	//setting the current area for the actor as this one
 	strnlwrcpy(actor->Area, scriptName, 8);
-	//if actor->globalID was already set, don't change it
-	actor->SetMap(this, ++localActorCounter,
-		actor->globalID?actor->globalID:++globalActorCounter);
+	actor->SetMap(this);
 	actors.push_back( actor );
 	//if a visible aggressive actor was put on the map, it is an autopause reason
 	//guess game is always loaded? if not, then we'll crash
@@ -1302,18 +1377,72 @@ bool Map::AnyPCSeesEnemy()
 	return false;
 }
 
+//Make an actor gone for (almost) good
+//If the actor was in the party, it will be moved to the npc storage
+//If the actor is in the NPC storage, its area and some other fields
+//that are needed for proper reentry will be zeroed out
+//If the actor isn't in the NPC storage, it is destructed
 void Map::DeleteActor(int i)
 {
 	Actor *actor = actors[i];
-
-	Game *game = core->GetGame();
-	game->LeaveParty( actor );
-	game->DelNPC( game->InStore(actor) );
-
-	ClearSearchMapFor(actor);
-
+	if (actor) {
+		Game *game = core->GetGame();
+		//this makes sure that a PC will be demoted to NPC
+		game->LeaveParty( actor );
+		//this frees up the spot under the feet circle
+		ClearSearchMapFor( actor );
+		//remove the area reference from the actor
+		actor->SetMap(NULL);
+		//don't destroy the object in case it is a persistent object
+		//otherwise there is a dead reference causing a crash on save
+		if (!game->InStore(actor) ) {
+			delete actor;
+		}
+	}
+	//remove the actor from the area's actor list
 	actors.erase( actors.begin()+i );
-	delete actor;
+}
+
+Door *Map::GetDoorByGlobalID(ieDword objectID)
+{
+	if (!objectID) return NULL;
+
+	int doorCount = 0;
+	while (true) {
+		Door* door = TMap->GetDoor( doorCount++ );
+		if (!door)
+			return NULL;
+		if (door->GetGlobalID() == objectID)
+			return door;
+	}
+}
+
+Container *Map::GetContainerByGlobalID(ieDword objectID)
+{
+	if (!objectID) return NULL;
+
+	int containerCount = 0;
+	while (true) {
+		Container* container = TMap->GetContainer( containerCount++ );
+		if (!container)
+			return NULL;
+		if (container->GetGlobalID() == objectID)
+			return container;
+	}
+}
+
+InfoPoint *Map::GetInfoPointByGlobalID(ieDword objectID)
+{
+	if (!objectID) return NULL;
+
+	int ipCount = 0;
+	while (true) {
+		InfoPoint* ip = TMap->GetInfoPoint( ipCount++ );
+		if (!ip)
+			return NULL;
+		if (ip->GetGlobalID() == objectID)
+			return ip;
+	}
 }
 
 Actor* Map::GetActorByGlobalID(ieDword objectID)
@@ -1321,13 +1450,11 @@ Actor* Map::GetActorByGlobalID(ieDword objectID)
 	if (!objectID) {
 		return NULL;
 	}
-	//truncation is intentional
-	ieWord globalID = (ieWord) objectID;
 	size_t i = actors.size();
 	while (i--) {
 		Actor* actor = actors[i];
 
-		if (actor->globalID==globalID) {
+		if (actor->GetGlobalID()==objectID) {
 			return actor;
 		}
 	}
@@ -1380,6 +1507,7 @@ Actor* Map::GetActorInRadius(const Point &p, int flags, unsigned int radius)
 	return NULL;
 }
 
+//maybe consider using a simple list
 Actor **Map::GetAllActorsInRadius(const Point &p, int flags, unsigned int radius)
 {
 	ieDword count = 1;
@@ -1398,6 +1526,11 @@ Actor **Map::GetAllActorsInRadius(const Point &p, int flags, unsigned int radius
 		if (!actor->Schedule(gametime, true) ) {
 			continue;
 		}
+		if (!(flags&GA_NO_LOS)) {
+			if (!IsVisible(actor->Pos, p)) {
+				continue;
+			}
+		}
 		count++;
 	}
 
@@ -1415,6 +1548,12 @@ Actor **Map::GetAllActorsInRadius(const Point &p, int flags, unsigned int radius
 		if (!actor->Schedule(gametime, true) ) {
 			continue;
 		}
+		if (!(flags&GA_NO_LOS)) {
+			if (!IsVisible(actor->Pos, p)) {
+				continue;
+			}
+		}
+
 		ret[j++]=actor;
 	}
 
@@ -1467,6 +1606,17 @@ void Map::JumpActors(bool jump)
 	}
 }
 
+void Map::SelectActors()
+{
+	size_t i = actors.size();
+	while (i--) {
+		Actor* actor = actors[i];
+		if (actor->Modified[IE_EA]<EA_CONTROLLABLE) {
+			core->GetGame()->SelectActor(actor, true, SELECT_QUIET);
+		}
+	}
+}
+
 //before writing the area out, perform some cleanups
 void Map::PurgeArea(bool items)
 {
@@ -1481,8 +1631,12 @@ void Map::PurgeArea(bool items)
 			if (ac->Modified[IE_MC_FLAGS] & MC_KEEP_CORPSE) {
 				continue;
 			}
-			delete ac;
-			actors.erase( actors.begin()+i );
+			//don't delete persistent actors
+			if (ac->Persistent()) {
+				continue;
+			}
+			//even if you delete it, be very careful!
+			DeleteActor (i);
 		}
 	}
 	//2. remove any non critical items
@@ -1546,6 +1700,18 @@ Actor* Map::GetActorByResource(const char *resref)
 	return NULL;
 }
 
+Actor* Map::GetActorByScriptName(const char *name)
+{
+	size_t i = actors.size();
+	while (i--) {
+		Actor* actor = actors[i];
+		if (strnicmp( actor->GetScriptName(), name, 8 ) == 0) {
+			return actor;
+		}
+	}
+	return NULL;
+}
+
 int Map::GetActorInRect(Actor**& actorlist, Region& rgn, bool onlyparty)
 {
 	actorlist = ( Actor * * ) malloc( actors.size() * sizeof( Actor * ) );
@@ -1557,7 +1723,10 @@ int Map::GetActorInRect(Actor**& actorlist, Region& rgn, bool onlyparty)
 		if (onlyparty && actor->GetStat(IE_EA)>EA_CHARMED) {
 			continue;
 		}
-		if (!actor->ValidTarget(GA_SELECT|GA_NO_DEAD) )
+		// this is called by non-selection code..
+		if (onlyparty && !actor->ValidTarget(GA_SELECT))
+			continue;
+		if (!actor->ValidTarget(GA_NO_DEAD) )
 			continue;
 		if ((actor->Pos.x<rgn.x) || (actor->Pos.y<rgn.y))
 			continue;
@@ -1569,26 +1738,35 @@ int Map::GetActorInRect(Actor**& actorlist, Region& rgn, bool onlyparty)
 	return count;
 }
 
-void Map::PlayAreaSong(int SongType, bool restart)
+void Map::PlayAreaSong(int SongType, bool restart, bool hard)
 {
 	//Ok, we use a non constant pointer here, so it is easy to disable
 	//a faulty music list on the fly. I don't want to add a method just for that
 	//crap when we already have that pointer at hand!
 	char* poi = core->GetMusicPlaylist( SongHeader.SongList[SongType] );
 	if (!poi) return;
+
+	//check if restart needed (either forced or the current song is different)
 	if (!restart && core->GetMusicMgr()->CurrentPlayList(poi)) return;
-	int ret = core->GetMusicMgr()->SwitchPlayList( poi, true );
+	int ret = core->GetMusicMgr()->SwitchPlayList( poi, hard );
 	//Here we disable the faulty musiclist entry
 	if (ret) {
 		//Apparently, the playlist manager prefers a *
 		*poi='*';
+		return;
+	}
+	if (SongType == SONG_BATTLE) {
+		core->GetGame()->CombatCounter = 150;
 	}
 }
 
-unsigned char Map::GetBlocked(unsigned int x, unsigned int y)
+unsigned int Map::GetBlocked(unsigned int x, unsigned int y)
 {
-	unsigned char ret = SearchMap->GetAt( x, y );
-	if (ret&(PATH_MAP_DOOR_TRANSPARENT|PATH_MAP_ACTOR)) {
+	if (y>=Height || x>=Width) {
+		return 0;
+	}
+	unsigned int ret = SrchMap[y*Width+x];
+	if (ret&(PATH_MAP_DOOR_IMPASSABLE|PATH_MAP_ACTOR)) {
 		ret&=~PATH_MAP_PASSABLE;
 	}
 	if (ret&PATH_MAP_DOOR_OPAQUE) {
@@ -1623,7 +1801,7 @@ bool Map::GetBlocked(unsigned int px, unsigned int py, unsigned int size)
 	return false;
 }
 
-unsigned char Map::GetBlocked(const Point &c)
+unsigned int Map::GetBlocked(const Point &c)
 {
 	return GetBlocked(c.x/16, c.y/12);
 }
@@ -1790,12 +1968,12 @@ void Map::SortQueues()
 	}
 }
 
-void Map::AddProjectile(Projectile* pro, const Point &source, ieWord actorID)
+void Map::AddProjectile(Projectile* pro, const Point &source, ieWord actorID, bool fake)
 {
 	proIterator iter;
 
 	pro->MoveTo(this,source);
-	pro->SetTarget(actorID);
+	pro->SetTarget(actorID, fake);
 	int height = pro->GetHeight();
 	for(iter=projectiles.begin();iter!=projectiles.end() && (*iter)->GetHeight()<height; iter++) ;
 	projectiles.insert(iter, pro);
@@ -1826,7 +2004,7 @@ ieDword Map::HasVVCCell(const ieResRef resource, const Point &p)
 			if ((*iter)->YPos!=p.y) continue;
 		}
 		if (strnicmp(resource, (*iter)->ResName, sizeof(ieResRef) )) continue;
-		ieDword tmp = (*iter)->GetSequenceDuration(15)-(*iter)->GetCurrentFrame();
+		ieDword tmp = (*iter)->GetSequenceDuration(AI_UPDATE_TIME)-(*iter)->GetCurrentFrame();
 		if (tmp>ret) {
 			ret = tmp;
 		}
@@ -1914,7 +2092,6 @@ void Map::RemoveActor(Actor* actor)
 	size_t i=actors.size();
 	while (i--) {
 		if (actors[i] == actor) {
-			//BlockSearchMap(actor->Pos, actor->size, PATH_MAP_FREE);
 			ClearSearchMapFor(actor);
 			actors.erase( actors.begin()+i );
 			return;
@@ -1943,7 +2120,19 @@ bool Map::CanFree()
 
 void Map::DebugDump(bool show_actors) const
 {
+	size_t i;
+
 	printf( "DebugDump of Area %s:\n", scriptName );
+	printf ("Scripts:");
+
+	for (i = 0; i < MAX_SCRIPTS; i++) {
+		const char* poi = "<none>";
+		if (Scripts[i]) {
+			poi = Scripts[i]->GetName();
+		}
+		printf( " %.8s", poi );
+	}
+	printf( "Area Global ID:  %d\n", GetGlobalID());
 	printf( "OutDoor: %s\n", YESNO(AreaType & AT_OUTDOOR ) );
 	printf( "Day/Night: %s\n", YESNO(AreaType & AT_DAYNIGHT ) );
 	printf( "Extended night: %s\n", YESNO(AreaType & AT_EXTENDED_NIGHT ) );
@@ -1953,7 +2142,7 @@ void Map::DebugDump(bool show_actors) const
 
 	if (show_actors) {
 		printf("\n");
-		size_t i = actors.size();
+		i = actors.size();
 		while (i--) {
 			if (!(actors[i]->GetInternalFlag()&(IF_JUSTDIED|IF_REALLYDIED))) {
 				printf("Actor: %s at %d.%d\n", actors[i]->GetName(1), actors[i]->Pos.x, actors[i]->Pos.y);
@@ -2361,8 +2550,8 @@ PathNode* Map::FindPathNear(const Point &s, const Point &d, unsigned int size, u
 			 * simply between the specified points
 			 */
 
-			int distx = (x - orig_goal.x)*16;
-			int disty = (y - orig_goal.y)*12;
+			int distx = (x*16 + 8) - d.x;
+			int disty = (y*12 + 6) - d.y;
 			if ((unsigned int)(distx*distx + disty*disty) <= squaredmindistance) {
 				// we are within the minimum distance of the goal
 				Point ourpos(x*16 + 8, y*12 + 6);
@@ -2586,30 +2775,46 @@ bool Map::IsVisible(const Point &s, const Point &d)
 	int dY=d.y/12;
 	int diffx = sX - dX;
 	int diffy = sY - dY;
+
+	// we basically draw a 'line' from (sX, sY) to (dX, dY)
+	// we want to move along the larger axis, to make sure we don't miss anything
 	if (abs( diffx ) >= abs( diffy )) {
-		//vertical
+		// (sX - startX)/elevationy = (sX - startX)/fabs(diffx) * diffy
 		double elevationy = fabs((double)diffx ) / diffy;
 		if (sX > dX) {
-			for (int startx = sX; startx > dX; startx--) {
-				if (GetBlocked( startx, sY - ( int ) ( ( sX - startx ) / elevationy ) ) & PATH_MAP_NO_SEE)
+			// right to left
+			for (int startx = sX; startx >= dX; startx--) {
+				// sX - startx >= 0, so subtract (due to sign of diffy)
+				//if (GetBlocked( startx, sY - ( int ) ( ( sX - startx ) / elevationy ) ) & PATH_MAP_NO_SEE)
+				if (GetBlocked( startx, sY - ( int ) ( ( sX - startx ) / elevationy ) ) & PATH_MAP_SIDEWALL)
 					return false;
 			}
 		} else {
-			for (int startx = sX; startx < dX; startx++) {
-				if (GetBlocked( startx, sY + ( int ) ( ( sX - startx ) / elevationy ) ) & PATH_MAP_NO_SEE)
+			// left to right
+			for (int startx = sX; startx <= dX; startx++) {
+				// sX - startx <= 0, so add (due to sign of diffy)
+				//if (GetBlocked( startx, sY + ( int ) ( ( sX - startx ) / elevationy ) ) & PATH_MAP_NO_SEE)
+				if (GetBlocked( startx, sY + ( int ) ( ( sX - startx ) / elevationy ) ) & PATH_MAP_SIDEWALL)
 					return false;
 			}
 		}
 	} else {
+		// (sY - startY)/elevationx = (sY - startY)/fabs(diffy) * diffx
 		double elevationx = fabs((double)diffy ) / diffx;
 		if (sY > dY) {
-			for (int starty = sY; starty > dY; starty--) {
-				if (GetBlocked( sX - ( int ) ( ( sY - starty ) / elevationx ), starty ) & PATH_MAP_NO_SEE)
+			// bottom to top
+			for (int starty = sY; starty >= dY; starty--) {
+				// sY - starty >= 0, so subtract (due to sign of diffx)
+				//if (GetBlocked( sX - ( int ) ( ( sY - starty ) / elevationx ), starty ) & PATH_MAP_NO_SEE)
+				if (GetBlocked( sX - ( int ) ( ( sY - starty ) / elevationx ), starty ) & PATH_MAP_SIDEWALL)
 					return false;
 			}
 		} else {
-			for (int starty = sY; starty < dY; starty++) {
-				if (GetBlocked( sX + ( int ) ( ( sY - starty ) / elevationx ), starty ) & PATH_MAP_NO_SEE)
+			// top to bottom
+			for (int starty = sY; starty <= dY; starty++) {
+				// sY - starty <= 0, so add (due to sign of diffx)
+				//if (GetBlocked( sX + ( int ) ( ( sY - starty ) / elevationx ), starty ) & PATH_MAP_NO_SEE)
+				if (GetBlocked( sX + ( int ) ( ( sY - starty ) / elevationx ), starty ) & PATH_MAP_SIDEWALL)
 					return false;
 			}
 		}
@@ -2790,7 +2995,7 @@ bool Map::Rest(const Point &pos, int hours, int day)
 			// ensure a minimum power level, since many creatures have this as 0
 			int cpl = creature->Modified[IE_XP] ? creature->Modified[IE_XP] : 1;
 
-			core->DisplayString( RestHeader.Strref[idx], 0x00404000, IE_STR_SOUND );
+			displaymsg->DisplayString( RestHeader.Strref[idx], 0x00404000, IE_STR_SOUND );
 			while (spawnamount > 0 && spawncount <= RestHeader.Maximum) {
 				SpawnCreature(pos, RestHeader.CreResRef[idx], 20);
 				spawnamount -= cpl;
@@ -2824,7 +3029,7 @@ void Map::SetMapVisibility(int setreset)
 	memset( VisibleBitmap, setreset, GetExploredMapSize() );
 }
 
-// x, y are in tile coordinates
+// x, y are not in tile coordinates
 void Map::ExploreTile(const Point &pos)
 {
 	int h = TMap->YCellCount * 2 + LargeFog;
@@ -2887,18 +3092,21 @@ void Map::UpdateFog()
 {
 	if (!(core->FogOfWar&FOG_DRAWFOG) ) {
 		SetMapVisibility( -1 );
-		return;
+		Explore(-1);
+	} else {
+		SetMapVisibility( 0 );
 	}
 
-	SetMapVisibility( 0 );
 	for (unsigned int e = 0; e<actors.size(); e++) {
 		Actor *actor = actors[e];
 		if (!actor->Modified[ IE_EXPLORE ] ) continue;
-		int state = actor->Modified[IE_STATE_ID];
-		if (state & STATE_CANTSEE) continue;
-		int vis2 = actor->Modified[IE_VISUALRANGE];
-		if ((state&STATE_BLIND) || (vis2<2)) vis2=2; //can see only themselves
-		ExploreMapChunk (actor->Pos, vis2, 1);
+		if (core->FogOfWar&FOG_DRAWFOG) {
+			int state = actor->Modified[IE_STATE_ID];
+			if (state & STATE_CANTSEE) continue;
+			int vis2 = actor->Modified[IE_VISUALRANGE];
+			if ((state&STATE_BLIND) || (vis2<2)) vis2=2; //can see only themselves
+			ExploreMapChunk (actor->Pos, vis2+actor->GetAnims()->GetCircleSize(), 1);
+		}
 		Spawn *sp = GetSpawnRadius(actor->Pos, SPAWN_RANGE); //30 * 12
 		if (sp) {
 			TriggerSpawn(sp);
@@ -2926,19 +3134,29 @@ void Map::BlockSearchMap(const Point &Pos, unsigned int size, unsigned int value
 	for (unsigned int i=0; i<size; i++) {
 		for (unsigned int j=0; j<size; j++) {
 			if (i*i+j*j <= r) {
-				unsigned char tmp;
+				unsigned int ppxpi = ppx+i;
+				unsigned int ppypj = ppy+j;
+				unsigned int ppxmi = ppx-i;
+				unsigned int ppymj = ppy-j;
+				if ((ppxpi<Width) && (ppypj<Height)) {
+					unsigned int pos = ppypj*Width+ppxpi;
+					SrchMap[pos] = (SrchMap[pos]&PATH_MAP_NOTACTOR) | value;
+				}
 
-				tmp = SearchMap->GetAt(ppx+i,ppy+j)&PATH_MAP_NOTACTOR;
-				SearchMap->SetAt(ppx+i,ppy+j,tmp|value);
+				if ((ppxpi<Width) && (ppymj<Height)) {
+					unsigned int pos = (ppymj)*Width+ppxpi;
+					SrchMap[pos] = (SrchMap[pos]&PATH_MAP_NOTACTOR) | value;
+				}
 
-				tmp = SearchMap->GetAt(ppx+i,ppy-j)&PATH_MAP_NOTACTOR;
-				SearchMap->SetAt(ppx+i,ppy-j,tmp|value);
+				if ((ppxmi<Width) && (ppypj<Height)) {
+					unsigned int pos = (ppypj)*Width+ppxmi;
+					SrchMap[pos] = (SrchMap[pos]&PATH_MAP_NOTACTOR) | value;
+				}
 
-				tmp = SearchMap->GetAt(ppx-i,ppy+j)&PATH_MAP_NOTACTOR;
-				SearchMap->SetAt(ppx-i,ppy+j,tmp|value);
-
-				tmp = SearchMap->GetAt(ppx-i,ppy-j)&PATH_MAP_NOTACTOR;
-				SearchMap->SetAt(ppx-i,ppy-j,tmp|value);
+				if ((ppxmi<Width) && (ppymj<Height)) {
+					unsigned int pos = (ppymj)*Width+ppxmi;
+					SrchMap[pos] = (SrchMap[pos]&PATH_MAP_NOTACTOR) | value;
+				}
 			}
 		}
 	}
@@ -3007,6 +3225,26 @@ void Map::CopyGroundPiles(Map *othermap, const Point &Pos)
 	}
 }
 
+void Map::MoveVisibleGroundPiles(const Point &Pos)
+{
+	//creating the container at the given position
+	Container *othercontainer;
+	othercontainer = GetPile(Pos);
+
+	int containercount = (int) TMap->GetContainerCount();
+	while (containercount--) {
+		Container * c = TMap->GetContainer( containercount);
+		if (c->Type==IE_CONTAINER_PILE && IsVisible(c->Pos, true)) {
+			//transfer the pile to the other container
+			unsigned int i=c->inventory.GetSlotCount();
+			while (i--) {
+				CREItem *item = c->RemoveItem(i, 0);
+				othercontainer->AddItem(item);
+			}
+		}
+	}
+}
+
 Container *Map::GetPile(Point position)
 {
 	Point tmp[4];
@@ -3112,9 +3350,13 @@ void Map::FadeSparkle(const Point &pos, bool forced)
 	}
 }
 
-void Map::Sparkle(ieDword color, ieDword type, const Point &pos, unsigned int FragAnimID)
+void Map::Sparkle(ieDword duration, ieDword color, ieDword type, const Point &pos, unsigned int FragAnimID, int Zpos)
 {
-	int style, path, grow, size, width;
+	int style, path, grow, size, width, ttl;
+
+	if (!Zpos) {
+		Zpos = 30;
+	}
 
 	//the high word is ignored in the original engine (compatibility hack)
 	switch(type&0xffff) {
@@ -3123,29 +3365,34 @@ void Map::Sparkle(ieDword color, ieDword type, const Point &pos, unsigned int Fr
 		grow = SP_SPAWN_FULL;
 		size = 100;
 		width = 40;
+		ttl = duration;
 		break;
 	case SPARKLE_PUFF:
 		path = SP_PATH_FOUNT; //sparks go up and down
-		grow = SP_SPAWN_FULL;
-		size = 100;
+		grow = SP_SPAWN_SOME;
+		size = 40;
 		width = 40;
+		ttl = core->GetGame()->GameTime+Zpos;
 		break;
 	case SPARKLE_EXPLOSION: //this isn't in the original engine, but it is a nice effect to have
 		path = SP_PATH_EXPL;
-		grow = SP_SPAWN_FULL;
+		grow = SP_SPAWN_SOME;
 		size = 10;
-		width = 140;
+		width = 40;
+		ttl = core->GetGame()->GameTime+Zpos;
 		break;
 	default:
 		path = SP_PATH_FLIT;
 		grow = SP_SPAWN_SOME;
 		size = 100;
 		width = 40;
+		ttl = duration;
 		break;
 	}
 	Particles *sparkles = new Particles(size);
 	sparkles->SetOwner(this);
-	sparkles->SetRegion(pos.x-width/2, pos.y-80, width, 80);
+	sparkles->SetRegion(pos.x-width/2, pos.y-Zpos, width, Zpos);
+	sparkles->SetTimeToLive(ttl);
 
 	if (FragAnimID) {
 		style = SP_TYPE_BITMAP;
@@ -3157,10 +3404,7 @@ void Map::Sparkle(ieDword color, ieDword type, const Point &pos, unsigned int Fr
 	sparkles->SetType(style, path, grow);
 	sparkles->SetColor(color);
 	sparkles->SetPhase(P_GROW);
-	printf("sparkle: %d %d\n", color, type);
-	printf("Position: %d.%d\n", pos.x,pos.y);
 
-	//AddParticle(sparkles, pos);
 	spaIterator iter;
 	for(iter=particles.begin(); (iter!=particles.end()) && ((*iter)->GetHeight()<pos.y); iter++) ;
 	particles.insert(iter, sparkles);
@@ -3194,16 +3438,16 @@ bool Map::DisplayTrackString(Actor *target)
 	int skill = target->GetStat(IE_TRACKING);
 	skill += (target->GetStat(IE_LEVEL)/3)*5 + target->GetStat(IE_WIS)*5;
 	if (core->Roll(1, 100, trackDiff) > skill) {
-		core->DisplayConstantStringName(STR_TRACKINGFAILED, 0xd7d7be, target);
+		displaymsg->DisplayConstantStringName(STR_TRACKINGFAILED, 0xd7d7be, target);
 		return true;
 	}
 	if (trackFlag) {
 			char * str = core->GetString( trackString);
 			core->GetTokenDictionary()->SetAt( "CREATURE", str);
-			core->DisplayConstantStringName(STR_TRACKING, 0xd7d7be, target);
+			displaymsg->DisplayConstantStringName(STR_TRACKING, 0xd7d7be, target);
 			return false;
 	}
-	core->DisplayStringName(trackString, 0xd7d7be, target, 0);
+	displaymsg->DisplayStringName(trackString, 0xd7d7be, target, 0);
 	return false;
 }
 
@@ -3336,7 +3580,7 @@ void AreaAnimation::BlendAnimation()
 	palette->CreateShadedAlphaChannel();
 }
 
-bool AreaAnimation::Schedule(ieDword gametime)
+bool AreaAnimation::Schedule(ieDword gametime) const
 {
 	if (!(Flags&A_ANI_ACTIVE) ) {
 		return false;
@@ -3350,9 +3594,17 @@ bool AreaAnimation::Schedule(ieDword gametime)
 	return false;
 }
 
+int AreaAnimation::GetHeight() const
+{
+	if (Flags&A_ANI_BACKGROUND) return -9999;
+	return Pos.y+height;
+	//FIXME: this is obviously a hack that is destined to crash, also useless, so
+	//See ar9101 in HoW and ar0602 in bg2 before committing anything
+	//return Pos.y+height-animation[0][0].GetFrame(0)->Height;
+}
+
 void AreaAnimation::Draw(const Region &screen, Map *area)
 {
-	int ac=animcount;
 	Video* video = core->GetVideoDriver();
 
 	//always draw the animation tinted because tint is also used for
@@ -3367,7 +3619,8 @@ void AreaAnimation::Draw(const Region &screen, Map *area)
 			covers=(SpriteCover **) calloc( animcount, sizeof(SpriteCover *) );
 		}
 	}
-	ac=animcount;
+
+	int ac = animcount;
 	while (ac--) {
 		Animation *anim = animation[ac];
 		Sprite2D *frame = anim->NextFrame();
@@ -3384,10 +3637,12 @@ void AreaAnimation::Draw(const Region &screen, Map *area)
 }
 
 //change the tileset if needed and possible, return true if changed
+//day_or_night = 1 means the normal day lightmap
 bool Map::ChangeMap(bool day_or_night)
 {
 	//no need of change if the area is not extended night
-	if (! (AreaType&AT_EXTENDED_NIGHT)) return false;
+	//if (((AreaType&(AT_DAYNIGHT|AT_EXTENDED_NIGHT))!=(AT_DAYNIGHT|AT_EXTENDED_NIGHT))) return false;
+	if (!(AreaType&AT_EXTENDED_NIGHT)) return false;
 	//no need of change if the area already has the right tilemap
 	if ((DayNight == day_or_night) && GetTileMap()) return false;
 
@@ -3395,7 +3650,11 @@ bool Map::ChangeMap(bool day_or_night)
 	//no need to open and read the .are file again
 	//using the ARE class for this because ChangeMap is similar to LoadMap
 	//it loads the lightmap and the minimap too, besides swapping the tileset
-	mM->ChangeMap(this, day_or_night);
+	if (!mM->ChangeMap(this, day_or_night) && !day_or_night) {
+		printMessage("Map", "Invalid night lightmap, falling back to day lightmap.\n", YELLOW);
+		mM->ChangeMap(this, 1);
+		DayNight = day_or_night;
+	}
 	return true;
 }
 
@@ -3405,7 +3664,7 @@ void Map::SeeSpellCast(Scriptable *caster, ieDword spell)
 		return;
 	}
 
-	LastCasterSeen = ((Actor *) caster)->GetID();
+	LastCasterSeen = caster->GetGlobalID();
 	LastSpellSeen = spell;
 
 	size_t i = actors.size();
@@ -3417,3 +3676,29 @@ void Map::SeeSpellCast(Scriptable *caster, ieDword spell)
 		}
 	}
 }
+
+short unsigned int Map::GetInternalSearchMap(int x, int y) {
+	if ((unsigned)x >= Width || (unsigned)y >= Height) {
+		return 0;
+	}
+	return SrchMap[x+y*Width];
+}
+
+void Map::SetInternalSearchMap(int x, int y, int value) {
+	if ((unsigned)x >= Width || (unsigned)y >= Height) {
+		return;
+	}
+	SrchMap[x+y*Width] = value;
+}
+
+void Map::SetBackground(const ieResRef &bgResRef, ieDword duration) {
+	Video* video = core->GetVideoDriver();
+
+	ResourceHolder<ImageMgr> bmp(bgResRef);
+
+	if (Background) {
+		video->FreeSprite(Background);
+	}
+	Background = bmp->GetSprite2D();
+	BgDuration = duration;
+}
