@@ -24,11 +24,12 @@
 
 #include "win32def.h"
 
-#include "Actor.h"
+#include "Audio.h"
 #include "Game.h"
 #include "Interface.h"
 #include "Projectile.h"
 #include "ProjectileServer.h"
+#include "Scriptable/Actor.h"
 
 SPLExtHeader::SPLExtHeader(void)
 {
@@ -72,11 +73,43 @@ int Spell::GetHeaderIndexFromLevel(int level) const
 //-1 will return cfb
 //0 will always return first spell block
 //otherwise set to caster level
-static EffectRef fx_casting_glow_ref={"CastingGlow",NULL,-1};
+static EffectRef fx_casting_glow_ref = { "CastingGlow", -1 };
 
-void Spell::AddCastingGlow(EffectQueue *fxqueue, ieDword duration)
+void Spell::AddCastingGlow(EffectQueue *fxqueue, ieDword duration, int gender)
 {
-	Effect *fx = EffectQueue::CreateEffect(fx_casting_glow_ref, 0, CastingGraphics, FX_DURATION_ABSOLUTE);
+	char g, t;
+	Effect *fx;
+	ieResRef Resource;
+	
+	int cgsound = CastingSound;
+	if (cgsound>=0 && duration > 1) {
+		//bg2 style
+		if(cgsound&0x100) {    
+			switch(gender) {
+			default: g = 'm'; break;
+			case SEX_FEMALE: g = 'f'; break;
+			case SEX_OTHER: case SEX_NEITHER: g = 's'; break;
+			}
+		} else {
+			//how style
+			
+			switch(gender) {
+			default: g = 'm'; break;
+			case SEX_FEMALE: g = 'f'; break;
+			}         
+		}
+		if (SpellType==IE_SPL_PRIEST) {
+			t = 'p';
+		} else {
+			t = 'm';
+		}
+		snprintf(Resource, 9,"CHA_%c%c%02d", g, t, cgsound&0xff);
+		// only actors have fxqueue's and also the parent function checks for that
+		Actor *caster = (Actor *) fxqueue->GetOwner();
+		caster->casting_sound = core->GetAudioDrv()->Play(Resource, caster->Pos.x, caster->Pos.y);
+	}
+
+	fx = EffectQueue::CreateEffect(fx_casting_glow_ref, 0, CastingGraphics, FX_DURATION_ABSOLUTE);
 	fx->Duration = core->GetGame()->GameTime + duration;
 	fx->InventorySlot = 0xffff;
 	fx->Projectile = 0;
@@ -85,7 +118,7 @@ void Spell::AddCastingGlow(EffectQueue *fxqueue, ieDword duration)
 	delete fx;
 }
 
-EffectQueue *Spell::GetEffectBlock(Scriptable *self, const Point &pos, int block_index, ieDword pro) const
+EffectQueue *Spell::GetEffectBlock(Scriptable *self, const Point &pos, int block_index, int level, ieDword pro) const
 {
 	Effect *features;
 	int count;
@@ -104,35 +137,57 @@ EffectQueue *Spell::GetEffectBlock(Scriptable *self, const Point &pos, int block
 		count = CastingFeatureCount;
 	}
 	EffectQueue *fxqueue = new EffectQueue();
+	EffectQueue *selfqueue = NULL;
 
 	for (int i=0;i<count;i++) {
-		if (Flags & SF_SIMPLIFIED_DURATION) {
+		Effect *fx = features+i;
+
+		if ((Flags & SF_SIMPLIFIED_DURATION) && (block_index>=0)) {
 			//hack the effect according to Level
 			//fxqueue->AddEffect will copy the effect,
 			//so we don't risk any overwriting
 			if (EffectQueue::HasDuration(features+i)) {
-				features[i].Duration = (TimePerLevel*block_index+TimeConstant)*7;
+				fx->Duration = (TimePerLevel*block_index+TimeConstant)*core->Time.round_sec;
 			}
 		}
 		//fill these for completeness, inventoryslot is a good way
 		//to discern a spell from an item effect
-		features[i].InventorySlot = 0xffff;
-		if (features[i].Target != FX_TARGET_SELF) {
-			features[i].Projectile = pro;
-			fxqueue->AddEffect( features+i );
-		} else {
-			Actor *target = (self->Type==ST_ACTOR)?(Actor *) self:NULL;
-			features[i].Projectile = 0;
-			features[i].PosX=pos.x;
-			features[i].PosY=pos.y;
-			//FIXME (r7193):
-			//This is bad, effects should be able to affect non living targets
-			//This is done by NULL target, the position should be enough
-			//to tell which non-actor object is affected
-			if (target) {
-				core->ApplyEffect(features+i, target, self);
+
+		fx->InventorySlot = 0xffff;
+		//the hostile flag is used to determine if this was an attack
+		fx->SourceFlags = Flags;
+		fx->CasterLevel = level;
+
+		// apply the stat-based spell duration modifier
+		if (self->Type == ST_ACTOR) {
+			Actor *caster = (Actor *) self;
+			if (caster->Modified[IE_SPELLDURATIONMODMAGE] && SpellType == IE_SPL_WIZARD) {
+				fx->Duration = (fx->Duration * caster->Modified[IE_SPELLDURATIONMODMAGE]) / 100;
+			} else if (caster->Modified[IE_SPELLDURATIONMODPRIEST] && SpellType == IE_SPL_PRIEST) {
+				fx->Duration = (fx->Duration * caster->Modified[IE_SPELLDURATIONMODPRIEST]) / 100;
 			}
 		}
+
+		if (fx->Target != FX_TARGET_SELF) {
+			fx->Projectile = pro;
+			fxqueue->AddEffect( fx );
+		} else {
+			fx->Projectile = 0;
+			fx->PosX=pos.x;
+			fx->PosY=pos.y;
+			if (!selfqueue) {
+				selfqueue = new EffectQueue();
+			}
+			// effects should be able to affect non living targets
+			//This is done by NULL target, the position should be enough
+			//to tell which non-actor object is affected
+			selfqueue->AddEffect( fx );
+		}
+	}
+	if (selfqueue) {
+		Actor *target = (self->Type==ST_ACTOR)?(Actor *) self:NULL;
+		core->ApplyEffectQueue(selfqueue, target, self);
+		delete selfqueue;
 	}
 	return fxqueue;
 }
@@ -155,21 +210,18 @@ Projectile *Spell::GetProjectile(Scriptable *self, int header, const Point &targ
 //get the casting distance of the spell
 //it depends on the casting level of the actor
 //if actor isn't given, then the first header is used
-//TODO: fix casting level for all class combos
-unsigned int Spell::GetCastingDistance(Actor *actor) const
+unsigned int Spell::GetCastingDistance(Scriptable *Sender) const
 {
-	int level = 1;
-	if(actor) {
-		level = actor->GetStat(IE_LEVEL);
-		if(SpellType==IE_SPL_WIZARD) {
-			level+=actor->GetStat(IE_CASTINGLEVELBONUSMAGE);
-		}
-		else if(SpellType==IE_SPL_PRIEST) {
-			level+=actor->GetStat(IE_CASTINGLEVELBONUSCLERIC);
-		}
+	int level = 0;
+	Actor *actor = NULL;
+	if (Sender && Sender->Type==ST_ACTOR) {
+		actor = (Actor *) Sender;
+		level = actor->GetCasterLevel(SpellType);
 	}
 
-	if(level<1) level=1;
+	if (level<1) {
+		level = 1;
+	}
 	int idx = GetHeaderIndexFromLevel(level);
 	SPLExtHeader *seh = GetExtHeader(idx);
 	if (!seh) {
@@ -177,5 +229,9 @@ unsigned int Spell::GetCastingDistance(Actor *actor) const
 		printf("required header: %d, maximum: %d\n", idx, (int) ExtHeaderCount);
 		return 0;
 	}
+
+	if (seh->Target==TARGET_DEAD) {
+		return 0xffffffff;
+	}
 	return (unsigned int) seh->Range;
 }
