@@ -30,10 +30,10 @@
 #include "globals.h"
 
 #include "Animation.h"
-#include "EventMgr.h"
 #include "Plugin.h"
 #include "Polygon.h"
 #include "ScriptedAnimation.h"
+#include "GUI/EventMgr.h"
 
 class AnimationFactory;
 class Palette;
@@ -49,8 +49,8 @@ enum SpriteBlitFlags {
 	BLIT_NOSHADOW = 0x1000,
 	BLIT_TRANSSHADOW = 0x2000,
 	BLIT_TINTED = 0x00010000, // IE_VVC_TINT = 0x00030000
-	BLIT_GREY = IE_VVC_GREYSCALE, // 0x80000
-	BLIT_RED = IE_VVC_RED_TINT, // 0x02000000
+	BLIT_GREY = IE_VVC_GREYSCALE, // 0x80000; timestop palette
+	BLIT_RED = IE_VVC_SEPIA, // 0x02000000; dream scene palette
 	BLIT_DARK = IE_VVC_DARKEN, // 0x00100000; not implemented in SDLVideo yet
 	BLIT_GLOW = IE_VVC_GLOWING // 0x00200000; not implemented in SDLVideo yet
 	// Note: bits 29,30,31 are used by SDLVideo internally
@@ -79,8 +79,6 @@ public:
 	virtual int CreateDisplay(int width, int height, int bpp, bool fullscreen) = 0;
 	/** Sets window title of GemRB window */
 	virtual void SetDisplayTitle(char* title, char* icon) = 0;
-	virtual VideoModes GetVideoModes(bool fullscreen = false) = 0;
-	virtual bool TestVideoMode(VideoMode& vm) = 0;
 	/** Toggles GemRB between fullscreen and windowed mode.
 	  * 0 = windowed, 1 = fullscreen, -1 (default) = toggle */
 	virtual bool ToggleFullscreenMode(int set_reset=-1) = 0;
@@ -203,6 +201,7 @@ public:
 		unsigned int w, unsigned int h,
 		unsigned int dstx, unsigned int dsty,
 		ieDword titleref) = 0;
+	virtual void DrawMovieSubtitle(ieStrRef text) = 0;
 	/** handles events during movie */
 	virtual int PollMovieEvents() = 0;
 	virtual void SetGamma(int brightness, int contrast) = 0;
@@ -211,6 +210,7 @@ public:
 
 	void SetMouseEnabled(int enabled);
 	void SetMouseGrayed(bool grayed);
+	bool GetFullscreenMode() const;
 
 	/** Scales down a sprite by a ratio */
 	Sprite2D* SpriteScaleDown( const Sprite2D* sprite, unsigned int ratio );
@@ -219,7 +219,7 @@ public:
 	Sprite2D* CreateLight(int radius, int intensity);
 
 	Color SpriteGetPixelSum (const Sprite2D* sprite, unsigned short xbase, unsigned short ybase, unsigned int ratio);
-	Region GetViewport(void);
+	Region GetViewport(void) const;
 	void SetViewport(int x, int y, unsigned int w, unsigned int h);
 	void MoveViewportTo(int x, int y);
 protected:
