@@ -28,8 +28,8 @@
 #define WORLDMAPMGR_H
 
 #include "Plugin.h"
-#include "DataStream.h"
 #include "WorldMap.h"
+#include "System/DataStream.h"
 
 /**
  * @class WorldMapMgr
@@ -40,11 +40,11 @@ class GEM_EXPORT WorldMapMgr : public Plugin {
 public:
 	WorldMapMgr(void);
 	virtual ~WorldMapMgr(void);
-	virtual bool Open(DataStream* stream, bool autoFree = true) = 0;
+	virtual bool Open(DataStream* stream1, DataStream* stream2, bool autoFree = true) = 0;
 	virtual WorldMapArray* GetWorldMapArray() = 0;
 
-	virtual int GetStoredFileSize(WorldMapArray *wmap) = 0;
-	virtual int PutWorldMap(DataStream* stream, WorldMapArray *wmap) = 0;
+	virtual int GetStoredFileSize(WorldMapArray *wmap, unsigned int index) = 0;
+	virtual int PutWorldMap(DataStream* stream1, DataStream* stream2, WorldMapArray *wmap) = 0;
 };
 
 #endif
