@@ -25,8 +25,8 @@
 
 #include "ActorMgr.h"
 #include "Ambient.h"
-#include "CachedFileStream.h"
 #include "DataFileMgr.h"
+#include "DisplayMessage.h"
 #include "EffectMgr.h"
 #include "Game.h"
 #include "GameData.h"
@@ -36,6 +36,12 @@
 #include "ProjectileServer.h"
 #include "TileMapMgr.h"
 #include "Video.h"
+#include "GameScript/GameScript.h"
+#include "Scriptable/Container.h"
+#include "Scriptable/Door.h"
+#include "Scriptable/InfoPoint.h"
+#include "System/CachedFileStream.h"
+#include "System/FileStream.h"
 
 #define DEF_OPEN   0
 #define DEF_CLOSE  1
@@ -85,7 +91,7 @@ void ReadAutonoteINI()
 int GetTrackString(const ieResRef areaName)
 {
 	int i;
-	bool trackflag = core->HasStringReference(STR_TRACKING);
+	bool trackflag = displaymsg->HasStringReference(STR_TRACKING);
 
 	if (!tracks) {
 		AutoTable tm("tracking");
@@ -316,7 +322,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 
 	if (!core->IsAvailable( IE_WED_CLASS_ID )) {
 		printf( "[AREImporter]: No Tile Map Manager Available.\n" );
-		return false;
+		return NULL;
 	}
 	ieResRef TmpResRef;
 
@@ -334,17 +340,24 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	TileMap* tm = tmm->GetTileMap(NULL);
 	if (!tm) {
 		printf( "[AREImporter]: No Tile Map Available.\n" );
-		return false;
+		return NULL;
 	}
 
 	// Small map for MapControl
-	ResourceHolder<ImageMgr> sm(TmpResRef);
 	// small map is *optional*!
+	ResourceHolder<ImageMgr> sm(TmpResRef);
+
+	//if the Script field is empty, the area name will be copied into it on first load
+	//this works only in the iwd branch of the games
+	if (!Script[0] && core->HasFeature(GF_FORCE_AREA_SCRIPT) ) {
+		memcpy(Script, ResRef, sizeof(ieResRef) );
+	}
 
 	if (Script[0]) {
-		map->Scripts[0] = new GameScript( Script, map );
-	} else {
-		map->Scripts[0] = NULL;
+		//for some reason the area's script is run from the last slot
+		//at least one area script depends on this, if you need something
+		//more customisable, add a game flag
+		map->Scripts[MAX_SCRIPTS-1] = new GameScript( Script, map );
 	}
 
 	if (day_or_night) {
@@ -356,7 +369,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	ResourceHolder<ImageMgr> lm(TmpResRef);
 	if (!lm) {
 		printf( "[AREImporter]: No lightmap available.\n" );
-		return false;
+		return NULL;
 	}
 
 	snprintf( TmpResRef, 9, "%sSR", WEDResRef);
@@ -364,7 +377,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	ResourceHolder<ImageMgr> sr(TmpResRef);
 	if (!sr) {
 		printf( "[AREImporter]: No searchmap available.\n" );
-		return false;
+		return NULL;
 	}
 
 	snprintf( TmpResRef, 9, "%sHT", WEDResRef);
@@ -372,7 +385,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	ResourceHolder<ImageMgr> hm(TmpResRef);
 	if (!hm) {
 		printf( "[AREImporter]: No heightmap available.\n" );
-		return false;
+		return NULL;
 	}
 
 	map->AddTileMap( tm, lm->GetImage(), sr->GetBitmap(), sm ? sm->GetSprite2D() : NULL, hm->GetBitmap() );
@@ -403,6 +416,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	str->ReadWord( &map->RestHeader.NightChance );
 
 	printf( "Loading regions\n" );
+	core->LoadProgress(70);
 	//Loading InfoPoints
 	for (i = 0; i < InfoPointsCount; i++) {
 		str->Seek( InfoPointsOffset + ( i * 0xC4 ), GEM_STREAM_START );
@@ -413,7 +427,8 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		ieWord PosX, PosY;
 		ieWord TalkX, TalkY;
 		ieVariable Name, Entrance;
-		ieResRef Script, DialogResRef, KeyResRef, Destination;
+		ieResRef Script, KeyResRef, Destination;
+		ieResRef DialogResRef, WavResRef; //adopted pst specific fields
 		ieStrRef DialogName;
 		str->Read( Name, 32 );
 		Name[32] = 0;
@@ -450,7 +465,8 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		str->ReadWord( &PosX);
 		str->ReadWord( &PosY);
 		//maybe we have to store this
-		str->Seek( 44, GEM_CURRENT_POS );
+		str->Seek( 36, GEM_CURRENT_POS );
+		str->ReadResRef( WavResRef );
 		str->ReadWord( &TalkX);
 		str->ReadWord( &TalkY);
 		str->ReadDword( &DialogName );
@@ -498,6 +514,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		ip->TalkPos.y=TalkY;
 		ip->DialogName=DialogName;
 		ip->SetDialog(DialogResRef);
+		ip->SetEnter(WavResRef);
 
 		if (Script[0]) {
 			ip->Scripts[0] = new GameScript( Script, ip );
@@ -597,6 +614,8 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 			//cannot add directly to inventory (ground piles)
 			c->AddItem( core->ReadItem(str));
 		}
+		//update item flags (like movable flag)
+		c->inventory.CalculateWeight();
 
 		if (Type==IE_CONTAINER_PILE)
 			Script[0]=0;
@@ -607,6 +626,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 			c->Scripts[0] = NULL;
 		}
 		strnlwrcpy(c->KeyResRef, KeyResRef, 8);
+		if (!OpenFail) OpenFail = (ieStrRef)-1; // rewrite 0 to -1
 		c->OpenFail = OpenFail;
 	}
 
@@ -628,7 +648,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		ieWord TrapDetect, TrapRemoval;
 		ieWord Trapped, TrapDetected;
 		ieWord LaunchX, LaunchY;
-		ieDword Locked, LockRemoval;
+		ieDword DiscoveryDiff, LockRemoval;
 		Region BBClosed, BBOpen;
 		ieStrRef OpenStrRef;
 		ieStrRef NameStrRef;
@@ -676,7 +696,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		str->ReadWord( &LaunchY );
 		str->ReadResRef( KeyResRef );
 		str->ReadResRef( Script );
-		str->ReadDword( &Locked );
+		str->ReadDword( &DiscoveryDiff );
 		str->ReadDword( &LockRemoval );
 		Point toOpen[2];
 		str->ReadWord( &minX );
@@ -803,7 +823,9 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 			else
 				memcpy( door->CloseSound, Sounds[DEF_CLOSE], 9 );
 		}
+		door->DiscoveryDiff=DiscoveryDiff;
 		door->LockDifficulty=LockRemoval;
+		if (!OpenStrRef) OpenStrRef = (ieStrRef)-1; // rewrite 0 to -1
 		door->OpenStrRef=OpenStrRef;
 		strnspccpy(door->LinkedInfo, LinkedInfo, 32);
 		//these 2 fields are not sure
@@ -865,6 +887,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		//the rest is not read, we seek for every record
 	}
 
+	core->LoadProgress(75);
 	printf( "Loading actors\n" );
 	//Loading Actors
 	str->Seek( ActorOffset, GEM_STREAM_START );
@@ -960,6 +983,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		}
 	}
 
+	core->LoadProgress(90);
 	printf( "Loading animations\n" );
 	//Loading Animations
 	str->Seek( AnimOffset, GEM_STREAM_START );
@@ -979,7 +1003,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 			str->ReadWord( &anim->sequence );
 			str->ReadWord( &anim->frame );
 			str->ReadDword( &anim->Flags );
-			str->ReadWord( (ieWord *) &anim->height );
+			str->ReadWordSigned( &anim->height );
 			str->ReadWord( &anim->transparency );
 			str->ReadWord( &anim->unknown3c ); //not completely understood, if not 0, sequence is started
 			str->Read( &anim->startchance,1 );
@@ -993,9 +1017,9 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 			//set up the animation, it cannot be done here
 			//because a StaticSequence action can change
 			//it later
-			anim->InitAnimation();
-
 			map->AddAnimation( anim );
+			//the animation was safely transferred to internal memory
+			delete anim;
 		}
 	}
 
@@ -1150,9 +1174,9 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		ieDword TrapEffOffset;
 		ieWord TrapSize, ProID;
 		ieWord X,Y;
-		ieDword Unknown1;
-		ieWord Unknown2;
-		ieByte Unknown3;
+		ieDword Ticks;
+		ieWord Unknown;
+		ieByte PartyID;
 		ieByte Owner;
 
 		str->Seek( TrapOffset + ( i * 0x1c ), GEM_STREAM_START );
@@ -1161,11 +1185,11 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		str->ReadDword( &TrapEffOffset );
 		str->ReadWord( &TrapSize );
 		str->ReadWord( &ProID );
-		str->ReadDword( &Unknown1 );
+		str->ReadDword( &Ticks );
 		str->ReadWord( &X );
 		str->ReadWord( &Y );
-		str->ReadWord( &Unknown2 );
-		str->Read( &Unknown3,1 );
+		str->ReadWord( &Unknown );
+		str->Read( &PartyID,1 );
 		str->Read( &Owner,1 );
 		int TrapEffectCount = TrapSize/0x108;
 		if(TrapEffectCount*0x108!=TrapSize) {
@@ -1184,9 +1208,12 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		CachedFileStream *fs = new CachedFileStream( (CachedFileStream *) str, TrapEffOffset, TrapSize, true);
 
 		ReadEffects((DataStream *) fs,fxqueue, TrapEffectCount);
-		Actor * caster = core->GetGame()->FindPC(Owner);
+		Actor * caster = core->GetGame()->FindPC(PartyID);
 		pro->SetEffects(fxqueue);
-		if (caster) pro->SetCaster(caster->GetGlobalID());
+		if (caster) {
+			//FIXME: i don't know the level info
+			pro->SetCaster(caster->GetGlobalID(), 10);
+		}
 		Point pos(X,Y);
 		map->AddProjectile( pro, pos, pos);
 	}
@@ -1423,7 +1450,9 @@ int AREImporter::PutHeader(DataStream *stream, Map *map)
 	stream->WriteDword( &VariablesOffset );
 	stream->WriteDword( &VariablesCount );
 	stream->WriteDword( &tmpDword);
-	GameScript *s = map->Scripts[0];
+
+	//the saved area script is in the last script slot!
+	GameScript *s = map->Scripts[MAX_SCRIPTS-1];
 	if (s) {
 		stream->WriteResRef( s->GetName() );
 	} else {
@@ -1462,7 +1491,6 @@ int AREImporter::PutHeader(DataStream *stream, Map *map)
 int AREImporter::PutDoors( DataStream *stream, Map *map, ieDword &VertIndex)
 {
 	char filling[8];
-	ieDword tmpDword = 0;
 	ieWord tmpWord = 0;
 
 	memset(filling,0,sizeof(filling) );
@@ -1526,9 +1554,7 @@ int AREImporter::PutDoors( DataStream *stream, Map *map, ieDword &VertIndex)
 		} else {
 			stream->Write( filling, 8);
 		}
-		//unknown field 0-100
-		//stream->WriteDword( &d->Locked);
-		stream->WriteDword( &tmpDword);
+		stream->WriteDword( &d->DiscoveryDiff);
 		//lock difficulty field
 		stream->WriteDword( &d->LockDifficulty);
 		//opening locations
@@ -1682,7 +1708,7 @@ int AREImporter::PutRegions( DataStream *stream, Map *map, ieDword &VertIndex)
 {
 	ieDword tmpDword = 0;
 	ieWord tmpWord;
-	char filling[56];
+	char filling[36];
 
 	memset(filling,0,sizeof(filling) );
 	for (unsigned int i=0;i<InfoPointsCount;i++) {
@@ -1732,8 +1758,9 @@ int AREImporter::PutRegions( DataStream *stream, Map *map, ieDword &VertIndex)
 		stream->WriteWord( &tmpWord);
 		tmpWord = (ieWord) ip->UsePoint.y;
 		stream->WriteWord( &tmpWord);
-		stream->Write( filling, 44); //unknown
+		stream->Write( filling, 36); //unknown
 		//these are probably only in PST
+		stream->WriteResRef( ip->EnterWav);
 		tmpWord = (ieWord) ip->TalkPos.x;
 		stream->WriteWord( &tmpWord);
 		tmpWord = (ieWord) ip->TalkPos.y;
@@ -1850,12 +1877,14 @@ int AREImporter::PutActors( DataStream *stream, Map *map)
 
 	CreatureOffset = EmbeddedCreOffset;
 	for (i=0;i<ActorCount;i++) {
+		assert(stream->GetPos() == CreatureOffset);
 		Actor *ac = map->GetActor(i, false);
 
 		//reconstructing offsets again
-		am->GetStoredFileSize(ac);
+		CreatureOffset += am->GetStoredFileSize(ac);
 		am->PutActor( stream, ac);
 	}
+	assert(stream->GetPos() == CreatureOffset);
 
 	return 0;
 }
@@ -2024,10 +2053,9 @@ int AREImporter::PutMapnotes( DataStream *stream, Map *map)
 
 int AREImporter::PutEffects( DataStream *stream, EffectQueue *fxqueue)
 {
-	ieDword tmpDword1,tmpDword2;
-	char filling[60];
+	PluginHolder<EffectMgr> eM(IE_EFF_CLASS_ID);
+	assert(eM != NULL);
 
-	memset(filling,0,sizeof(filling) );
 	std::list< Effect* >::const_iterator f=fxqueue->GetFirstEffect();
 	ieDword EffectsCount = fxqueue->GetSavedEffectsCount();
 	for(unsigned int i=0;i<EffectsCount;i++) {
@@ -2035,54 +2063,7 @@ int AREImporter::PutEffects( DataStream *stream, EffectQueue *fxqueue)
 
 		assert(fx!=NULL);
 
-		stream->Write( filling,8 ); //signature
-		stream->WriteDword( &fx->Opcode);
-		stream->WriteDword( &fx->Target);
-		stream->WriteDword( &fx->Power);
-		stream->WriteDword( &fx->Parameter1);
-		stream->WriteDword( &fx->Parameter2);
-		stream->WriteWord( &fx->TimingMode);
-		stream->WriteWord( &fx->unknown2);
-		stream->WriteDword( &fx->Duration);
-		stream->WriteWord( &fx->Probability1);
-		stream->WriteWord( &fx->Probability2);
-		stream->WriteResRef(fx->Resource);
-		stream->WriteDword( &fx->DiceThrown );
-		stream->WriteDword( &fx->DiceSides );
-		stream->WriteDword( &fx->SavingThrowType );
-		stream->WriteDword( &fx->SavingThrowBonus );
-		//isvariable
-		stream->Write( filling,4 );
-		stream->WriteDword( &fx->PrimaryType );
-		stream->Write( filling,12 );
-		stream->WriteDword( &fx->Resistance );
-		stream->WriteDword( &fx->Parameter3 );
-		stream->WriteDword( &fx->Parameter4 );
-		stream->Write( filling,8 );
-		if (fx->IsVariable) {
-			stream->Write(fx->Resource+8, 8);
-			//resource1-4 are used as a continuous memory
-			stream->Write(((ieByte *) fx->Resource)+16, 8);
-		} else {
-			stream->WriteResRef(fx->Resource2);
-			stream->WriteResRef(fx->Resource3);
-		}
-		tmpDword1 = (ieDword) fx->PosX;
-		tmpDword2 = (ieDword) fx->PosY;
-		stream->WriteDword( &tmpDword1 );
-		stream->WriteDword( &tmpDword2 );
-		//FIXME: these two points are actually different
-		stream->WriteDword( &tmpDword1 );
-		stream->WriteDword( &tmpDword2 );
-		stream->WriteDword( &fx->SourceType );
-		stream->WriteResRef( fx->Source );
-		stream->WriteDword( &fx->SourceFlags );
-		stream->WriteDword( &fx->Projectile );
-		tmpDword1 = (ieDword) fx->InventorySlot;
-		stream->WriteDword( &tmpDword1 );
-		stream->Write( filling,40 ); //12+32+8
-		stream->WriteDword( &fx->SecondaryType );
-		stream->Write( filling,60 );
+		eM->PutEffectV2(stream, fx);
 	}
 	return 0;
 }
