@@ -18,16 +18,24 @@
  *
  */
 
+#if defined(__HAIKU__)
+#include <unistd.h>
+#endif
+
+#ifdef ANDROID
+#include "swab.h"
+#endif
+
 #include "BAMImporter.h"
 
 #include "win32def.h"
 
 #include "Compressor.h"
-#include "FileStream.h"
 #include "GameData.h"
 #include "Interface.h"
 #include "Palette.h"
 #include "Video.h"
+#include "System/FileStream.h"
 
 BAMImporter::BAMImporter(void)
 {
