@@ -24,16 +24,16 @@
 #include "win32def.h"
 
 #include "AnimationFactory.h"
-#include "Button.h"
 #include "GameData.h"
 #include "ImageMgr.h"
 #include "Interface.h"
-#include "Label.h"
-#include "Progressbar.h"
-#include "ScrollBar.h"
-#include "Slider.h"
-#include "TextArea.h"
-#include "TextEdit.h"
+#include "GUI/Button.h"
+#include "GUI/Label.h"
+#include "GUI/Progressbar.h"
+#include "GUI/ScrollBar.h"
+#include "GUI/Slider.h"
+#include "GUI/TextArea.h"
+#include "GUI/TextEdit.h"
 
 CHUImporter::CHUImporter()
 {
@@ -140,14 +140,24 @@ Window* CHUImporter::GetWindow(unsigned int wid)
 				btn->Height = Height;
 				btn->ControlType = ControlType;
 				ieResRef BAMFile;
-				ieWord Cycle, UnpressedIndex, PressedIndex,
-				SelectedIndex, DisabledIndex;
+				ieByte Cycle, tmp;
+				ieDword Flags;
+				ieByte UnpressedIndex, x1;
+				ieByte PressedIndex, x2;
+				ieByte SelectedIndex, y1;
+				ieByte DisabledIndex, y2;
 				str->ReadResRef( BAMFile );
-				str->ReadWord( &Cycle );
-				str->ReadWord( &UnpressedIndex );
-				str->ReadWord( &PressedIndex );
-				str->ReadWord( &SelectedIndex );
-				str->ReadWord( &DisabledIndex );
+				str->Read( &Cycle, 1 );
+				str->Read( &tmp, 1 );
+				Flags = ((ieDword) tmp)<<8;
+				str->Read( &UnpressedIndex, 1 );
+				str->Read( &x1, 1 );
+				str->Read( &PressedIndex, 1 );
+				str->Read( &x2, 1 );
+				str->Read( &SelectedIndex, 1 );
+				str->Read( &y1, 1 );
+				str->Read( &DisabledIndex, 1 );
+				str->Read( &y2, 1 );
 				btn->Owner = win;
 				/** Justification comes from the .chu, other bits are set by script */
 				if (!Width) {
@@ -157,10 +167,14 @@ Window* CHUImporter::GetWindow(unsigned int wid)
 					btn->SetFlags(IE_GUI_BUTTON_CAPS, BM_OR);
 				}
 
-				btn->SetFlags( Cycle&0xff00, BM_OR );
+				btn->SetFlags( Flags, BM_OR );
+				if (Flags & IE_GUI_BUTTON_ANCHOR) {
+					btn->SetAnchor(x1 | (x2<<8), y1 | (y2<<8));
+				}
+
 				if (strnicmp( BAMFile, "guictrl\0", 8 ) == 0) {
 					if (UnpressedIndex == 0) {
-						printMessage("CHUImporter", "Special Button Control, Skipping Image Loading\n",GREEN );
+						//printMessage("CHUImporter", "Special Button Control, Skipping Image Loading\n",GREEN );
 						win->AddControl( btn );
 						break;
 					}
@@ -171,18 +185,18 @@ Window* CHUImporter::GetWindow(unsigned int wid)
 				if (!bam ) {
 					printMessage( "CHUImporter","Cannot Load Button Images, skipping control\n",LIGHT_RED );
 					/* IceWind Dale 2 has fake BAM ResRefs for some Buttons,
-					   this will handle bad ResRefs */
+					this will handle bad ResRefs */
 					win->AddControl( btn );
 					break;
 				}
 				/** Cycle is only a byte for buttons */
 				Sprite2D* tspr = bam->GetFrame( UnpressedIndex, (unsigned char) Cycle );
 				btn->SetImage( IE_GUI_BUTTON_UNPRESSED, tspr );
-				tspr = bam->GetFrame( PressedIndex, (unsigned char) Cycle );
+				tspr = bam->GetFrame( PressedIndex, Cycle );
 				btn->SetImage( IE_GUI_BUTTON_PRESSED, tspr );
 				//ignorebuttonframes is a terrible hack
 				if (core->HasFeature( GF_IGNORE_BUTTON_FRAMES) ) {
-					if (bam->GetCycleSize( (unsigned char) Cycle) == 4 )
+					if (bam->GetCycleSize(Cycle) == 4 )
 						SelectedIndex=2;
 				}
 				tspr = bam->GetFrame( SelectedIndex, (unsigned char) Cycle );
@@ -216,7 +230,7 @@ Window* CHUImporter::GetWindow(unsigned int wid)
 				str->ReadWord( &KnobYPos );
 				str->ReadWord( &CapXPos );
 				str->ReadWord( &CapYPos );
-				Progressbar* pbar = new Progressbar(KnobStepsCount, true ); 
+				Progressbar* pbar = new Progressbar(KnobStepsCount, true );
 				pbar->ControlID = ControlID;
 				pbar->XPos = XPos;
 				pbar->YPos = YPos;
@@ -235,7 +249,7 @@ Window* CHUImporter::GetWindow(unsigned int wid)
 					ResourceHolder<ImageMgr> mos(MOSFile2);
 					img2 = mos->GetSprite2D();
 				}
-				
+
 				pbar->SetImage( img, img2 );
 				if( KnobStepsCount ) {
 					/* getting the bam */
@@ -288,7 +302,7 @@ Window* CHUImporter::GetWindow(unsigned int wid)
 					sldr->SetImage( IE_GUI_SLIDER_GRABBEDKNOB, img );
 				}
 				else {
-					 sldr->SetState(IE_GUI_SLIDER_BACKGROUND);
+					sldr->SetState(IE_GUI_SLIDER_BACKGROUND);
 				}
 				win->AddControl( sldr );
 			}
@@ -300,30 +314,41 @@ Window* CHUImporter::GetWindow(unsigned int wid)
 				ieResRef BGMos;
 				ieResRef FontResRef, CursorResRef;
 				ieWord maxInput;
-	      ieWord CurCycle, CurFrame;
-	      ieWord PosX, PosY;
+				ieWord CurCycle, CurFrame;
+				ieWord PosX, PosY;
+				ieWord Pos2X, Pos2Y;
+				ieVariable Initial;
 
 				str->ReadResRef( BGMos );
+				//These are two more MOS resrefs, probably unused
 				str->Seek( 16, GEM_CURRENT_POS );
 				str->ReadResRef( CursorResRef );
-	      str->ReadWord( &CurCycle );
-	      str->ReadWord( &CurFrame );
-	      str->ReadWord( &PosX );
-	      str->ReadWord( &PosY );
-				str->Seek( 4, GEM_CURRENT_POS );
+				str->ReadWord( &CurCycle );
+				str->ReadWord( &CurFrame );
+				str->ReadWord( &PosX );
+				str->ReadWord( &PosY );
+				//FIXME: I still don't know what to do with this point
+				//Contrary to forum posts, it is definitely not a scrollbar ID
+				str->ReadWord( &Pos2X );
+				str->ReadWord( &Pos2Y );
 				str->ReadResRef( FontResRef );
-				str->Seek( 34, GEM_CURRENT_POS );
+				//this field is still unknown or unused
+				str->Seek( 2, GEM_CURRENT_POS );
+				//This is really a text field, but apparently the original engine
+				//always writes it over, and never uses it
+				str->Read( Initial, 32 );
+				Initial[32]=0;
 				str->ReadWord( &maxInput );
 				Font* fnt = core->GetFont( FontResRef );
-	      
+
 				AnimationFactory* bam = ( AnimationFactory* )
 					gamedata->GetFactoryResource( CursorResRef,
 							IE_BAM_CLASS_ID,
 							IE_NORMAL );
 				Sprite2D *cursor = NULL;
-	      if (bam) {
+				if (bam) {
 					cursor = bam->GetFrame( CurCycle, CurFrame );
-	      }
+				}
 
 				ResourceHolder<ImageMgr> mos(BGMos);
 				Sprite2D *img = NULL;
@@ -341,6 +366,8 @@ Window* CHUImporter::GetWindow(unsigned int wid)
 				te->SetFont( fnt );
 				te->SetCursor( cursor );
 				te->SetBackGround( img );
+				//The original engine always seems to ignore this textfield
+				//te->SetText (Initial );
 				win->AddControl( te );
 			}
 			break;
