@@ -27,11 +27,10 @@
 #include "GameData.h"
 #include "Interface.h"
 #include "MapMgr.h"
-#include "MemoryStream.h"
+#include "System/MemoryStream.h"
 
 #include <cassert>
 
-#define MAZE_DATA_SIZE 1720
 #define FAMILIAR_FILL_SIZE 324
 // if your compiler chokes on this, use -1 or 0xff whichever works for you
 #define UNINITIALIZED_CHAR '\xff'
@@ -114,16 +113,8 @@ Game* GAMImporter::LoadGame(Game *newGame, int ver_override)
 			newGame->version = ver_override;
 		}
 		else {
-			newGame->version=version;
+			newGame->version = version;
 		}
-/*
-		if (stricmp( core->GameType, "iwd2" ) == 0 && version != GAM_VER_IWD2) {
-			newGame->version = GAM_VER_IWD2;
-			printMessage("GAMImporter"," ",LIGHT_RED);
-			printf("Trying to load a non-iwd2 game (%d) in iwd2 mode! Patching version up.\n",
-				(int)version);
-		}
-*/
 	}
 
 	ieDword GameTime;
@@ -140,7 +131,8 @@ Game* GAMImporter::LoadGame(Game *newGame, int ver_override)
 		newGame->WhichFormation = 0;
 	}
 	str->ReadDword( &newGame->PartyGold );
-	str->ReadWord( &newGame->NpcInParty);                 //npc count in party???
+	//npc count in party???
+	str->ReadWord( &newGame->NpcInParty );
 	str->ReadWord( &newGame->WeatherBits );
 	str->ReadDword( &PCOffset );
 	str->ReadDword( &PCCount );
@@ -153,7 +145,7 @@ Game* GAMImporter::LoadGame(Game *newGame, int ver_override)
 	str->ReadDword( &GlobalOffset );
 	str->ReadDword( &GlobalCount );
 	str->ReadResRef( newGame->CurrentArea );
-	str->ReadDword( &newGame->Unknown48 );
+	str->ReadDword( &newGame->Unknown48 );//this is still unknown
 	str->ReadDword( &JournalCount );
 	str->ReadDword( &JournalOffset );
 	switch (version) {
@@ -163,7 +155,7 @@ Game* GAMImporter::LoadGame(Game *newGame, int ver_override)
 			str->ReadResRef( newGame->CurrentArea ); // FIXME: see above
 			memcpy(newGame->AnotherArea, newGame->CurrentArea, sizeof(ieResRef) );
 			str->ReadDword( &newGame->ControlStatus );
-			str->ReadDword( &KillVarsCount ); //this is still unknown
+			str->ReadDword( &newGame->Expansion );
 			str->ReadDword( &FamiliarsOffset );
 			str->ReadDword( &SavedLocOffset );
 			str->ReadDword( &SavedLocCount );
@@ -208,10 +200,7 @@ Game* GAMImporter::LoadGame(Game *newGame, int ver_override)
 	for (i = 0; i < PCCount; i++) {
 		str->Seek( PCOffset + ( i * PCSize ), GEM_STREAM_START );
 		Actor *actor = GetActor( aM, true );
-		newGame->JoinParty( actor, 0 );
-		if (actor->Selected) {
-			newGame->SelectActor(actor, true, SELECT_QUIET);
-		}
+		newGame->JoinParty( actor, actor->Selected?JP_SELECT:0 );
 	}
 
 	//Loading NPCs
@@ -223,8 +212,9 @@ Game* GAMImporter::LoadGame(Game *newGame, int ver_override)
 
 	//apparently BG1/IWD2 relies on this, if chapter is unset, it is
 	//set to -1, hopefully it won't break anything
-	newGame->locals->SetAt("CHAPTER", (ieDword) -1);
-	
+	//PST has no chapter variable by default, and would crash on one
+	newGame->locals->SetAt("CHAPTER", (ieDword) -1, core->HasFeature(GF_NO_NEW_VARIABLES));
+
 	// load initial values from var.var
 	newGame->locals->LoadInitialValues("GLOBAL");
 
@@ -243,6 +233,7 @@ Game* GAMImporter::LoadGame(Game *newGame, int ver_override)
 	if(core->HasFeature(GF_HAS_KAPUTZ) ) {
 		newGame->kaputz = new Variables();
 		newGame->kaputz->SetType( GEM_VARIABLES_INT );
+		newGame->kaputz->ParseKey( 1 );
 		// load initial values from var.var
 		newGame->kaputz->LoadInitialValues("KAPUTZ");
 		str->Seek( KillVarsOffset, GEM_STREAM_START );
@@ -266,9 +257,13 @@ Game* GAMImporter::LoadGame(Game *newGame, int ver_override)
 	if (version == GAM_VER_PST) {
 		//loading maze
 		if (MazeOffset) {
-			newGame->mazedata = (ieByte*)malloc(MAZE_DATA_SIZE);
+			//Don't allocate memory in plugins (MSVC chokes on this)
+			newGame->AllocateMazeData();
 			str->Seek(MazeOffset, GEM_STREAM_START );
-			str->Read(newGame->mazedata, MAZE_DATA_SIZE);
+			for (i = 0; i<MAZE_ENTRY_COUNT;i++) {
+				GetMazeEntry(newGame->mazedata+i*MAZE_ENTRY_SIZE);
+			}
+			GetMazeHeader(newGame->mazedata+MAZE_ENTRY_COUNT*MAZE_ENTRY_SIZE);
 		}
 		str->Seek( FamiliarsOffset, GEM_STREAM_START );
 	} else {
@@ -277,6 +272,11 @@ Game* GAMImporter::LoadGame(Game *newGame, int ver_override)
 			for (i=0; i<9;i++) {
 				str->ReadResRef( newGame->GetFamiliar(i) );
 			}
+		} else {
+			//clear these fields up
+			for (i=0;i<9;i++) {
+				memset(newGame->GetFamiliar(i), 0, sizeof(ieResRef));
+			}
 		}
 	}
 	// Loading known creatures array (beasts)
@@ -353,9 +353,12 @@ Actor* GAMImporter::GetActor(Holder<ActorMgr> aM, bool is_in_party )
 	str->ReadWord( &pcInfo.ViewYPos );
 	str->ReadWord( &pcInfo.ModalState ); //see Modal.ids
 	str->ReadWord( &pcInfo.Happiness );
-	str->Read( &pcInfo.Unknown2c, 96 );
+	for (i=0;i<24;i++) {
+		str->ReadDword( &pcInfo.Interact[i] ); //interact counters
+	}
 
-	if (version==GAM_VER_GEMRB || version==GAM_VER_IWD2) {
+	bool extended = version==GAM_VER_GEMRB || version==GAM_VER_IWD2;
+	if (extended) {
 		ieResRef tmp;
 
 		for (i = 0; i < 4; i++) {
@@ -413,9 +416,13 @@ Actor* GAMImporter::GetActor(Holder<ActorMgr> aM, bool is_in_party )
 		}
 		//QuickSlots are customisable in iwd2 and GemRB
 		//thus we adopt the iwd2 style actor info
+		//the first 3 slots are hardcoded anyway
+		pcInfo.QSlots[0] = ACT_TALK;
+		pcInfo.QSlots[1] = ACT_WEAPON1;
+		pcInfo.QSlots[2] = ACT_WEAPON2;
 		for (i=0;i<MAX_QSLOTS;i++) {
 			str->ReadDword( &tmpDword );
-			pcInfo.QSlots[i] = (ieByte) tmpDword;
+			pcInfo.QSlots[i+3] = (ieByte) tmpDword;
 		}
 	} else {
 		for (i = 0; i < 4; i++) {
@@ -473,7 +480,7 @@ Actor* GAMImporter::GetActor(Holder<ActorMgr> aM, bool is_in_party )
 		}
 		free (Buffer);
 
- 		//torment has them as 0 or -1
+		//torment has them as 0 or -1
 		if (pcInfo.Name[0]!=0 && pcInfo.Name[0]!=UNINITIALIZED_CHAR) {
 			actor->SetName(pcInfo.Name,0); //setting both names
 		}
@@ -497,7 +504,7 @@ Actor* GAMImporter::GetActor(Holder<ActorMgr> aM, bool is_in_party )
 	//
 	actor->CreateStats();
 	PCStatsStruct *ps = actor->PCStats;
-	GetPCStats(ps);
+	GetPCStats(ps, extended);
 	memcpy(ps->QSlots, pcInfo.QSlots, sizeof(pcInfo.QSlots) );
 	memcpy(ps->QuickSpells, pcInfo.QuickSpellResRef, MAX_QSLOTS*sizeof(ieResRef) );
 	memcpy(ps->QuickSpellClass, pcInfo.QuickSpellClass, MAX_QSLOTS );
@@ -512,6 +519,8 @@ Actor* GAMImporter::GetActor(Holder<ActorMgr> aM, bool is_in_party )
 	actor->TalkCount = pcInfo.TalkCount;
 	actor->ModalState = pcInfo.ModalState;
 	actor->SetModalSpell(pcInfo.ModalState, 0);
+	ps->Happiness = pcInfo.Happiness;
+	memcpy(ps->Interact, pcInfo.Interact, MAX_INTERACT *sizeof(ieDword) );
 
 	actor->SetPersistent( tmpWord );
 
@@ -519,7 +528,7 @@ Actor* GAMImporter::GetActor(Holder<ActorMgr> aM, bool is_in_party )
 	return actor;
 }
 
-void GAMImporter::GetPCStats (PCStatsStruct *ps)
+void GAMImporter::GetPCStats (PCStatsStruct *ps, bool extended)
 {
 	int i;
 
@@ -549,6 +558,18 @@ void GAMImporter::GetPCStats (PCStatsStruct *ps)
 	if (core->HasFeature(GF_SOUNDFOLDERS) ) {
 		str->Read( ps->SoundFolder, 32);
 	}
+	
+	//iwd2 has some PC only stats that the player can set (this can be done via a guiscript interface)
+	if (extended) {
+		//3 - expertise
+		//4 - power attack
+		//5 - arterial strike
+		//6 - hamstring
+		//7 - rapid shot
+		for (i=0;i<16;i++) {
+			str->ReadDword( &ps->ExtraSettings[i] );
+		}
+	}
 }
 
 GAMJournalEntry* GAMImporter::GetJournalEntry()
@@ -611,21 +632,22 @@ int GAMImporter::GetStoredFileSize(Game *game)
 	PCCount = game->GetPartySize(false);
 	headersize += PCCount * PCSize;
 	for (i = 0;i<PCCount; i++) {
-		Actor *ac=game->GetPC(i, false);
-		headersize +=am->GetStoredFileSize(ac);
+		Actor *ac = game->GetPC(i, false);
+		headersize += am->GetStoredFileSize(ac);
 	}
 	NPCOffset = headersize;
 
 	NPCCount = game->GetNPCCount();
 	headersize += NPCCount * PCSize;
 	for (i = 0;i<NPCCount; i++) {
-		Actor *ac=game->GetNPC(i);
-		headersize +=am->GetStoredFileSize(ac);
+		Actor *ac = game->GetNPC(i);
+		headersize += am->GetStoredFileSize(ac);
 	}
 
 	if (game->mazedata) {
 		MazeOffset = headersize;
-		headersize += MAZE_DATA_SIZE;
+		//due to alignment the internal size is not the same as the external size
+		headersize += MAZE_DATA_SIZE_HARDCODED;
 	} else {
 		MazeOffset = 0;
 	}
@@ -649,21 +671,25 @@ int GAMImporter::GetStoredFileSize(Game *game)
 	} else {
 		FamiliarsOffset = headersize;
 		if (core->GetBeastsINI()) {
-			headersize +=BESTIARY_SIZE;
+			headersize += BESTIARY_SIZE;
 		}
 		if (game->version!=GAM_VER_PST) {
 			headersize += 9 * 8 + 82 * 4;
 		}
 	}
 
-	//TODO: Fix this for ToB and BG2 (saved locations and pocket plane locations)
 	SavedLocOffset = headersize;
-	SavedLocCount = 0;
+	SavedLocCount = game->GetSavedLocationCount();
+	//there is an unknown dword at the end of iwd2 savegames
+	if (game->version==GAM_VER_IWD2) {
+		headersize += 4;
+	}
+	headersize += SavedLocCount*12;
 
 	PPLocOffset = headersize;
-	PPLocCount = 0;
+	PPLocCount = game->GetPlaneLocationCount();
 
-	return headersize + (PPLocCount+SavedLocCount) * 20;
+	return headersize + PPLocCount * 12;
 }
 
 int GAMImporter::PutJournals(DataStream *stream, Game *game)
@@ -683,10 +709,19 @@ int GAMImporter::PutJournals(DataStream *stream, Game *game)
 	return 0;
 }
 
-//only in ToB
+//only in ToB (and iwd2)
 int GAMImporter::PutSavedLocations(DataStream *stream, Game *game)
 {
 	ieWord tmpWord;
+	ieDword filling = 0;
+
+	//iwd2 has a single 0 dword here (at the end of the file)
+	//it could be a hacked out saved location list (inherited from SoA)
+	//if the field is missing, original engine cannot load this saved game
+	if (game->version==GAM_VER_IWD2) {
+		stream->WriteDword(&filling);
+		return 0;
+	}
 
 	for (unsigned int i=0;i<SavedLocCount;i++) {
 			GAMLocationEntry *j = game->GetSavedLocationEntry(i);
@@ -751,7 +786,7 @@ int GAMImporter::PutVariables(DataStream *stream, Game *game)
 	for (unsigned int i=0;i<GlobalCount;i++) {
 		//global variables are locals for game, that's why the local/global confusion
 		pos=game->locals->GetNextAssoc( pos, name, value);
-		strnspccpy(tmpname,name,32);
+		strnspccpy(tmpname, name, 32);
 		stream->Write( tmpname, 32);
 		stream->Write( filling, 8);
 		stream->WriteDword( &value);
@@ -769,7 +804,7 @@ int GAMImporter::PutHeader(DataStream *stream, Game *game)
 
 	memcpy( Signature, "GAMEV0.0", 8);
 	Signature[5]+=game->version/10;
-	if (game->version==GAM_VER_PST) { //pst version
+	if (game->version==GAM_VER_PST || game->version==GAM_VER_BG) { //pst/bg1 saved version
 		Signature[7]+=1;
 	}
 	else {
@@ -806,8 +841,7 @@ int GAMImporter::PutHeader(DataStream *stream, Game *game)
 	stream->WriteDword( &GlobalOffset );
 	stream->WriteDword( &GlobalCount );
 	stream->WriteResRef( game->CurrentArea );
-	//this is always 0xffffffff
-	stream->WriteDword( &game->Unknown48 );        
+	stream->WriteDword( &game->Unknown48 );
 	stream->WriteDword( &JournalCount );
 	stream->WriteDword( &JournalOffset );
 
@@ -821,7 +855,7 @@ int GAMImporter::PutHeader(DataStream *stream, Game *game)
 		stream->WriteDword( &game->Reputation );
 		stream->WriteResRef( game->CurrentArea );
 		stream->WriteDword( &game->ControlStatus );
-		stream->WriteDword( &tmpDword);
+		stream->WriteDword( &game->Expansion);
 		stream->WriteDword( &FamiliarsOffset);
 		stream->WriteDword( &SavedLocOffset);
 		stream->WriteDword( &SavedLocCount);
@@ -837,9 +871,15 @@ int GAMImporter::PutHeader(DataStream *stream, Game *game)
 		break;
 	}
 	stream->WriteDword( &game->RealTime ); //this isn't correct, this field is the realtime
-	stream->WriteDword( &SavedLocOffset);
-	for (i=0;i<7;i++) {
-		stream->Write( Signature, 8);
+	stream->WriteDword( &PPLocOffset);
+	stream->WriteDword( &PPLocCount);
+	char filling[52];
+	memset( filling, 0, sizeof(filling) );
+	stream->Write( &filling, 52); //unknown
+
+	//save failed, but it is not our fault, returning now before the asserts kill us
+	if (stream->GetPos()==0) {
+		return -1;
 	}
 	return 0;
 }
@@ -849,7 +889,7 @@ int GAMImporter::PutActor(DataStream *stream, Actor *ac, ieDword CRESize, ieDwor
 	int i;
 	ieDword tmpDword;
 	ieWord tmpWord;
-	char filling[194];
+	char filling[130];
 
 	memset(filling,0,sizeof(filling) );
 	if (ac->Selected) {
@@ -865,9 +905,9 @@ int GAMImporter::PutActor(DataStream *stream, Actor *ac, ieDword CRESize, ieDwor
 	stream->WriteDword( &CREOffset);
 	stream->WriteDword( &CRESize);
 	//creature resref is always unused in saved games
-	filling[0]='*';                       //make it the same as in the original
+	//BG1 doesn't even like the * in there, zero fill
+	//seems to be accepted by all
 	stream->Write( filling, 8);
-	filling[0]=0;
 	tmpDword = ac->GetOrientation();
 	stream->WriteDword( &tmpDword);
 	stream->WriteResRef(ac->Area);
@@ -882,10 +922,12 @@ int GAMImporter::PutActor(DataStream *stream, Actor *ac, ieDword CRESize, ieDwor
 	stream->WriteWord( &tmpWord);
 	tmpWord = (ieWord) ac->ModalState;
 	stream->WriteWord( &tmpWord);
-	tmpWord = 0; //happiness
+	tmpWord = ac->PCStats->Happiness;
 	stream->WriteWord( &tmpWord);
-	//a field of zeroes
-	stream->Write(filling, 96);
+	//interact counters
+	for (i=0;i<24;i++) {
+		stream->WriteDword( ac->PCStats->Interact+i);
+	}
 
 	//quickweapons
 	if (version==GAM_VER_IWD2 || version==GAM_VER_GEMRB) {
@@ -970,7 +1012,7 @@ int GAMImporter::PutActor(DataStream *stream, Actor *ac, ieDword CRESize, ieDwor
 			}
 		}
 		for (i=0;i<MAX_QSLOTS;i++) {
-			tmpDword = ac->PCStats->QSlots[i];
+			tmpDword = ac->PCStats->QSlots[i+3];
 			stream->WriteDword( &tmpDword);
 		}
 	}
@@ -1011,8 +1053,14 @@ int GAMImporter::PutActor(DataStream *stream, Actor *ac, ieDword CRESize, ieDwor
 		stream->Write(ac->PCStats->SoundFolder, 32);
 	}
 	if (version==GAM_VER_IWD2 || version==GAM_VER_GEMRB) {
-		stream->Write(filling, 194);
+		//I don't know how many fields are actually used in IWD2 saved game
+		//but we got at least 8 (and only 5 of those are actually used)
+		for(i=0;i<16;i++) {
+			stream->WriteDword( &ac->PCStats->ExtraSettings[i]);
+		}
+		stream->Write(filling, 130);
 	}
+
 	return 0;
 }
 
@@ -1029,9 +1077,10 @@ int GAMImporter::PutPCs(DataStream *stream, Game *game)
 		PutActor(stream, ac, CRESize, CREOffset, game->version);
 		CREOffset += CRESize;
 	}
-	assert(stream->GetPos() == PCOffset + PCCount * PCSize);
 
 	CREOffset = PCOffset + PCCount * PCSize; // just for the asserts..
+	assert(stream->GetPos() == CREOffset);
+
 	for(i=0;i<PCCount;i++) {
 		assert(stream->GetPos() == CREOffset);
 		Actor *ac = game->GetPC(i, false);
@@ -1039,6 +1088,7 @@ int GAMImporter::PutPCs(DataStream *stream, Game *game)
 		CREOffset += am->GetStoredFileSize(ac);
 		am->PutActor( stream, ac);
 	}
+	assert(stream->GetPos() == CREOffset);
 	return 0;
 }
 
@@ -1049,23 +1099,95 @@ int GAMImporter::PutNPCs(DataStream *stream, Game *game)
 	ieDword CREOffset = NPCOffset + NPCCount * PCSize;
 
 	for(i=0;i<NPCCount;i++) {
+		assert(stream->GetPos() == NPCOffset + i * PCSize);
 		Actor *ac = game->GetNPC(i);
 		ieDword CRESize = am->GetStoredFileSize(ac);
 		PutActor(stream, ac, CRESize, CREOffset, game->version);
 		CREOffset += CRESize;
 	}
+	CREOffset = NPCOffset + NPCCount * PCSize; // just for the asserts..
+	assert(stream->GetPos() == CREOffset);
+
 	for(i=0;i<NPCCount;i++) {
+		assert(stream->GetPos() == CREOffset);
 		Actor *ac = game->GetNPC(i);
 		//reconstructing offsets again
-		am->GetStoredFileSize(ac);
+		CREOffset += am->GetStoredFileSize(ac);
 		am->PutActor( stream, ac);
 	}
+	assert(stream->GetPos() == CREOffset);
 	return 0;
 }
 
+void GAMImporter::GetMazeHeader(void *memory)
+{
+	maze_header *m = (maze_header *) memory;
+	str->ReadDword( &m->maze_sizex );
+	str->ReadDword( &m->maze_sizey );
+	str->ReadDword( &m->pos1x );
+	str->ReadDword( &m->pos1y );
+	str->ReadDword( &m->pos2x );
+	str->ReadDword( &m->pos2y );
+	str->ReadDword( &m->pos3x );
+	str->ReadDword( &m->pos3y );
+	str->ReadDword( &m->pos4x );
+	str->ReadDword( &m->pos4y );
+	str->ReadDword( &m->trapcount );
+	str->ReadDword( &m->initialized );
+	str->ReadDword( &m->unknown2c );
+	str->ReadDword( &m->unknown30 );
+}
+
+void GAMImporter::GetMazeEntry(void *memory)
+{
+	maze_entry *h = (maze_entry *) memory;
+
+	str->ReadDword( &h->override );
+	str->ReadDword( &h->valid );
+	str->ReadDword( &h->accessible );
+	str->ReadDword( &h->trapped );
+	str->ReadDword( &h->traptype );
+	str->ReadWord( &h->walls );
+	str->ReadDword( &h->visited );
+}
+
+void GAMImporter::PutMazeHeader(DataStream *stream, void *memory)
+{
+	maze_header *m = (maze_header *) memory;
+	stream->WriteDword( &m->maze_sizex );
+	stream->WriteDword( &m->maze_sizey );
+	stream->WriteDword( &m->pos1x );
+	stream->WriteDword( &m->pos1y );
+	stream->WriteDword( &m->pos2x );
+	stream->WriteDword( &m->pos2y );
+	stream->WriteDword( &m->pos3x );
+	stream->WriteDword( &m->pos3y );
+	stream->WriteDword( &m->pos4x );
+	stream->WriteDword( &m->pos4y );
+	stream->WriteDword( &m->trapcount );
+	stream->WriteDword( &m->initialized );
+	stream->WriteDword( &m->unknown2c );
+	stream->WriteDword( &m->unknown30 );
+}
+
+void GAMImporter::PutMazeEntry(DataStream *stream, void *memory)
+{
+	maze_entry *h = (maze_entry *) memory;
+	stream->WriteDword( &h->override );
+	stream->WriteDword( &h->valid );
+	stream->WriteDword( &h->accessible );
+	stream->WriteDword( &h->trapped );
+	stream->WriteDword( &h->traptype );
+	stream->WriteWord( &h->walls );
+	stream->WriteDword( &h->visited );
+}
+
 int GAMImporter::PutMaze(DataStream *stream, Game *game)
 {
-	stream->Write( game->mazedata, MAZE_DATA_SIZE);
+	for(int i=0;i<MAZE_ENTRY_COUNT;i++) {
+		PutMazeEntry(stream, game->mazedata+i*MAZE_ENTRY_SIZE);
+	}
+	PutMazeHeader(stream, game->mazedata+MAZE_ENTRY_COUNT*MAZE_ENTRY_SIZE);
 	return 0;
 }
 
@@ -1149,6 +1271,18 @@ int GAMImporter::PutGame(DataStream *stream, Game *game)
 			return ret;
 		}
 	}
+	if (SavedLocOffset) {
+		ret = PutSavedLocations( stream, game);
+		if (ret) {
+			return ret;
+		}
+	}
+	if (PPLocOffset) {
+		ret = PutPlaneLocations( stream, game);
+		if (ret) {
+			return ret;
+		}
+	}
 
 	return 0;
 }
