@@ -19,12 +19,15 @@
 #ifndef ACMREADER_H
 #define ACMREADER_H
 
-#include "DataStream.h"
 #include "SoundMgr.h"
+#include "System/DataStream.h"
 
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#ifdef __APPLE_CC__
+#define OV_EXCLUDE_STATIC_CALLBACKS
+#endif
 #include <vorbis/vorbisfile.h>
 
 class OGGReader : public SoundMgr {
