@@ -29,7 +29,7 @@ bool checkALError(const char* msg, const char* status) {
 	int error = alGetError();
 	if (error != AL_NO_ERROR) {
 		printMessage("OpenAL", msg, WHITE );
-		printf (": %d ", error);
+		printf (": 0x%x ", error);
 		printStatus(status, YELLOW);
 		return true;
 	}
@@ -40,11 +40,40 @@ void showALCError(const char* msg, const char* status, ALCdevice *device) {
 	int error = alcGetError(device);
 	printMessage("OpenAL", msg, WHITE );
 	if (error != AL_NO_ERROR) {
-		printf (": %d ", error);
+		printf (": 0x%x ", error);
 	}
 	printStatus(status, YELLOW);
 }
 
+void OpenALSoundHandle::SetPos(int XPos, int YPos) {
+	if (!parent) return;
+
+	ALfloat SourcePos[] = {
+		(float) XPos, (float) YPos, 0.0f
+	};
+
+	alSourcefv(parent->Source, AL_POSITION, SourcePos);
+}
+
+bool OpenALSoundHandle::Playing() {
+	if (!parent) return false;
+
+	parent->ClearIfStopped();
+	return parent != 0;
+}
+
+void OpenALSoundHandle::Stop() {
+	if (!parent) return;
+
+	parent->ForceClear();
+}
+
+void OpenALSoundHandle::StopLooping() {
+	if (!parent) return;
+
+	alSourcei(parent->Source, AL_LOOPING, 0);
+}
+
 void AudioStream::ClearProcessedBuffers()
 {
 	ALint processed = 0;
@@ -84,6 +113,7 @@ void AudioStream::ClearIfStopped()
 		Source = 0;
 		Buffer = 0;
 		free = true;
+		if (handle) { handle->Invalidate(); handle.release(); }
 		ambient = false;
 		locked = false;
 		delete_buffers = false;
@@ -104,7 +134,7 @@ OpenALAudioDriver::OpenALAudioDriver(void)
 {
 	alutContext = NULL;
 	MusicPlaying = false;
-	music_memory = (unsigned char*) malloc(ACM_BUFFERSIZE);
+	music_memory = (short*) malloc(ACM_BUFFERSIZE);
 	MusicSource = 0;
 	memset(MusicBuffer, 0, MUSICBUFFERS*sizeof(ALuint));
 	musicMutex = SDL_CreateMutex();
@@ -248,9 +278,9 @@ ALuint OpenALAudioDriver::loadSound(const char *ResRef, unsigned int &time_lengt
 	int samplerate = acm->get_samplerate();
 	//multiply always by 2 because it is in 16 bits
 	int rawsize = cnt * 2;
-	unsigned char * memory = (unsigned char*) malloc(rawsize);
+	short* memory = (short*) malloc(rawsize);
 	//multiply always with 2 because it is in 16 bits
-	int cnt1 = acm->read_samples( ( short* ) memory, cnt ) * 2;
+	int cnt1 = acm->read_samples( memory, cnt ) * 2;
 	//Sound Length in milliseconds
 	time_length = ((cnt / riff_chans) * 1000) / samplerate;
 	//it is always reading the stuff into 16 bits
@@ -276,24 +306,28 @@ ALuint OpenALAudioDriver::loadSound(const char *ResRef, unsigned int &time_lengt
 	return Buffer;
 }
 
-unsigned int OpenALAudioDriver::Play(const char* ResRef, int XPos, int YPos, unsigned int flags)
+Holder<SoundHandle> OpenALAudioDriver::Play(const char* ResRef, int XPos, int YPos, unsigned int flags, unsigned int *length)
 {
 	ALuint Buffer;
 	unsigned int time_length;
 
 	if(ResRef == NULL) {
-        if((flags & GEM_SND_SPEECH) && alIsSource(speech.Source)) {
-            //So we want him to be quiet...
-            alSourceStop( speech.Source );
+		if((flags & GEM_SND_SPEECH) && alIsSource(speech.Source)) {
+			//So we want him to be quiet...
+			alSourceStop( speech.Source );
 			checkALError("Unable to stop speech", "WARNING");
 			speech.ClearProcessedBuffers();
 		}
-		return 0;
+		return Holder<SoundHandle>();
 	}
 
 	Buffer = loadSound( ResRef, time_length );
 	if (Buffer == 0) {
-		return 0;
+		return Holder<SoundHandle>();
+	}
+
+	if (length) {
+		*length = time_length;
 	}
 
 	ALuint Source;
@@ -310,38 +344,39 @@ unsigned int OpenALAudioDriver::Play(const char* ResRef, int XPos, int YPos, uns
 		//speech has a single channel, if a new speech started
 		//we stop the previous one
 		if(!speech.free && alIsSource(speech.Source)) {
-		    alSourceStop( speech.Source );
+			alSourceStop( speech.Source );
 			checkALError("Unable to stop speech", "WARNING");
 			speech.ClearProcessedBuffers();
 		}
-        if(!alIsSource(speech.Source)) {
-            alGenSources( 1, &speech.Source );
-            if (checkALError("Error creating source for speech", "ERROR")) {
-                return 0;
-            }
-        }
+		if(!alIsSource(speech.Source)) {
+			alGenSources( 1, &speech.Source );
+			if (checkALError("Error creating source for speech", "ERROR")) {
+				return Holder<SoundHandle>();
+			}
+		}
 
-        alSourcef( speech.Source, AL_PITCH, 1.0f );
-        alSourcefv( speech.Source, AL_VELOCITY, SourceVel );
-        alSourcei( speech.Source, AL_LOOPING, 0 );
-        alSourcef( speech.Source, AL_REFERENCE_DISTANCE, REFERENCE_DISTANCE );
-        checkALError("Unable to set speech parameters", "WARNING");
-        speech.free = false;
-        printf("speech.free: %d source:%d\n", speech.free,speech.Source);
+		alSourcef( speech.Source, AL_PITCH, 1.0f );
+		alSourcefv( speech.Source, AL_VELOCITY, SourceVel );
+		alSourcei( speech.Source, AL_LOOPING, 0 );
+		alSourcef( speech.Source, AL_REFERENCE_DISTANCE, REFERENCE_DISTANCE );
+		checkALError("Unable to set speech parameters", "WARNING");
+		speech.free = false;
+		printf("speech.free: %d source:%d\n", speech.free,speech.Source);
 
-        core->GetDictionary()->Lookup( "Volume Voices", volume );
-        alSourcef( speech.Source, AL_GAIN, 0.01f * volume );
-        alSourcei( speech.Source, AL_SOURCE_RELATIVE, flags & GEM_SND_RELATIVE );
-        alSourcefv( speech.Source, AL_POSITION, SourcePos );
-        assert(!speech.delete_buffers);
-        alSourcei( speech.Source, AL_BUFFER, Buffer );
-        checkALError("Unable to set speech parameters", "WARNING");
-        speech.Buffer = Buffer;
-        alSourcePlay( speech.Source );
-        if (checkALError("Unable to play speech", "ERROR")) {
-            return 0;
-        }
-        return time_length;
+		core->GetDictionary()->Lookup( "Volume Voices", volume );
+		alSourcef( speech.Source, AL_GAIN, 0.01f * volume );
+		alSourcei( speech.Source, AL_SOURCE_RELATIVE, flags & GEM_SND_RELATIVE );
+		alSourcefv( speech.Source, AL_POSITION, SourcePos );
+		assert(!speech.delete_buffers);
+		alSourcei( speech.Source, AL_BUFFER, Buffer );
+		checkALError("Unable to set speech parameters", "WARNING");
+		speech.Buffer = Buffer;
+		alSourcePlay( speech.Source );
+		if (checkALError("Unable to play speech", "ERROR")) {
+			return Holder<SoundHandle>();
+		}
+		speech.handle = new OpenALSoundHandle(&speech);
+		return speech.handle.get();
 	}
 
 	int stream = -1;
@@ -356,18 +391,18 @@ unsigned int OpenALAudioDriver::Play(const char* ResRef, int XPos, int YPos, uns
 	if (stream == -1) {
 		// Failed to assign new sound.
 		// The buffercache will handle deleting Buffer.
-		return 0;
+		return Holder<SoundHandle>();
 	}
 
 	// not speech
 	alGenSources( 1, &Source );
 	if (checkALError("Unable to create source", "ERROR")) {
-		return 0;
+		return Holder<SoundHandle>();
 	}
 
 	alSourcef( Source, AL_PITCH, 1.0f );
 	alSourcefv( Source, AL_VELOCITY, SourceVel );
-	alSourcei( Source, AL_LOOPING, 0 );
+	alSourcei( Source, AL_LOOPING, (flags & GEM_SND_LOOPING ? 1 : 0) );
 	alSourcef( Source, AL_REFERENCE_DISTANCE, REFERENCE_DISTANCE );
 	core->GetDictionary()->Lookup( "Volume SFX", volume );
 	alSourcef( Source, AL_GAIN, 0.01f * volume );
@@ -377,7 +412,7 @@ unsigned int OpenALAudioDriver::Play(const char* ResRef, int XPos, int YPos, uns
 	alSourcei( Source, AL_BUFFER, Buffer );
 
 	if (checkALError("Unable to set sound parameters", "ERROR")) {
-		return 0;
+		return Holder<SoundHandle>();
 	}
 
 	streams[stream].Buffer = Buffer;
@@ -386,10 +421,11 @@ unsigned int OpenALAudioDriver::Play(const char* ResRef, int XPos, int YPos, uns
 	alSourcePlay( Source );
 
 	if (checkALError("Unable to play sound", "ERROR")) {
-		return 0;
+		return Holder<SoundHandle>();
 	}
 
-	return time_length;
+	streams[stream].handle = new OpenALSoundHandle(&streams[stream]);
+	return streams[stream].handle.get();
 }
 
 bool OpenALAudioDriver::IsSpeaking()
@@ -470,6 +506,42 @@ bool OpenALAudioDriver::Stop()
 	return true;
 }
 
+bool OpenALAudioDriver::Pause()
+{
+	SDL_mutexP( musicMutex );
+	if (!alIsSource( MusicSource )) {
+		SDL_mutexV( musicMutex );
+		return false;
+	}
+	alSourcePause(MusicSource);
+	checkALError("Unable to pause music source", "WARNING");
+	MusicPlaying = false;
+	SDL_mutexV( musicMutex );
+	((AmbientMgrAL*) ambim)->deactivate();
+#ifdef ANDROID
+	al_android_pause_playback(); //call AudioTrack.pause() from JNI
+#endif
+	return true;
+}
+
+bool OpenALAudioDriver::Resume()
+{
+#ifdef ANDROID
+	al_android_resume_playback(); //call AudioTrack.play() from JNI
+#endif
+	SDL_mutexP( musicMutex );
+	if (!alIsSource( MusicSource )) {
+		SDL_mutexV( musicMutex );
+		return false;
+	}
+	alSourcePlay(MusicSource);
+	checkALError("Unable to resume music source", "WARNING");
+	MusicPlaying = true;
+	SDL_mutexV( musicMutex );
+	((AmbientMgrAL*) ambim)->activate();
+	return true;
+}
+
 int OpenALAudioDriver::CreateStream(Holder<SoundMgr> newMusic)
 {
 	StackLock l(musicMutex, "musicMutex in CreateStream()");
@@ -729,7 +801,7 @@ int OpenALAudioDriver::MusicManager(void* arg)
 					 {
 						printMessage("OPENAL", "Music in INITIAL State. AutoStarting\n", WHITE );
 						for (int i = 0; i < MUSICBUFFERS; i++) {
-							driver->MusicReader->read_samples( ( short* ) driver->music_memory, ACM_BUFFERSIZE >> 1 );
+							driver->MusicReader->read_samples( driver->music_memory, ACM_BUFFERSIZE >> 1 );
 							alBufferData( driver->MusicBuffer[i], AL_FORMAT_STEREO16,
 								driver->music_memory, ACM_BUFFERSIZE,
 								driver->MusicReader->get_samplerate() );
@@ -769,7 +841,7 @@ int OpenALAudioDriver::MusicManager(void* arg)
 					}
 					if (bFinished == AL_FALSE) {
 						int size = ACM_BUFFERSIZE;
-						int cnt = driver->MusicReader->read_samples( ( short* ) driver->music_memory, ACM_BUFFERSIZE >> 1 );
+						int cnt = driver->MusicReader->read_samples( driver->music_memory, ACM_BUFFERSIZE >> 1 );
 						size -= ( cnt * 2 );
 						if (size != 0)
 							bFinished = AL_TRUE;
@@ -778,11 +850,11 @@ int OpenALAudioDriver::MusicManager(void* arg)
 							core->GetMusicMgr()->PlayNext();
 							if (driver->MusicPlaying) {
 								printMessage( "OpenAL", "Queuing New Music\n", WHITE );
-								driver->MusicReader->read_samples( ( short* ) ( driver->music_memory + ( cnt*2 ) ), size >> 1 );
+								driver->MusicReader->read_samples( ( driver->music_memory + cnt ), size >> 1 );
 								bFinished = AL_FALSE;
 							} else {
 								printMessage( "OpenAL", "No Other Music to play\n", WHITE );
-								memset( driver->music_memory + ( cnt * 2 ), 0, size );
+								memset( driver->music_memory + cnt, 0, size );
 								driver->MusicPlaying = false;
 								break;
 							}
