@@ -21,24 +21,27 @@
 #include "strrefs.h"
 #include "win32def.h"
 
-#include "Actor.h"
 #include "EffectQueue.h"
 #include "Game.h"
 #include "GameData.h"
 #include "Interface.h"
 #include "TileMap.h"
 #include "Video.h" //for tints
+#include "Scriptable/Actor.h"
 
 int fx_retreat_from (Scriptable* Owner, Actor* target, Effect* fx);//6e
 int fx_set_status (Scriptable* Owner, Actor* target, Effect* fx);//ba
 int fx_play_bam_blended (Scriptable* Owner, Actor* target, Effect* fx);//bb
 int fx_play_bam_not_blended (Scriptable* Owner, Actor* target, Effect* fx);//bc
 int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx);//c0
-//int fx_shake_screen (Scriptable* Owner, Actor* target, Effect* fx);//c1 already implemented
+//int fx_shake_screen (Scriptable* Owner, Actor* target, Effect* fx);//c1 already implemented in fxopcodes
 int fx_flash_screen (Scriptable* Owner, Actor* target, Effect* fx);//c2
 int fx_tint_screen (Scriptable* Owner, Actor* target, Effect* fx);//c3
 int fx_special_effect (Scriptable* Owner, Actor* target, Effect* fx);//c4
-//unknown 0xc5-c8
+int fx_multiple_vvc (Scriptable* Owner, Actor* target, Effect* fx);//c5 //gemrb specific
+//int fx_modify_global ((Scriptable* Owner, Actor* target, Effect* fx);//c6 already implemented in fxopcodes
+int fx_change_background (Scriptable* Owner, Actor* target, Effect* fx);//c7 //gemrb specific
+//unknown 0xc7-c8
 int fx_overlay (Scriptable* Owner, Actor* target, Effect* fx);//c9
 //unknown 0xca
 int fx_bless (Scriptable* Owner, Actor* target, Effect* fx);//82 (this is a modified effect)
@@ -51,38 +54,41 @@ int fx_iron_fist (Scriptable* Owner, Actor* target, Effect* fx);//d0
 int fx_hostile_image(Scriptable* Owner, Actor* target, Effect* fx);//d1
 int fx_detect_evil (Scriptable* Owner, Actor* target, Effect* fx);//d2
 int fx_jumble_curse (Scriptable* Owner, Actor* target, Effect* fx);//d3
-//int fx_unknown (Scriptable* Owner, Actor* target, Effect* fx);//d4
+int fx_speak_with_dead (Scriptable* Owner, Actor* target, Effect* fx);//d4
 
-// FIXME: Make this an ordered list, so we could use bsearch!
-static EffectRef effectnames[] = {
-	{ "RetreatFrom", fx_retreat_from, -1 },//6e
-	{ "Bless", fx_bless, -1},//82
-	{ "Curse", fx_curse, -1},//cb
-	{ "DetectEvil", fx_detect_evil, -1}, //d2
-	{ "Embalm", fx_embalm, -1}, //0xce
-	{ "FlashScreen", fx_flash_screen, -1}, //c2
-	{ "HostileImage", fx_hostile_image, -1},//d1
-	{ "IronFist", fx_iron_fist, -1}, //d0
-	{ "JumbleCurse", fx_jumble_curse, -1}, //d3
-	{ "MoveView", fx_move_view, -1},//cd
-	{ "Overlay", fx_overlay, -1}, //c9
-	{ "PlayBAM1", fx_play_bam_blended, -1}, //bb
-	{ "PlayBAM2", fx_play_bam_not_blended, -1},//bc
-	{ "PlayBAM3", fx_play_bam_not_blended, -1}, //bd
-	{ "PlayBAM4", fx_play_bam_not_blended, -1}, //be
-	{ "PlayBAM5", fx_play_bam_not_blended, -1}, //bf
-	{ "Prayer", fx_prayer, -1},//cc
-	{ "SetStatus", fx_set_status, -1}, //ba
-	{ "SpecialEffect", fx_special_effect, -1},//c4
-	{ "StopAllAction", fx_stop_all_action, -1}, //cf
-	{ "TintScreen", fx_tint_screen, -1}, //c3
-	{ "TransferHP", fx_transfer_hp, -1}, //c0
-	{ NULL, NULL, 0 },
+//the engine sorts these, feel free to use any order
+static EffectDesc effectnames[] = {
+	{ "Bless", fx_bless, 0, -1 },//82
+	{ "ChangeBackground", fx_change_background, EFFECT_NO_ACTOR, -1 }, //c6
+	{ "Curse", fx_curse, 0, -1 },//cb
+	{ "DetectEvil", fx_detect_evil, 0, -1 }, //d2
+	{ "Embalm", fx_embalm, 0, -1 }, //0xce
+	{ "FlashScreen", fx_flash_screen, EFFECT_NO_ACTOR, -1 }, //c2
+	{ "HostileImage", fx_hostile_image, 0, -1 },//d1
+	{ "IronFist", fx_iron_fist, 0, -1 }, //d0
+	{ "JumbleCurse", fx_jumble_curse, 0, -1 }, //d3
+	{ "MoveView", fx_move_view, EFFECT_NO_ACTOR, -1 },//cd
+	{ "MultipleVVC", fx_multiple_vvc, EFFECT_NO_ACTOR, -1 }, //c5
+	{ "Overlay", fx_overlay, 0, -1 }, //c9
+	{ "PlayBAM1", fx_play_bam_blended, 0, -1 }, //bb
+	{ "PlayBAM2", fx_play_bam_not_blended, 0, -1 },//bc
+	{ "PlayBAM3", fx_play_bam_not_blended, 0, -1 }, //bd
+	{ "PlayBAM4", fx_play_bam_not_blended, 0, -1 }, //be
+	{ "PlayBAM5", fx_play_bam_not_blended, 0, -1 }, //bf
+	{ "Prayer", fx_prayer, 0, -1 },//cc
+	{ "RetreatFrom", fx_retreat_from, 0, -1 },//6e
+	{ "SetStatus", fx_set_status, 0, -1 }, //ba
+	{ "SpeakWithDead", fx_speak_with_dead, 0, -1 }, //d4
+	{ "SpecialEffect", fx_special_effect, 0, -1 },//c4
+	{ "StopAllAction", fx_stop_all_action, EFFECT_NO_ACTOR, -1 }, //cf
+	{ "TintScreen", fx_tint_screen, EFFECT_NO_ACTOR, -1 }, //c3
+	{ "TransferHP", fx_transfer_hp, EFFECT_DICED, -1 }, //c0
+	{ NULL, NULL, 0, 0 },
 };
 
 void RegisterTormentOpcodes()
 {
-	core->RegisterOpcodes( sizeof( effectnames ) / sizeof( EffectRef ) - 1, effectnames );
+	core->RegisterOpcodes( sizeof( effectnames ) / sizeof( EffectDesc ) - 1, effectnames );
 }
 
 //retreat_from (works only in PST) - forces target to run away/walk away from Owner
@@ -144,6 +150,15 @@ int fx_play_bam_blended (Scriptable* Owner, Actor* target, Effect* fx)
 	bool playonce;
 
 	if (0) printf( "fx_play_bam_blended (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (!Owner)
+		Owner = target;
+	if (!Owner)
+		return FX_NOT_APPLIED;
+	//delay effect
+	Map *area = Owner->GetCurrentArea();
+	if (!area)
+		return FX_APPLIED;
+
 	//play once set to true
 	//check tearring.itm (0xbb effect)
 	ScriptedAnimation *sca = gamedata->GetScriptedAnimation(fx->Resource, true);
@@ -169,20 +184,25 @@ int fx_play_bam_blended (Scriptable* Owner, Actor* target, Effect* fx)
 	} else {
 		playonce=false;
 	}
-	if (fx->Parameter2&1) {
-		//four cycles, duration is in millisecond
-		sca->SetDefaultDuration(sca->GetSequenceDuration(4000));
-	} else {
-		if (playonce) {
+	if (playonce) {
 			sca->PlayOnce();
+	} else {
+		if (fx->Parameter2&1) {
+			//four cycles, duration is in millisecond
+			sca->SetDefaultDuration(sca->GetSequenceDuration(AI_UPDATE_TIME));		
 		} else {
 			sca->SetDefaultDuration(fx->Duration-core->GetGame()->Ticks);
 		}
 	}
+	//convert it to an area VVC
+	if (!target) {
+		fx->Parameter2|=2;
+	}
+
 	if (fx->Parameter2&2) {
 		sca->XPos+=fx->PosX;
 		sca->YPos+=fx->PosY;
-		Owner->GetCurrentArea()->AddVVCell(sca);
+		area->AddVVCell(sca);
 	} else {
 		ScriptedAnimation *twin = sca->DetachTwin();
 		if (twin) {
@@ -207,6 +227,15 @@ int fx_play_bam_not_blended (Scriptable* Owner, Actor* target, Effect* fx)
 	bool doublehint;
 
 	if (0) printf( "fx_play_bam_not_blended (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (!Owner)
+		Owner = target;
+	if (!Owner)
+		return FX_NOT_APPLIED;
+
+	Map *area = Owner->GetCurrentArea();
+	if (!area)
+		return FX_APPLIED;
+
 	//play once set to true
 	//check tearring.itm (0xbb effect)
 	if ((fx->Parameter2&0x30000)==0x30000) {
@@ -243,23 +272,27 @@ int fx_play_bam_not_blended (Scriptable* Owner, Actor* target, Effect* fx)
 			sca->AlterPalette(rgb);
 		}
 	}
-	if (fx->TimingMode==FX_DURATION_INSTANT_PERMANENT) {
-		playonce=true;
-	} else {
+	if (fx->TimingMode==FX_DURATION_INSTANT_LIMITED) {
 		playonce=false;
+	} else {
+		playonce=true;
 	}
 	switch (fx->Parameter2&0x30000) {
 	case 0x20000://foreground
 		sca->ZPos+=9999;
+		sca->YPos+=9999;
 		break;
 	case 0x30000: //both
 		sca->ZPos+=9999;
+		sca->YPos+=9999;
 		if (sca->twin) {
 			sca->twin->ZPos-=9999;
+			sca->twin->YPos-=9999;
 		}
 		break;
 	default: //background
 		sca->ZPos-=9999;
+		sca->YPos-=9999;
 		break;
 	}
 	if (playonce) {
@@ -268,7 +301,8 @@ int fx_play_bam_not_blended (Scriptable* Owner, Actor* target, Effect* fx)
 		sca->SetDefaultDuration(fx->Duration-core->GetGame()->Ticks);
 	}
 	ScriptedAnimation *twin = sca->DetachTwin();
-	if (fx->Parameter2&4096) {
+
+	if (target && (fx->Parameter2&4096)) {
 		if (twin) {
 			target->AddVVCell(twin);
 		}
@@ -288,9 +322,9 @@ int fx_play_bam_not_blended (Scriptable* Owner, Actor* target, Effect* fx)
 		if (twin) {
 			twin->XPos+=fx->PosX-x;
 			twin->YPos+=fx->PosY+twin->ZPos-y;
-			Owner->GetCurrentArea()->AddVVCell(twin);
+			area->AddVVCell(twin);
 		}
-		Owner->GetCurrentArea()->AddVVCell(sca);
+		area->AddVVCell(sca);
 	}
 	return FX_NOT_APPLIED;
 }
@@ -303,7 +337,7 @@ int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx)
 		return FX_NOT_APPLIED;
 	}
 
-	Actor *owner = (Actor *) Owner;
+	Actor *owner = GetCasterObject();
 
 	if (owner==target) {
 		return FX_NOT_APPLIED;
@@ -313,6 +347,12 @@ int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx)
 	Actor *donor;
 	int a,b;
 
+	//handle variable level hp drain (for blood bridge)
+	if (fx->IsVariable) {
+		fx->Parameter1+=fx->CasterLevel;
+		fx->IsVariable=0;
+	}
+
 	switch(fx->Parameter2) {
 		case 3:
 		case 0: receiver = target; donor = owner; break;
@@ -327,8 +367,14 @@ int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx)
 		default:
 			return FX_NOT_APPLIED;
 	}
-	int damage = donor->Damage(fx->Parameter1, fx->Parameter2, owner);
-	receiver->SetBase( IE_HITPOINTS, BASE_GET( IE_HITPOINTS ) + ( damage ) );
+	int damage = receiver->GetStat(IE_MAXHITPOINTS)-receiver->GetStat(IE_HITPOINTS);
+	if (damage>(signed) fx->Parameter1) {
+		damage=(signed) fx->Parameter1;
+	}
+	if (damage) {
+		damage = donor->Damage(damage, fx->Parameter2, owner);
+		receiver->NewBase( IE_HITPOINTS, damage, MOD_ADDITIVE );
+	}
 	return FX_NOT_APPLIED;
 }
 
@@ -338,29 +384,105 @@ int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx)
 int fx_flash_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
 	if (0) printf( "fx_flash_screen (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
-	core->GetVideoDriver()->SetFadeColor(((char *) &fx->Parameter1)[0],((char *) &fx->Parameter1)[1],((char *) &fx->Parameter1)[2]);
-	core->timer->SetFadeFromColor(1);
-	core->timer->SetFadeToColor(1);
+	core->GetVideoDriver()->SetFadeColor(((unsigned char *) &fx->Parameter1)[0],((unsigned char *) &fx->Parameter1)[1],((unsigned char *) &fx->Parameter1)[2]);
+	//this needs to be at least 2 for any effect
+	core->timer->SetFadeFromColor(2);
 	return FX_NOT_APPLIED;
 }
 
 //0xc3 fx_tint_screen
+//FIXME: implement bit4 which would mean duration
 int fx_tint_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
 	if (0) printf( "fx_tint_screen (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
-	core->timer->SetFadeFromColor(10);
-	core->timer->SetFadeToColor(10);
+	int fromTime = fx->DiceSides;
+	int toTime = fx->DiceSides;
+	switch(fx->Parameter2&6) {
+		case 0: toTime = 0; break;
+		case 2: fromTime = 0; break;
+	}
+	core->timer->SetFadeFromColor(fromTime);
+	core->timer->SetFadeToColor(toTime);
 	return FX_NOT_APPLIED;
 }
 
 //0xc4 fx_special_effect
-int fx_special_effect (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
+//it is a mystery, why they needed to make this effect
+int fx_special_effect (Scriptable* Owner, Actor* target, Effect* fx)
 {
 	if (0) printf( "fx_special_effect (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	//param2 determines the effect's behaviour
+	//0 - adder's kiss projectile (0xcd)
+	//  adds play bam and damage opcodes to the projectile
+	//1 - ball lightning projectile (0xe0)
+	//  adds a play bam opcode to the projectile
+	//2 - raise dead projectile - the projectile itself has the effect (why is it so complicated)
 
+	//TODO: create the spells
+	switch(fx->Parameter2) {
+		case 0:
+			strnuprcpy(fx->Resource,"adder",8);
+			break;
+		case 1:
+			strnuprcpy(fx->Resource,"ball",8);
+			break;
+		case 2:
+			strnuprcpy(fx->Resource,"rdead",8);
+			break;
+	}
+	Owner->CastSpell(fx->Resource, target, false);
+	Owner->CastSpellEnd(fx->CasterLevel);
 	return FX_NOT_APPLIED;
 }
-//0xc5-c8 fx_unknown
+//0xc5 fx_multiple_vvc
+//this is a gemrb specific opcode to support the rune of torment projectile
+//it plays multiple vvc's with a given delay and duration
+int fx_multiple_vvc (Scriptable* Owner, Actor* /*target*/, Effect* fx)
+{
+	if (0) printf( "fx_multiple_vvc (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+
+	Map *area = Owner->GetCurrentArea();
+	if (!area)
+		return FX_NOT_APPLIED;
+
+	AutoTable tab(fx->Resource);
+	if (!tab)
+		return FX_NOT_APPLIED;
+
+	int rows = tab->GetRowCount();
+	while(rows--) {
+		Point offset;
+		int delay, duration;
+
+		offset.x=atoi(tab->QueryField(rows,0));
+		offset.y=atoi(tab->QueryField(rows,1));
+		delay = atoi(tab->QueryField(rows,3));
+		duration = atoi(tab->QueryField(rows,4));
+	 	ScriptedAnimation *sca = gamedata->GetScriptedAnimation(tab->QueryField(rows,2), true);
+		if (!sca) continue;
+		sca->SetBlend();
+		sca->SetDelay(AI_UPDATE_TIME*delay);
+		sca->SetDefaultDuration(AI_UPDATE_TIME*duration);
+		sca->XPos+=fx->PosX+offset.x;
+		sca->YPos+=fx->PosY+offset.y;
+		area->AddVVCell(sca);
+	}
+	return FX_NOT_APPLIED;
+}
+
+//0xc6 ChangeBackground
+//GemRB specific, to support BMP area background changes (desert hell projectile)
+int fx_change_background (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
+{
+	if (0) printf( "fx_change_background (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	Map *map = core->GetGame()->GetCurrentArea();
+	if (map) {
+		map->SetBackground(fx->Resource, fx->Duration);
+	}
+	return FX_NOT_APPLIED;
+}
+
+//0xc7-c8 fx_unknown
 //0xc9 fx_overlay
 int fx_overlay (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
@@ -372,7 +494,7 @@ int fx_overlay (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 //0xca fx_unknown
 
 //0x82 fx_bless
-//static EffectRef fx_glow_ref ={"Color:PulseRGBGlobal",NULL,-1};
+//static EffectRef fx_glow_ref = { "Color:PulseRGBGlobal", -1 };
 //pst bless effect spawns a color glow automatically
 //but i would rather use the IWD2 method
 int fx_bless (Scriptable* /*Owner*/, Actor* target, Effect* fx)
@@ -418,8 +540,8 @@ int fx_curse (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 }
 
 //0xcc fx_prayer
-static EffectRef fx_curse_ref={"Curse",NULL,-1};
-static EffectRef fx_bless_ref={"Bless",NULL,-1};
+static EffectRef fx_curse_ref = { "Curse", -1 };
+static EffectRef fx_bless_ref = { "Bless", -1 };
 
 int fx_prayer (Scriptable* Owner, Actor* target, Effect* fx)
 {
@@ -524,7 +646,7 @@ int fx_hostile_image (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 }
 
 //0xd2 fx_detect_evil
-static EffectRef fx_single_color_pulse_ref={"Color:BriefRGB",NULL,-1};
+static EffectRef fx_single_color_pulse_ref = { "Color:BriefRGB", -1 };
 
 int fx_detect_evil (Scriptable* Owner, Actor* target, Effect* fx)
 {
@@ -582,6 +704,20 @@ int fx_jumble_curse (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 	return FX_APPLIED;
 }
 
+//0xd4 fx_speak_with_dead
+//This opcode is directly employed by the speak with dead projectile in the original engine
+//In GemRB it is used in a custom spell
+int fx_speak_with_dead (Scriptable* /*Owner*/, Actor* target, Effect* fx)
+{
+	if (0) printf("fx_speak_with_dead (%2d)\n", fx->Opcode );
+	if (!STATE_GET( STATE_DEAD) ) {
+		return FX_NOT_APPLIED;
+	}
+
+	//TODO: reverse engineer the original opcode
+	return FX_NOT_APPLIED;
+}
+
 #include "plugindef.h"
 
 GEMRB_PLUGIN(0x115A670, "Effect opcodes for the torment branch of the games")
